scriptname RealNamesSSEPatchRenameEffectScript extends ActiveMagicEffect

import Debug
import StringUtil
import jaxonzStringUtil
import Utility

RealNamesSSEPatchPlayerAlias Property scrNameGen Auto

int iClassIndex
int iRaceIndex

string sGenericNameSearch
string sGenericBossSearch

Event OnEffectStart(Actor akTarget, Actor akCaster)
	string sName = akTarget.GetDisplayName()
	if !scrNameGen.flstAlreadyEvaluated.HasForm(akTarget) && sName == akTarget.GetLeveledActorBase().GetName()
		AssignName(akTarget)
	else	;Special Cases for Random Encounters (which are "resistant" to renaming)
		sName = "," + sName + ","
		if Find(akTarget, "(FF") && Find(",Afflicted,Afflicted Refugee,Afflicted Remnant,Alik'r Warrior,Ancient Vampire,Apprentice Conjurer,Apprentice Fire Mage,Apprentice Ice Mage,Apprentice Necromancer,Apprentice Storm Mage,Arch Conjurer,Arch Cryomancer,Arch Cultist,Arch Electromancer,Arch Necromancer,Arch Pyromancer,Argonian,Ascendant Conjurer,Ascendant Cultist,Ascendant Necromancer,Assassin,Bandit,Bandit Chief,Bandit Highwayman,Bandit Marauder,Bandit Plunderer,Bandit Outlaw,Bandit Thug,Blooded Vampire,Bodyguard,Boethiah Cultist,Bouncer,Bounty Collector,Bounty Hunter,Breton,Challenger,Conjurer,Conjurer Adept,Courier,Cryomancer,Cultist,Cultist Adept,Dark Elf,Dawnguard,Dragon,Electromancer,Farmer,Fire Mage,Fire Mage Adept,Fire Wizard,Forsworn,Forsworn Briarheart,Forsworn Forager,Forsworn Looter,Forsworn Pillager,Forsworn Ravager,Forsworn Shaman,Frightened Woman,Fugitive,Giant,Hagraven,High Elf,Hunter,Ice Mage,Ice Mage Adept,Ice Wizard,Imperial,Imperial Courier,Imperial Soldier,Khajiit,Madman,Madwoman,Master Conjurer,Master Cultist,Master Necromancer,Master Vampire,Mercenary,Morag Tong Assassin,Necromage,Necromancer Adept,Nightlord Vampire,Nightmaster Vampire,Noble,Nord,Novice Conjurer,Novice Fire Mage,Novice Ice Mage,Novice Necromancer,Novice Storm Mage,Old Orc,Orc,Peddler,Pirate,Pirate Mage,Pyromancer,Reaver,Reaver Highwayman,Reaver Lord,Reaver Marauder,Reaver Plunderer,Reaver Outlaw,Reaver Thug,Redguard,Redguard Woman,Reveler,Scavenger,Stormcloak Courier,Stormcloak Soldier,Storm Mage,Storm Mage Adept,Storm Wizard,Student,Thalmor Justiciar,Thalmor Prisoner,Thief,Thug,Vampire,Vampire Assassin,Vampire Fledgling,Vampire Mistwalker,Vampire Nightstalker,Vampire's Thrall,Vigilant of Stendarr,Vigilant Vampire Hunter,Volkihar Vampire,Volkihar Master Vampire,Witch,Wizard,Wood Elf,", sName) != -1
			AssignName(akTarget, true)
		endif
	endif
EndEvent

;Hard import the name generator

Bool Function AssignName(ObjectReference objNPC, bool bForce = false)

;   This is the overhead name assignment function which gets
;   called by the PlayerAlias Events and determines which other
;   functions also need to be called.
;
;   Returns TRUE upon successful name assignment.

    string sName = objNPC.GetDisplayName()
;Trace("AssignName called on objNPC:" + objNPC + ", GetDisplayName:" + sName + ", GetType:" + objNPC.GetType())
    String sNewName
    String sNamesList
    
    if !(objNPC as Actor)
        return false
    elseIf !bForce && scrNameGen.flstAlreadyEvaluated.HasForm(objNPC) && (sName != (objNPC as Actor).GetLeveledActorBase().GetName() || GetException(sName))
;Trace("AssignName, objNPC:" + objNPC + " already evaluated")
        return false
    elseIf scrNameGen.bCorpseExclusion && (objNPC as Actor).IsDead() 
        return false
    else
        scrNameGen.flstAlreadyEvaluated.AddForm(objNPC)
    endIf

    ;Don't even attempt to rename NPCs in the Exception list, unless ForceRename function is used
    if !bForce && GetException(sName)
        return false
    endif

    ;string sRaceNameKey = GetRaceIniKey(objNPC as Actor)
    int iRaceGroupIndex = GetRaceGroupIndex(objNPC as Actor)
    
;Trace("objNPC:" + objNPC + ", GetDisplayName:" + sName + ", iRaceGroupIndex:" + iRaceGroupIndex)
    if (iRaceGroupIndex != -1) && (bForce || DetectGenericName(sName))
;Trace("iRaceIndex:" + iRaceIndex + ", Sex:" + iNPCSex + ", sRaceNameKey:" + sRaceNameKey + "=" + sNamesList)

        ;fetch an appropriate name
        sNewName = GenerateNameforActor (objNPC as Actor)

        ;assign the name
        if sNewName != ""
Trace("Real Names renaming " + objNPC + " from '" + sName + "' to '" + sNewName + "'")
            if scrNameGen.iGenericName == 0
                return objNPC.SetDisplayName(sNewName, true)
            elseIf scrNameGen.iGenericName == 1
                objNPC.SetDisplayName((objNPC as Actor).GetLeveledActorBase().GetName())
                return objNPC.SetDisplayName(sNewName + " [" + sName + "]", true)
            elseIf scrNameGen.iGenericName == 2
                objNPC.SetDisplayName((objNPC as Actor).GetLeveledActorBase().GetName())
                return objNPC.SetDisplayName(sNewName + "\n[" + sName + "]", true)
            elseIf scrNameGen.iGenericName == 3
                objNPC.SetDisplayName((objNPC as Actor).GetLeveledActorBase().GetName())
                return objNPC.SetDisplayName(sNewName + ", " + sName, true)
            elseIf scrNameGen.iGenericName == 4
                objNPC.SetDisplayName((objNPC as Actor).GetLeveledActorBase().GetName())
                return objNPC.SetDisplayName(sNewName + "\n" + sName, true)
            endIf
        endIf
    elseif iRaceGroupIndex != -1
        scrNameGen.flstAlreadyEvaluated.RemoveAddedForm(objNPC) ; Allow RealNames to try again, should changes be made to SearchLists or the NPC's Display Name
    endIf
;Trace("AssignName returning false, objNPC:" + objNPC + ", Old Name:" + sName)
    return false
endFunction


String Function GenerateNameforActor (Actor akActor)

;   This function gets called after the AssignName function
;   determines that the detected/targeted NPC is a valid
;   candidate for renaming. This is the function which
;   actually calls the NameGenerator Functions to pull names
;   from the Name Strings and construct new names.

;trace("JaxonzNameGenPlayerAlias::GenerateNameforActor, akActor:" + akActor + ", GetName():" + akActor.GetName() + ", akActor.GetActorBase().GetRace():" + akActor.GetActorBase().GetRace() + ", akActor.GetRace():" + akActor.GetRace() + ", akActor.GetActorBase().GetSex():" + akActor.GetActorBase().GetSex())
    ActorBase kActorBase = akActor.GetLeveledActorBase()
    Race racNPC = kActorBase.GetRace()
    Int iNPCSex = kActorBase.GetSex()    ;0=male, 1=female
    Class clasNPC = kActorBase.GetClass()

    bool bNPCisBoss = DetectGenericBoss(kActorBase.GetName())

    iClassIndex = 0
    bool bClassIdentified = false
    While (iClassIndex < scrNameGen.flstClassGroups.GetSize()) && !bClassIdentified
        if (scrNameGen.flstClassGroups.GetAt(iClassIndex) as FormList).HasForm(clasNPC)
            bClassIdentified = True
        else
            iClassIndex += 1
        endif
    endwhile

    iRaceIndex = 0
    Bool bRaceIdentified = false
    While (iRaceIndex < scrNameGen.flstRaceGroups.GetSize()) && !bRaceIdentified
        if (scrNameGen.flstRaceGroups.GetAt(iRaceIndex) as FormList).HasForm(racNPC)
            bRaceIdentified = True
        Else
            iRaceIndex += 1
        EndIf
    EndWhile

    string sActor = akActor
    ;Special treatment of Torturer to give him an Imperial name
    if Find(sActor, "000B79BA") >= 0 ;/Torturer RefID/;
        iRaceIndex = 5
    endif

    if iRaceIndex == 12 ;/Dragon/;
        if scrNameGen.bDragonExclusion
            return ""
        else
            return GenerateMonsterName(iRaceIndex, bNPCisBoss)
        endIf
    elseif iRaceIndex == 13 ;/Dragon Priest/;
        if scrNameGen.bDragonPriestExclusion
            return ""
        else
            return GenerateMonsterName(iRaceIndex, bNPCisBoss)
        endIf
    elseif iRaceIndex == 14 ;/Giant/;
        if scrNameGen.bGiantExclusion
            return ""
        else
            return GenerateMonsterName(iRaceIndex, bNPCisBoss)
        endIf
    elseif iRaceIndex == 15 ;/Hagraven/;
        if scrNameGen.bHagravenExclusion
            return ""
        else
            return GenerateMonsterName(iRaceIndex, bNPCisBoss)
        endIf
    elseif bRaceIdentified
        if iRaceIndex == 10 && scrNameGen.bDraugrExclusion
            return ""
        elseif iRaceIndex == 11 && scrNameGen.bDremoraExclusion
            return ""
        endif

        ;Special cases for "anonymous" NPCs with contextual names
        if Find(sActor, "00039FE4") >= 0 ;Arvel the Swift's Crew, Male watch
            return "Harknir the Stout"
        elseif Find(sActor, "00039FE2") >= 0 ;Arvel the Swift's Crew, Female watch
            return "Soling Dead-Eye"
        elseif Find(sActor, "00039FE1") >= 0 ;Arvel the Swift's Crew, Puzzle-Trap victim
            return "Ill-Fated Bjorn"
        elseif Find(sActor, "000977FC") >= 0 ;Agent Sanyon, Thalmor Soldier at Talos Shrine near Helgen
            return "Agent Sanyon"
        elseif Find(sActor, "000812FD") >= 0 ;Rodulf, Bandit outside White River Watch
            return "Rodulf Red-Nose"
        elseif Find(akActor.GetBaseObject(), "00090C87") >= 0 && Find(sActor, "000FE54B") == -1 ;Dead Bouncer, Random Encounter
            return "Adril the Handler"
        elseif Find(sActor, "00F496F") >= 0 ;Savos Aren's Companion in Labyrinthian
            return "Hafnar Ice-Fist"
        elseif Find(sActor, "00F4927") >= 0 ;Savos Aren's Companion in Labyrinthan
            return "Atmah"
        endif

        return GenerateName(iRaceIndex, iClassIndex, iNPCSex as bool, bNPCisBoss)
    Else
        return ""
    EndIf

EndFunction


Bool Function DetectGenericName(string ActorName)

;   This function gets called as part of the process to determine
;   an NPCs candidacy for renaming. It compares the base objects
;   name with the Generic Search Terms String. Specifically, it
;   crawls through the Generic Search Terms and checks to see if
;   any of the listed terms is contained in the base name.

;Trace("DetectGenericName, ActorName:" + ActorName )
    ;exit if already renamed
    if Find(ActorName, "[") > 0 || Find(ActorNAme, "\n") > 0
        return false
    EndIf

;Trace("DetectGenericName, before get sGenericNameSearch from Script, ActorName:" + ActorName )

    if sGenericNameSearch == ""
        ;Original coding, for reference
        ;sGenericNameSearch = FetchSetting("General", "GenericNameSearchTerms")
        sGenericNameSearch = scrNameGen.GenericNameSearchTerms
;Trace("sGenericNameSearch:" + sGenericNameSearch)
    EndIf

;Trace("DetectGenericName, before sSearchTerm loop, ActorName:" + ActorName )
    string sSearchTerm

    int iOffset = 0
    bool bEndOfString = false
    While (iOffset < GetLength(sGenericNameSearch)) && !bEndOfString
        iOffset = Find(sGenericNameSearch, ",", iOffset) + 1
;Trace("iOffset:" + iOffset)
        if iOffset == 0 ;nothing found
;           iOffset = GetLength(sGenericNameSearch) + 10
            bEndOfString = true
        EndIf
        sSearchTerm = Substring(sGenericNameSearch, iOffset, Find(sGenericNameSearch, ",", iOffset) - iOffset)
;Trace("ActorName:" + ActorName + ", sSearchTerm:" + sSearchTerm + ", iOffset:" + iOffset)
        if Find(ActorName, sSearchTerm) != -1
;Trace("DetectGenericName found match, '" + sSearchTerm + "' in '" + ActorName + "'")
            return true
        EndIf
    EndWhile
;Trace("DetectGenericName, none found, ActorName:" + ActorName + ", sGenericNameSearch:" + sGenericNameSearch)
    return false
EndFunction


bool Function DetectGenericBoss(string ActorName)

;   One of my new functions. Almost the same as DetectGenericName
;   except that this function compares the base name to the
;   Generic Boss Search Terms. It is used as part of the logic
;   to determine whether a Boss Moniker should be assigned.

;Trace("DetectGenericBoss, ActorName:" + ActorName )

;Trace("DetectGenericBoss, before get sGenericBossSearch from Script, ActorName:" + ActorName )

    if sGenericBossSearch == ""
        sGenericBossSearch = scrNameGen.GenericBossSearchTerms
;Trace("sGenericBossSearch:" + sGenericBossSearch)
    EndIf

;Trace("DetectGenericBoss, before sSearchTerm loop, ActorName:" + ActorName )
    string sSearchTerm

    int iOffset = 0
    bool bEndOfString = false
    While (iOffset < GetLength(sGenericBossSearch)) && !bEndOfString
        iOffset = Find(sGenericBossSearch, ",", iOffset) + 1
;Trace("iOffset:" + iOffset)
        if iOffset == 0 ;nothing found
;           iOffset = GetLength(sGenericBossSearch) + 10
            bEndOfString = true
        EndIf
        sSearchTerm = Substring(sGenericBossSearch, iOffset, Find(sGenericBossSearch, ",", iOffset) - iOffset)
;Trace("ActorName:" + ActorName + ", sSearchTerm:" + sSearchTerm + ", iOffset:" + iOffset)
        if Find(ActorName, sSearchTerm) != -1
;Trace("DetectGenericBoss found match, '" + sSearchTerm + "' in '" + ActorName + "'")
            return true
        EndIf
    EndWhile
;Trace("DetectGenericBoss, none found, ActorName:" + ActorName + ", sGenericBossSearch:" + sGenericBossSearch)
    return false
EndFunction


bool Function GetException(string ActorName)

;   Another of my functions. This one functions as a counter to
;   DetectGenericName, as a failsafe to prevent pre-Named NPCs
;   whose names happen to contain terms from the Generic Name
;   Search Terms (e.g. Ulfric Stormcloak) from being renamed.
;
;   Instead of crawling through the list searching the ActorName
;   for each term, this function searches the Exceptions List(s)
;   for each term in the ActorName. This reverse comparison helps
;   to prevent any false positives from occurring.

    int iOffset = 0
    string sFullName = "," + ActorName + ","
    if Find(scrNameGen.Exceptions1, sFullName) > -1 || Find(scrNameGen.Exceptions2, sFullName) > -1 || Find(scrNameGen.Exceptions3, sFullName) > -1 || Find(scrNameGen.ExceptionsINPC, sFullName) > -1 || Find(scrNameGen.PlayerExceptions, sFullName) > -1
        return true
    else
        iOffset = Find(ActorName, " ")
        if iOffset > 0
            string sFirstWord = "," + Substring(ActorName, 0, iOffset) + ","
            if Find(scrNameGen.Exceptions1, sFirstWord) > -1 || Find(scrNameGen.Exceptions2, sFirstWord) > -1 || Find(scrNameGen.Exceptions3, sFirstWord) > -1 || Find(scrNameGen.ExceptionsINPC, sFirstWord) > -1 || Find(scrNameGen.PlayerExceptions, sFirstWord) > -1
                return true
            else
                string sSecondWord
                int iOffset2 = Find(ActorName, " ", iOffset + 1)
                if iOffset2 > 0
                    sSecondWord = "," + Substring(ActorName, iOffset + 1, iOffset2 - iOffset - 1) + ","
                else
                    sSecondWord = "," + Substring(ActorName, iOffset + 1) + ","
                endif
                if Find(scrNameGen.Exceptions1, sSecondWord) > -1 || Find(scrNameGen.Exceptions2, sSecondWord) > -1 || Find(scrNameGen.Exceptions3, sSecondWord) > -1 || Find(scrNameGen.ExceptionsINPC, sSecondWord) > -1 || Find(scrNameGen.PlayerExceptions, sSecondWord) > -1
                    return true
                endif
            endif
        endif
    endif

    ;if no other valid return
    return false

endfunction

Int Function GetRaceGroupIndex(Actor akActor)

;   This function is called by AssignName as part of candidacy
;   logic. It is only used as a check to ensure that the querried
;   Actor belongs to one of the pre-selected races before moving
;   on to name generation.
;
;   If you wanted to, you could remove this function from the
;   logic chain, and the NameGenerator would be able to rename
;   any NPC, regardless of Race.

    Race racNPC = akActor.GetLeveledActorBase().GetRace()
    Int iNPCSex = akActor.GetLeveledActorBase().GetSex()    ;0=male, 1=female

    iRaceIndex = 0
    Bool bRaceIdentified = false
    while (iRaceIndex < scrNameGen.flstRaceGroups.GetSize()) && !bRaceIdentified
        if (scrNameGen.flstRaceGroups.GetAt(iRaceIndex) as FormList).HasForm(racNPC)
            bRaceIdentified = True
        else
            iRaceIndex += 1
        endIf
    endWhile

    if !bRaceIdentified
        return -1
    endIf
    
    ;make sure monster names index selected if appropriate
    if iRaceIndex >= 12
        iNPCSex = 1
    endIf
    
    return iRaceIndex + (12*iNPCSex)
EndFunction

String Function GenerateName (int iRace, int iClass = 3, bool bSex, bool bBoss = false) ;0=male, 1=female

;   This function is much, much longer than the original script
;   merely because all of the checks which the .INI method was
;   able to perform with string building (e.g. "race" + "sex" + ...)
;   now needs to be performed by scripted logic, and there are 12
;   races (including Draugr and Dremora) which need to be checked.
;
;   Ultimately, what the function does is after checking the NPC's
;   race and all proscribed probabilites, a name is construceted
;   by adding substrings pulled from the Name Strings. If all
;   probabilities hit, the end result is FirstName + LastName + Epithet.

    ;/String[] sRaceNames = new string[10]  ;array of race names used for reference
    sRaceNames[0] = "Altmer"
    sRaceNames[1] = "Argonian"
    sRaceNames[2] = "Bosmer"
    sRaceNames[3] = "Breton"
    sRaceNames[4] = "Dunmer"
    sRaceNames[5] = "Imperial"
    sRaceNames[6] = "Kajiit"
    sRaceNames[7] = "Nord"
    sRaceNames[8] = "Orc"
    sRaceNames[9] = "Redguard"
    sRaceNames[10] = "Draugr"
    sRaceNames[11] = "Dremora"/;

    
    ;check for Ashlander
    bool bAshlander = false
    if iRace == 4 ;/Dunmer/;
        if scrNameGen.DunmerAshlanderChance >= RandomInt(1,100)
            bAshlander = true
        endif
    endif

    ;Check for Off-Race Name Chance
    string oRaceList = ""
    int oRace = -1
    if iRace == 0  && scrNameGen.AltmerOffRaceNameChance >= Utility.RandomInt(1,100)
        if scrNameGen.AltmerArgonianName
            oRaceList += "1"
        endif
        if scrNameGen.AltmerBosmerName
            oRaceList += "2"
        endif
        if scrNameGen.AltmerBretonName
            oRaceList += "3"
        endif
        if scrNameGen.AltmerDunmerName
            oRaceList += "4"
        endif
        if scrNameGen.AltmerImperialName
            oRaceList += "5"
        endif
        if scrNameGen.AltmerKhajiitName
            oRaceList += "6"
        endif
        if scrNameGen.AltmerNordName
            oRaceList += "7"
        endif
        if scrNameGen.AltmerOrcName
            oRaceList += "8"
        endif
        if scrNameGen.AltmerRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 1  && scrNameGen.ArgonianOffRaceNameChance >= Utility.RandomInt(1,100)
        if scrNameGen.ArgonianAltmerName
            oRaceList += "0"
        endif
        if scrNameGen.ArgonianBosmerName
            oRaceList += "2"
        endif
        if scrNameGen.ArgonianBretonName
            oRaceList += "3"
        endif
        if scrNameGen.ArgonianDunmerName
            oRaceList += "4"
        endif
        if scrNameGen.ArgonianImperialName
            oRaceList += "5"
        endif
        if scrNameGen.ArgonianKhajiitName
            oRaceList += "6"
        endif
        if scrNameGen.ArgonianNordName
            oRaceList += "7"
        endif
        if scrNameGen.ArgonianOrcName
            oRaceList += "8"
        endif
        if scrNameGen.ArgonianRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 2  && scrNameGen.BosmerOffRaceNameChance >= Utility.RandomInt(1,100)
        if scrNameGen.BosmerAltmerName
            oRaceList += "0"
        endif
        if scrNameGen.BosmerArgonianName
            oRaceList += "1"
        endif
        if scrNameGen.BosmerBretonName
            oRaceList += "3"
        endif
        if scrNameGen.BosmerDunmerName
            oRaceList += "4"
        endif
        if scrNameGen.BosmerImperialName
            oRaceList += "5"
        endif
        if scrNameGen.BosmerKhajiitName
            oRaceList += "6"
        endif
        if scrNameGen.BosmerNordName
            oRaceList += "7"
        endif
        if scrNameGen.BosmerOrcName
            oRaceList += "8"
        endif
        if scrNameGen.BosmerRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 3  && scrNameGen.BretonOffRaceNameChance >= Utility.RandomInt(1,100)
        if scrNameGen.BretonAltmerName
            oRaceList += "0"
        endif
        if scrNameGen.BretonArgonianName
            oRaceList += "1"
        endif
        if scrNameGen.BretonBosmerName
            oRaceList += "2"
        endif
        if scrNameGen.BretonDunmerName
            oRaceList += "4"
        endif
        if scrNameGen.BretonImperialName
            oRaceList += "5"
        endif
        if scrNameGen.BretonKhajiitName
            oRaceList += "6"
        endif
        if scrNameGen.BretonNordName
            oRaceList += "7"
        endif
        if scrNameGen.BretonOrcName
            oRaceList += "8"
        endif
        if scrNameGen.BretonRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 4  && scrNameGen.DunmerOffRaceNameChance >= Utility.RandomInt(1,100)
        if scrNameGen.DunmerAltmerName
            oRaceList += "0"
        endif
        if scrNameGen.DunmerArgonianName
            oRaceList += "1"
        endif
        if scrNameGen.DunmerBosmerName
            oRaceList += "2"
        endif
        if scrNameGen.DunmerBretonName
            oRaceList += "3"
        endif
        if scrNameGen.DunmerImperialName
            oRaceList += "5"
        endif
        if scrNameGen.DunmerKhajiitName
            oRaceList += "6"
        endif
        if scrNameGen.DunmerNordName
            oRaceList += "7"
        endif
        if scrNameGen.DunmerOrcName
            oRaceList += "8"
        endif
        if scrNameGen.DunmerRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 5  && scrNameGen.ImperialOffRaceNameChance >= Utility.RandomInt(1,100)
        if scrNameGen.ImperialAltmerName
            oRaceList += "0"
        endif
        if scrNameGen.ImperialArgonianName
            oRaceList += "1"
        endif
        if scrNameGen.ImperialBosmerName
            oRaceList += "2"
        endif
        if scrNameGen.ImperialBretonName
            oRaceList += "3"
        endif
        if scrNameGen.ImperialDunmerName
            oRaceList += "4"
        endif
        if scrNameGen.ImperialKhajiitName
            oRaceList += "6"
        endif
        if scrNameGen.ImperialNordName
            oRaceList += "7"
        endif
        if scrNameGen.ImperialOrcName
            oRaceList += "8"
        endif
        if scrNameGen.ImperialRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 6  && scrNameGen.KhajiitOffRaceNameChance >= Utility.RandomInt(1,100)
        if scrNameGen.KhajiitAltmerName
            oRaceList += "0"
        endif
        if scrNameGen.KhajiitArgonianName
            oRaceList += "1"
        endif
        if scrNameGen.KhajiitBosmerName
            oRaceList += "2"
        endif
        if scrNameGen.KhajiitBretonName
            oRaceList += "3"
        endif
        if scrNameGen.KhajiitDunmerName
            oRaceList += "4"
        endif
        if scrNameGen.KhajiitImperialName
            oRaceList += "5"
        endif
        if scrNameGen.KhajiitNordName
            oRaceList += "7"
        endif
        if scrNameGen.KhajiitOrcName
            oRaceList += "8"
        endif
        if scrNameGen.KhajiitRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 7  && scrNameGen.NordOffRaceNameChance >= Utility.RandomInt(1,100)
        if scrNameGen.NordAltmerName
            oRaceList += "0"
        endif
        if scrNameGen.NordArgonianName
            oRaceList += "1"
        endif
        if scrNameGen.NordBosmerName
            oRaceList += "2"
        endif
        if scrNameGen.NordBretonName
            oRaceList += "3"
        endif
        if scrNameGen.NordDunmerName
            oRaceList += "4"
        endif
        if scrNameGen.NordImperialName
            oRaceList += "5"
        endif
        if scrNameGen.NordKhajiitName
            oRaceList += "6"
        endif
        if scrNameGen.NordOrcName
            oRaceList += "8"
        endif
        if scrNameGen.NordRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 8  && scrNameGen.OrcOffRaceNameChance >= Utility.RandomInt(1,100)
        if scrNameGen.OrcAltmerName
            oRaceList += "0"
        endif
        if scrNameGen.OrcArgonianName
            oRaceList += "1"
        endif
        if scrNameGen.OrcBosmerName
            oRaceList += "2"
        endif
        if scrNameGen.OrcBretonName
            oRaceList += "3"
        endif
        if scrNameGen.OrcDunmerName
            oRaceList += "4"
        endif
        if scrNameGen.OrcImperialName
            oRaceList += "5"
        endif
        if scrNameGen.OrcKhajiitName
            oRaceList += "6"
        endif
        if scrNameGen.OrcNordName
            oRaceList += "7"
        endif
        if scrNameGen.OrcRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 9  && scrNameGen.RedguardOffRaceNameChance >= Utility.RandomInt(1,100)
        if scrNameGen.RedguardAltmerName
            oRaceList += "0"
        endif
        if scrNameGen.RedguardArgonianName
            oRaceList += "1"
        endif
        if scrNameGen.RedguardBosmerName
            oRaceList += "2"
        endif
        if scrNameGen.RedguardBretonName
            oRaceList += "3"
        endif
        if scrNameGen.RedguardDunmerName
            oRaceList += "4"
        endif
        if scrNameGen.RedguardImperialName
            oRaceList += "5"
        endif
        if scrNameGen.RedguardKhajiitName
            oRaceList += "6"
        endif
        if scrNameGen.RedguardNordName
            oRaceList += "7"
        endif
        if scrNameGen.RedguardOrcName
            oRaceList += "8"
        endif
    endif

    if oRaceList != ""
        oRace = GetNthChar(oRaceList, RandomInt(0, GetLength(oRaceList) - 1)) as int
    endif

    ;Check if iRace has Family Names
    bool biNoTribe = false
    if (iRace == 0 && scrNameGen.AltmerFamilyNames == "") || (iRace == 1 && scrNameGen.ArgonianFamilyNames == "") || (iRace == 2 && scrNameGen.BosmerFamilyNames == "") \
|| (iRace == 3 && scrNameGen.BretonFamilyNames == "") || (iRace == 4 && scrNameGen.DunmerFamilyNames == "") || (iRace == 5 && scrNameGen.ImperialFamilyNames == "") || \
(iRace == 6 && scrNameGen.KhajiitFamilyNames == "") || (iRace == 7 && scrNameGen.NordFamilyNames == "") || (iRace == 8 && scrNameGen.OrcFamilyNames == "") || (iRace == 9 && scrNameGen.RedguardFamilyNames == "")
        biNoTribe = true
    endif

    ;Check if oRace has Family Names
    bool boNoTribe = false
    if (oRace == 0 && scrNameGen.AltmerFamilyNames == "") || (oRace == 1 && scrNameGen.ArgonianFamilyNames == "") || (oRace == 2 && scrNameGen.BosmerFamilyNames == "") \
|| (oRace == 3 && scrNameGen.BretonFamilyNames == "") || (oRace == 4 && scrNameGen.DunmerFamilyNames == "") || (oRace == 5 && scrNameGen.ImperialFamilyNames == "") || \
(oRace == 6 && scrNameGen.KhajiitFamilyNames == "") || (oRace == 7 && scrNameGen.NordFamilyNames == "") || (oRace == 8 && scrNameGen.OrcFamilyNames == "") || (oRace == 9 && scrNameGen.RedguardFamilyNames == "")
        boNoTribe = true
    endif

    if bAshlander
        biNoTribe = false
        if scrNameGen.AshlanderFamilyNames == ""
            biNoTribe = true
        endif
    endif

    ;Check for Family Name Only Chance
    bool LastNameOnly = false
    if oRace != -1 && boNoTribe == false
        if oRace == 0  ;/Altmer/;
            if scrNameGen.AltmerFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 1  ;/Argonian/;
            if scrNameGen.ArgonianFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 2  ;/Bosmer/;
            if scrNameGen.BosmerFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 3  ;/Breton/;
            if scrNameGen.BretonFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 5  ;/Imperial/;
            if scrNameGen.ImperialFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 6  ;/Khajiit/;
            if scrNameGen.KhajiitFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 7  ;/Nord/;
            if scrNameGen.NordFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 8  ;/Orc/;
            if scrNameGen.OrcFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 9  ;/Redguard/;
            if scrNameGen.RedguardFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        endif
    endif
    if LastNameOnly == false && ((oRace != -1 && boNoTribe == false) || biNoTribe == false)
        if iRace == 0  ;/Altmer/;
            if scrNameGen.AltmerFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 1  ;/Argonian/;
            if scrNameGen.ArgonianFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 2  ;/Bosmer/;
            if scrNameGen.BosmerFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 3  ;/Breton/;
            if scrNameGen.BretonFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 4  ;/Dunmer/;
            if scrNameGen.DunmerFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 5  ;/Imperial/;
            if scrNameGen.ImperialFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 6  ;/Khajiit/;
            if scrNameGen.KhajiitFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 7  ;/Nord/;
            if scrNameGen.NordFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 8  ;/Orc/;
            if scrNameGen.OrcFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 9  ;/Redguard/;
            if scrNameGen.RedguardFamilyNameOnlyChance >= RandomInt(1,100)
                LastNameOnly = true
            endif
        endif
    endif


    int aRace
    string sFirstName
    if LastNameOnly == false
        if oRace != -1
            if boNoTribe
                aRace = oRace
            elseif RandomInt(1,2) == 1
                aRace = oRace
            else
                aRace = iRace
            endif
        else
            aRace = iRace
        endif
        if aRace == 0  ;/Altmer/;
            if !bSex
                sFirstName = scrNameGen.AltmerMaleFirstNames
            elseif bSex
                sFirstName = scrNameGen.AltmerFemaleFirstNames
            endif
        elseif aRace == 1  ;/Argonian/;
            if !bSex
                sFirstName = scrNameGen.ArgonianMaleFirstNames
            elseif bSex
                sFirstName = scrNameGen.ArgonianFemaleFirstNames
            endif
        elseif aRace == 2  ;/Bosmer/;
            if !bSex
                sFirstName = scrNameGen.BosmerMaleFirstNames
            elseif bSex
                sFirstName = scrNameGen.BosmerFemaleFirstNames
            endif
        elseif aRace == 3  ;/Breton/;
            if !bSex
                sFirstName = scrNameGen.BretonMaleFirstNames
            elseif bSex
                sFirstName = scrNameGen.BretonFemaleFirstNames
            endif
        elseif aRace == 4  ;/Dunmer/;
            if !bAshlander
                if !bSex
                    sFirstName = scrNameGen.DunmerMaleFirstNames
                elseif bSex
                    sFirstName = scrNameGen.DunmerFemaleFirstNames
                endif
            elseif bAshlander
                if !bSex
                    sFirstName = scrNameGen.AshlanderMaleFirstNames
                elseif bSex
                    sFirstName = scrNameGen.AshlanderFemaleFirstNames
                endif
            endif
        elseif aRace == 5  ;/Imperial/;
            if !bSex
                sFirstName = scrNameGen.ImperialMaleFirstNames
            elseif bSex
                sFirstName = scrNameGen.ImperialFemaleFirstNames
            endif
        elseif aRace == 6  ;/Khajiit/;
            if !bSex
                sFirstName = scrNameGen.KhajiitMaleFirstNames
            elseif bSex
                sFirstName = scrNameGen.KhajiitFemaleFirstNames
            endif
        elseif aRace == 7 || aRace == 10 ;/Nord or Draugr/;
            if !bSex
                sFirstName = scrNameGen.NordMaleFirstNames
            elseif bSex
                sFirstName = scrNameGen.NordFemaleFirstNames
            endif
        elseif aRace == 8  ;/Orc/;
            if !bSex
                sFirstName = scrNameGen.OrcMaleFirstNames
            elseif bSex
                sFirstName = scrNameGen.OrcFemaleFirstNames
            endif
        elseif aRace == 9  ;/Redguard/;
            if !bSex
                sFirstName = scrNameGen.RedguardMaleFirstNames
            elseif bSex
                sFirstName = scrNameGen.RedguardFemaleFirstNames
            endif
        elseif aRace == 11 ;/Dremora/;
            if !bSex
                sFirstName = scrNameGen.DremoraMaleFirstNames
            elseif bSex
                sFirstName = scrNameGen.DremoraFemaleFirstNames
            endif
        endif

        sFirstName = RandomStringFromCSV(sFirstName)

	;special treatment for Khajiit honorifics
	if sFirstName != "" && (aRace == 6 || iRace == 6) ;/Khajiit/;
		string sHonorificPre = ""
		string sHonorificSuf = ""
		if !bSex
			if scrNameGen.KhajiitMalePrefixChance >= RandomInt(1,100)
				sHonorificPre = scrNameGen.KhajiitMalePrefices
			endif
			if scrNameGen.KhajiitMaleSuffixChance >= RandomInt(1,100)
				sHonorificSuf = scrNameGen.KhajiitMaleSuffices
			endif
		elseif bSex
			if scrNameGen.KhajiitFemalePrefixChance >= RandomInt(1,100)
				sHonorificPre = scrNameGen.KhajiitFemalePrefices
			endif
			if scrNameGen.KhajiitFemaleSuffixChance >= RandomInt(1,100)
				sHonorificSuf = scrNameGen.KhajiitFemaleSuffices
			endif
		endif
		if sHonorificPre != ""
			sHonorificPre = RandomStringFromCSV(sHonorificPre)
		endif
		if sHonorificSuf != ""
			sHonorificSuf = RandomStringFromCSV(sHonorificSuf)
			if sHonorificPre != "" && scrNameGen.KhajiitHonorificPreference == 2
				while Substring(sHonorificSuf, 0, 1) == Substring(sHonorificPre, 0, 1)
					if !bSex
						sHonorificSuf = scrNameGen.KhajiitMaleSuffices
						sHonorificSuf = RandomStringFromCSV(sHonorificSuf)
					elseif bSex
						sHonorificSuf = scrNameGen.KhajiitFemaleSuffices
						sHonorificSuf = RandomStringFromCSV(sHonorificSuf)
					endif
				endwhile
			endif
		endif
		if sHonorificPre != "" && (scrNameGen.KhajiitHonorificPreference != 1 || sHonorificSuf == "")
			sFirstName = sHonorificPre + "'" + sFirstName
		endif
		if sHonorificSuf != "" && (scrNameGen.KhajiitHonorificPreference != 0 || sHonorificPre == "")
			sFirstName += "-" + sHonorificSuf
		endif
	endif

    endif

    ;Check for Family Name
    bool LastName = false
    if LastNameOnly == false
        if oRace != -1
            if aRace != oRace
                aRace = oRace
                LastName = true
            elseif boNoTribe
                aRace = iRace
            elseif RandomInt(1,2) == 1
                aRace = oRace
            else
                aRace = iRace
            endif
        endif
	if boNoTribe && biNoTribe
		;skip this step
        elseif aRace == 0  && !LastName ;/Altmer/;
            if scrNameGen.AltmerFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 1  && !LastName ;/Argonian/;
            if scrNameGen.ArgonianFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 2  && !LastName ;/Bosmer/;
            if scrNameGen.BosmerFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 3  && !LastName ;/Breton/;
            if scrNameGen.BretonFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif (aRace == iRace && bAshlander) || (aRace == 4 && !LastName)  ;/Dunmer/;
            if scrNameGen.DunmerFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 5 && !LastName ;/Imperial/;
            if scrNameGen.ImperialFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 6  && !LastName ;/Khajiit/;
            if scrNameGen.KhajiitFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 7 && !LastName ;/Nord/;
            if scrNameGen.NordFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 8 && !LastName ;/Orc/;
            if scrNameGen.OrcFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 9 && !LastName ;/Redguard/;
            if scrNameGen.RedguardFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 10 && scrNameGen.NordFamilyNames != "" ;/Draugr/;
            if scrNameGen.DraugrFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 11 && scrNameGen.DremoraFamilyNames != "" ;/Dremora/;
            if scrNameGen.DremoraFamilyNameChance >= RandomInt(1,100)
                LastName = true
            endif
        endif
    elseif LastNameOnly
        if oRace != -1
            aRace = oRace
        else
            aRace = iRace
        endif
    endif

    string sFamilyName
    bool bPatronym
    if LastNameOnly || LastName
        if aRace == 0  ;/Altmer/;
            sFamilyName = scrNameGen.AltmerFamilyNames
        elseif aRace == 1 ;/Argonian/;
            sFamilyName = scrNameGen.ArgonianFamilyNames
        elseif aRace == 2  ;/Bosmer/;
            sFamilyName = scrNameGen.BosmerFamilyNames
        elseif aRace == 3  ;/Breton/;
            sFamilyName = scrNameGen.BretonFamilyNames
        elseif aRace == 4  ;/Dunmer/;
            if !bAshlander
                sFamilyName = scrNameGen.DunmerFamilyNames
            elseif bAshlander
                sFamilyName = scrNameGen.AshlanderFamilyNames
            endif
        elseif aRace == 5  ;/Imperial/;
            sFamilyName = scrNameGen.ImperialFamilyNames
        elseif aRace == 6  ;/Khajiit/;
            sFamilyName = scrNameGen.KhajiitFamilyNames
        elseif aRace == 7 || aRace == 10 ;/Nord or Draugr/;
            if aRace == 7
                bPatronym = (scrNameGen.NordPatronymicChance >= RandomInt(1,100))
            elseif aRace == 10
                bPatronym = (scrNameGen.DraugrPatronymicChance >= RandomInt(1,100))
            endif

            if !bPatronym
                sFamilyName = scrNameGen.NordFamilyNames
            elseif bSex
                sFamilyName = scrNameGen.NordFemaleFirstNames
            else
                sFamilyName = scrNameGen.NordMaleFirstNames
            endif
        elseif aRace == 8  ;/Orc/;
            sFamilyName = scrNameGen.OrcFamilyNames
        elseif aRace == 9  ;/Redguard/;
            sFamilyName = scrNameGen.RedguardFamilyNames
        elseif aRace == 11 ;/Dremora/;
            sFamilyName = scrNameGen.DremoraFamilyNames
        endif

        sFamilyName = RandomStringFromCSV(sFamilyName)
    
        ;special treatment of orc last names
        If (sFamilyName != "") && (aRace == 8 || iRace == 8) && (scrNameGen.OrcFamilyPrefixChance >= RandomInt(1,100))
            If bSex
                sFamilyName = "gra-"+ sFamilyName
            Else
                sFamilyName = "gro-"+ sFamilyName
            EndIf
        EndIf

        ;special treatment of nord or draugr last names
        If (sFamilyName != "") && (aRace == 7 || aRace == 10) && bPatronym
            If bSex
                sFamilyName += "sdottir"
            Else
                sFamilyName += "son"
            EndIf
        EndIf

    endif

    ;Check for Boss
    bool bMonikerTi = false
    bool bMonikerEp = false
    if bBoss
        if scrNameGen.BossSirChance >= RandomInt(1,100)
            bMonikerTi = true
        endif
        if scrNameGen.BossNameChance >= RandomInt(1,100)
            bMonikerEp = true
        endif
    endif

    ;Check for Title
    bool bTitle = false
    if (scrNameGen.TitlePreference == 1 || scrNameGen.TitlePreference == 5 || scrNameGen.TitlePreference == 8 \
|| scrNameGen.TitlePreference == 14 || scrNameGen.TitlePreference == 15 || scrNameGen.TitlePreference == 17) && sFamilyName != ""
        ;Skip this step
    elseif bMonikerTi == false
        if oRace != -1
            if oRace == 0  ;/Altmer/;
                if scrNameGen.AltmerTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 1  ;/Argonian/;
                if scrNameGen.ArgonianTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 2  ;/Bosmer/;
                if scrNameGen.BosmerTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 3  ;/Breton/;
                if scrNameGen.BretonTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 4  ;/Dunmer/;
                if scrNameGen.DunmerTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 5  ;/Imperial/;
                if scrNameGen.ImperialTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 6  ;/Khajiit/;
                if scrNameGen.KhajiitTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 7  ;/Nord/;
                if scrNameGen.NordTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 8  ;/Orc/;
                if scrNameGen.OrcTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 9  ;/Redguard/;
                if scrNameGen.RedguardTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            endif
        endif
        if bTitle == false
            if iRace == 0  ;/Altmer/;
                if scrNameGen.AltmerTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 1  ;/Argonian/;
                if scrNameGen.ArgonianTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 2  ;/Bosmer/;
                if scrNameGen.BosmerTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 3  ;/Breton/;
                if scrNameGen.BretonTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 4  ;/Dunmer/;
                if scrNameGen.DunmerTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 5  ;/Imperial/;
                if scrNameGen.ImperialTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 6  ;/Khajiit/;
                if scrNameGen.KhajiitTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 7  ;/Nord/;
                if scrNameGen.NordTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 8  ;/Orc/;
                if scrNameGen.OrcTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 9  ;/Redguard/;
                if scrNameGen.RedguardTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 10 ;/Draugr/;
                if scrNameGen.DraugrTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 11 ;/Dremora/;
                if scrNameGen.DremoraTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            endif
        endif
    endif

    string sTitle
    if bMonikerTi
        sTitle = RandomStringFromCSV(scrNameGen.BossSir)
    elseif bTitle
        if iClass == 0 ;/Warrior/;
            sTitle = scrNameGen.WarriorTitles + scrNameGen.GenericTitles
        elseif iClass == 1 ;/Warrior-Thief/;
            sTitle = scrNameGen.WarriorTitles + scrNameGen.ThiefTitles + scrNameGen.GenericTitles
        elseif iClass == 2 ;/Warrior-Mage/;
            sTitle =scrNameGen. WarriorTitles + scrNameGen.MageTitles + scrNameGen.GenericTitles
        elseif iClass == 3 ;/Thief/;
            sTitle = scrNameGen.ThiefTitles + scrNameGen.GenericTitles
        elseif iClass == 4 ;/Thief-Mage/;
            sTitle = scrNameGen.ThiefTitles + scrNameGen.MageTitles + scrNameGen.GenericTitles
        elseif iClass == 5 ;/Mage/;
            sTitle = scrNameGen.MageTitles + scrNameGen.GenericTitles
        else ;/Catch-All/;
            sTitle = scrNameGen.WarriorTitles + scrNameGen.ThiefTitles + scrNameGen.MageTitles + scrNameGen.GenericTitles
        endif
        sTitle = RandomStringFromCSV(sTitle)
    endif

    ;Check for Epithet
    bool bEpithet = false
    if ((scrNameGen.TitlePreference == 0 || scrNameGen.TitlePreference == 7 || scrNameGen.TitlePreference == 13) && sTitle != "") \
|| ((scrNameGen.TitlePreference == 1 || scrNameGen.TitlePreference == 9 || scrNameGen.TitlePreference == 15) && sFamilyName != "") \
|| ((scrNameGen.TitlePreference == 3 || scrNameGen.TitlePreference == 12 || scrNameGen.TitlePreference == 14) && sTitle != "" && sFamilyName != "")
        ;Skip this step
    elseif bMonikerEp == false
        if oRace != -1
            if oRace == 0  ;/Altmer/;
                if scrNameGen.AltmerEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 1  ;/Argonian/;
                if scrNameGen.ArgonianEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 2  ;/Bosmer/;
                if scrNameGen.BosmerEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 3  ;/Breton/;
                if scrNameGen.BretonEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 4  ;/Dunmer/;
                if scrNameGen.DunmerEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 5  ;/Imperial/;
                if scrNameGen.ImperialEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 6  ;/Khajiit/;
                if scrNameGen.KhajiitEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 7  ;/Nord/;
                if scrNameGen.NordEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 8  ;/Orc/;
                if scrNameGen.OrcEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 9  ;/Redguard/;
                if scrNameGen.RedguardEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            endif
        endif
        if bEpithet == false
            if iRace == 0  ;/Altmer/;
                if scrNameGen.AltmerEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 1  ;/Argonian/;
                if scrNameGen.ArgonianEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 2  ;/Bosmer/;
                if scrNameGen.BosmerEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 3  ;/Breton/;
                if scrNameGen.BretonEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 4  ;/Dunmer/;
                if scrNameGen.DunmerEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 5  ;/Imperial/;
                if scrNameGen.ImperialEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 6  ;/Khajiit/;
                if scrNameGen.KhajiitEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 7 ;/Nord/;
                if scrNameGen.NordEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 8  ;/Orc/;
                if scrNameGen.OrcEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 9  ;/Redguard/;
                if scrNameGen.RedguardEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 10 ;/Draugr/;
                if scrNameGen.DraugrEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 11 ;/Dremora/;
                if scrNameGen.DremoraEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            endif
        endif
    endif

    string sEpithet
    if bMonikerEp
        sEpithet = RandomStringFromCSV(scrNameGen.BossName)
        if sTitle != "" && (scrNameGen.TitlePreference == 1 || scrNameGen.TitlePreference == 4 || scrNameGen.TitlePreference == 6 \
|| scrNameGen.TitlePreference == 8 || scrNameGen.TitlePreference == 9 || scrNameGen.TitlePreference ==  11 || scrNameGen.TitlePreference ==  18) 
            while Find(sEpithet, Substring(sTitle, 0, 6)) != -1
                sEpithet = RandomStringFromCSV(scrNameGen.BossName)
            endwhile
        endif
    elseif bEpithet
        if iClass == 0 ;/Warrior/;
            sEpithet = scrNameGen.WarriorEpithets + scrNameGen.GenericEpithets
        elseif iClass == 1 ;/Warrior-Thief/;
            sEpithet = scrNameGen.WarriorEpithets + scrNameGen.ThiefEpithets + scrNameGen.GenericEpithets
        elseif iClass == 2 ;/Warrior-Mage/;
            sEpithet = scrNameGen.WarriorEpithets + scrNameGen.MageEpithets + scrNameGen.GenericEpithets
        elseif iClass == 3 ;/Thief/;
            sEpithet = scrNameGen.ThiefEpithets + scrNameGen.GenericEpithets
        elseif iClass == 4 ;/Thief-Mage/;
            sEpithet = scrNameGen.ThiefEpithets + scrNameGen.MageEpithets + scrNameGen.GenericEpithets
        elseif iClass == 5 ;/Mage/;
            sEpithet = scrNameGen.MageEpithets + scrNameGen.GenericEpithets
        else ;/Catch-All/;
            sEpithet = scrNameGen.WarriorEpithets + scrNameGen.ThiefEpithets + scrNameGen.MageEpithets + scrNameGen.GenericEpithets
        endif
        sEpithet = RandomStringFromCSV(sEpithet)
        if sTitle != "" && (scrNameGen.TitlePreference == 1 || scrNameGen.TitlePreference == 4 || scrNameGen.TitlePreference == 6 \
|| scrNameGen.TitlePreference == 8 || scrNameGen.TitlePreference == 9 || scrNameGen.TitlePreference ==  11 || scrNameGen.TitlePreference ==  18)
            while Find(sEpithet, Substring(sTitle, 0, 6)) != -1
                if iClass == 0 ;/Warrior/;
                    sEpithet = scrNameGen.WarriorEpithets + scrNameGen.GenericEpithets
                elseif iClass == 1 ;/Warrior-Thief/;
                    sEpithet = scrNameGen.WarriorEpithets + scrNameGen.ThiefEpithets + scrNameGen.GenericEpithets
                elseif iClass == 2 ;/Warrior-Mage/;
                    sEpithet = scrNameGen.WarriorEpithets + scrNameGen.MageEpithets + scrNameGen.GenericEpithets
                elseif iClass == 3 ;/Thief/;
                    sEpithet = scrNameGen.ThiefEpithets + scrNameGen.GenericEpithets
                elseif iClass == 4 ;/Thief-Mage/;
                    sEpithet = scrNameGen.ThiefEpithets + scrNameGen.MageEpithets + scrNameGen.GenericEpithets
                elseif iClass == 5 ;/Mage/;
                    sEpithet = scrNameGen.MageEpithets + scrNameGen.GenericEpithets
                else ;/Catch-All/;
                    sEpithet = scrNameGen.WarriorEpithets + scrNameGen.ThiefEpithets + scrNameGen.MageEpithets + scrNameGen.GenericEpithets
                endif
                sEpithet = RandomStringFromCSV(sEpithet)
            endwhile
	endif
    endif

    if LastNameOnly
        sFirstName = sFamilyName
        sFamilyName = ""
    endif

    string sFinalName
    if sFirstName != ""
        sFinalName = sFirstName
    endif

    if sFamilyName != "" && ((scrNameGen.TitlePreference != 0 && scrNameGen.TitlePreference != 6 && scrNameGen.TitlePreference != 12) || sTitle == "") \
&& ((scrNameGen.TitlePreference != 2 && scrNameGen.TitlePreference != 11 && scrNameGen.TitlePreference != 17) || sEpithet == "") \
&& ((scrNameGen.TitlePreference != 4 && scrNameGen.TitlePreference != 13 && scrNameGen.TitlePreference != 16) || (sTitle == "" && sEpithet == ""))
        if sFinalName != ""
            sFinalName += " " + sFamilyName
        else
            sFinalName = sFamilyName
        endif
    endif

    if sFinalName != ""
        if sTitle != "" && ((scrNameGen.TitlePreference != 1 && scrNameGen.TitlePreference != 8 && scrNameGen.TitlePreference != 14) || sFamilyName == "") \
&& ((scrNameGen.TitlePreference != 2 && scrNameGen.TitlePreference != 10 && scrNameGen.TitlePreference != 16) || sEpithet == "") \
&& ((scrNameGen.TitlePreference != 5 && scrNameGen.TitlePreference != 15 && scrNameGen.TitlePreference != 17) || (sFamilyName == "" && sEpithet == ""))
            sFinalName = sTitle + " " + sFinalName
        endif
        if sEpithet != "" && ((scrNameGen.TitlePreference != 0 && scrNameGen.TitlePreference != 7 && scrNameGen.TitlePreference != 13) || sTitle != "") \
&& ((scrNameGen.TitlePreference != 1 && scrNameGen.TitlePreference != 9 && scrNameGen.TitlePreference != 15) || sFamilyName != "") \
&& ((scrNameGen.TitlePreference != 3 && scrNameGen.TitlePreference != 12 && scrNameGen.TitlePreference != 14) || (sTitle == "" && sFamilyName == ""))
            sFinalName += " " + sEpithet
        endif
    endif

    return sFinalName
EndFunction


String Function GenerateMonsterName (int iRace, bool bBoss)

;   When the detected NPC is a Monster (Dragon, Dragon Priest,
;   Giant, or Hagraven) this is the function which gets called to
;   generate a name, since these Races have special treatment
;   which needs to be performed.

    ;/Race Values for Reference:
        [12]: Dragon
        [13]: Dragon Priest
        [14]: Giant
        [15]: Hagraven/;        

    string sFinalName = ""
    if iRace == 12 || iRace == 13 ;/Dragon or Dragon Priest/;
        string sFirstWord = scrNameGen.DragonNames
        sFirstWord = RandomStringFromCSV(sFirstWord)
        sFinalName = sFirstWord

        if iRace == 12 ;/Dragon/;
            If scrNameGen.Dragon3SyllableChance >= RandomInt(1,100)
                string sDragonSyllable = scrNameGen.DragonOneSyllable
                string sSecondWord = RandomStringFromCSV(sDragonSyllable)
                While sSecondWord == sFirstWord
                    sSecondWord = RandomStringFromCSV(sDragonSyllable)
                EndWhile
                string sThirdWord = RandomStringFromCSV(sDragonSyllable)
                While sThirdWord == sSecondWord || sThirdWord == sFirstWord
                    sThirdWord = RandomStringFromCSV(sDragonSyllable)
                EndWhile
                sFinalName += sSecondWord + sThirdWord
            Else
                string sSecondWord = scrNameGen.DragonTwoSyllable
                sSecondWord = RandomStringFromCSV(sSecondWord)
                sFinalName += sSecondWord
            EndIf
        elseif iRace == 13 ;/DragonPriest/;
            string sSecondWord = ""
            if scrNameGen.DragonPriest1SyllableChance < RandomInt(1,100)
                if RandomInt(1,2) == 1
                    string sDragonSyllable = scrNameGen.DragonOneSyllable
                    sSecondWord = RandomStringFromCSV(sDragonSyllable)
                    while sSecondWord == sFirstWord
                        sSecondWord = RandomStringFromCSV(sDragonSyllable)
                    endwhile
                else
                    sSecondWord = scrNameGen.DragonTwoSyllable
                    sSecondWord = RandomStringFromCSV(sSecondWord)
                endif
            endif
            if sSecondWord != ""
                sFinalName += sSecondWord
            endif
        endif

    elseif iRace == 14 ;/Giant/;
        sFinalName = scrNameGen.GiantNames
        sFinalName = RandomStringFromCSV(sFinalName)

    elseif iRace == 15 ;/Hagraven/;
        sFinalName = scrNameGen.BretonFemaleFirstNames
        sFinalName = RandomStringFromCSV(sFinalName)
    endif

    string sMonikerTi = ""
    if bBoss && scrNameGen.BossSirChance >= RandomInt(1,100)
        sMonikerTi = scrNameGen.BossSir
        sMonikerTi = RandomStringFromCSV(sMonikerTi)
    endif

    string sMonikerEp = ""
    if bBoss && scrNameGen.BossNameChance >= RandomInt(1,100)
        sMonikerEp = scrNameGen.BossName
        sMonikerEp = RandomStringFromCSV(sMonikerEp)
        if sMonikerTi != "" && (scrNameGen.TitlePreference == 1 || scrNameGen.TitlePreference == 4 || scrNameGen.TitlePreference == 6 \
|| scrNameGen.TitlePreference == 8 || scrNameGen.TitlePreference == 9 || scrNameGen.TitlePreference ==  11 || scrNameGen.TitlePreference ==  18)
            while Find(sMonikerTi, Substring(sMonikerEp, 0, 6)) != -1
                sMonikerEp = scrNameGen.BossName
                sMonikerEp = RandomStringFromCSV(sMonikerEp)
            endwhile
        endif
    endif
    
    if sFinalName != ""
        if sMonikerTi != "" && ((scrNameGen.TitlePreference != 2 && scrNameGen.TitlePreference != 10 \
&& scrNameGen.TitlePreference != 16 && scrNameGen.TitlePreference != 5 && scrNameGen.TitlePreference != 15 \
&& scrNameGen.TitlePreference != 17) || sMonikerEp == "")
            sFinalName = sMonikerTi + " " + sFinalName
        endif
        if sMonikerEp != "" && ((scrNameGen.TitlePreference != 0 && scrNameGen.TitlePreference != 7 \
&& scrNameGen.TitlePreference != 13 && scrNameGen.TitlePreference != 3 && scrNameGen.TitlePreference != 12 \
&& scrNameGen.TitlePreference != 14) || sMonikerTi == "")
            sFinalName += " " + sMonikerEp
        endif
    endif

    return sFinalName
    
EndFunction

String Function RandomStringFromCSV (string sCSV)

;   This function is almost unchanged from the original script,
;   with the exception of a small change which prevents the mod
;   from pulling the entire Name String as a valid Name value.
;   The bug occurred rarely, but by its nature was painfully
;   obvious when it came up.

;   New version: To even out the probability distribution, the
;   function now incorporates random search logic based on each
;   list containing only 378 entries. The old model, by its
;   nature, had probability skewed by the length of the entries.

    int iComma = 0
    bool EndOfString = false
    int iOffset = 0
    while !EndOfString
        iOffset = Find(sCSV, ",", iOffset) + 1
        if iOffset > 0
            iComma += 1
        else
            EndOfString = true
        endif
    endwhile

    iOffset = 0
    EndOfString = false
    int iRando = Utility.RandomInt(0,iComma - 1)
    while iRando > 0 && !EndOfString
        iRando -= 1
        int oOffset = iOffset
        iOffset = Find(sCSV, ",", iOffset) + 1
        if iOffset == 0
            iOffset = oOffset
            EndOfString = true
        endif
    endwhile

    ;/ This is the old randomizer
    int iOffset = Find(sCSV, ",", RandomInt(0, GetLength(sCSV))) + 1
    if iOffset == -1
        iOffset = 0
    endif/;

    string sName = Substring(sCSV, iOffset, Find(sCSV, ",", iOffset) - iOffset)

    ;This logic prevents the mod from pulling the entire Name String
    if Find(sName, ",") == -1
        return sName
    else
        return RandomStringFromCSV(sName)
    endif
EndFunction