Scriptname RealNamesSSEPatchPlayerAlias extends ReferenceAlias

;/  -=INTRODUCTION=-
;   Since this is such a large Script, I thought it would be
;   appropriate include this introductory header, as well as a
;   Table of Contents and Search Tags.
;
;   This script folds together the script of the two primary
;   scripts from Nellshini and Jaxonz Real Names mods for
;   OG Skyrim, the NameGenerator and the PlayerAlias, and
;   contains additional strings and scripts to emulate the
;   functionality of the old .INI file.
;
;   The .INI functionality is no longer present, and this mod
;   does not make use of Jaxonz Console Plugin, so neither the
;   .INI nor the .dll files are required.
;
;   -=TABLE OF CONTENTS=-
;   RF00 PROPERTIES AND VARIABLES, ETC.
;       RF00.0 IMPORTS
;       RF00.1 PROPERTIES
;       RF00.2 PRIVATE VARIABLES
;       RF00.X NO LONGER USED
;   CH00 PLAYER ALIAS
;       CH00.0 EVENTS AND STATES
;       CH00.1 FUNCTIONS
;   CH01 NAME GENERATOR
;       CH01.0 EVENTS AND STATES
;       CH01.1 FUNCTIONS
;   CH02 LIST MANAGER
;       CH02.0 EVENTS AND STATES
;       CH02.1 FUNCTIONS 
/;


;/  RF00 -=PROPERTIES AND VARIABLES, ETC.=- /;

;/  RF00.0 -=IMPORTS=-  /;

import Utility
import StringUtil
import JaxonzStringUtil
import Debug
import Input
Import UI
Import Game

;/  RF00.1 -=PROPERTIES=-   /;

;FormLists
FormList Property flstRaceGroups Auto
FormList Property flstAlreadyEvaluated Auto
FormList Property flstClassGroups Auto

;Messages
Message Property RealNamesSSEPatchException Auto

;Spells
Spell Property RealNamesSSEPatchCloakAbility Auto
Spell Property RealNamesSSEPatchRefreshName Auto

;Public Integers
int Property iForceRenameKey = -1 Auto
int Property iRevertNameKey = -1 Auto
int Property iGenericName = 0 Auto
int property BosmerFamilyNameChance = 1 Auto
int property BosmerFamilyNameOnlyChance = 0 Auto
int property BosmerTitleChance = 1 Auto
int property BosmerEpithetChance = 1 Auto
int property BosmerOffRaceNameChance = 0 Auto
int property AltmerFamilyNameChance = 10 Auto
int property AltmerFamilyNameOnlyChance = 0 Auto
int property AltmerTitleChance = 0 Auto
int property AltmerEpithetChance = 0 Auto
int property AltmerOffRaceNameChance = 0 Auto
int property DunmerFamilyNameChance = 80 Auto
int property DunmerFamilyNameOnlyChance = 0 Auto
int property DunmerTitleChance = 0 Auto
int property DunmerEpithetChance = 0 Auto
int property DunmerAshlanderChance = 5 Auto
int property DunmerOffRaceNameChance = 0 Auto
int property NordFamilyNameChance = 23 Auto
int property NordFamilyNameOnlyChance = 0 Auto
int property NordTitleChance = 20 Auto
int property NordEpithetChance = 20 Auto
int property NordPatronymicChance = 25 Auto
int property NordOffRaceNameChance = 0 Auto
int property KhajiitFamilyNameChance = 0 Auto
int property KhajiitFamilyNameOnlyChance = 0 Auto
int property KhajiitTitleChance = 1 Auto
int property KhajiitEpithetChance = 1 Auto
int property KhajiitMalePrefixChance = 90 Auto
int property KhajiitMaleSuffixChance = 10 Auto
int property KhajiitFemalePrefixChance = 5 Auto
int property KhajiitFemaleSuffixChance = 1 Auto
int property KhajiitOffRaceNameChance = 0 Auto
int property KhajiitHonorificPreference = 2 Auto
int property ArgonianFamilyNameChance = 0 Auto
int property ArgonianFamilyNameOnlyChance = 0 Auto
int property ArgonianTitleChance = 0 Auto
int property ArgonianEpithetChance = 0 Auto
int property ArgonianOffRaceNameChance = 0 Auto
int property ImperialFamilyNameChance = 90 Auto
int property ImperialFamilyNameOnlyChance = 0 Auto
int property ImperialTitleChance = 0 Auto
int property ImperialEpithetChance = 0 Auto
int property ImperialOffRaceNameChance = 0 Auto
int property OrcFamilyNameChance = 80 Auto
int property OrcFamilyNameOnlyChance = 0 Auto
int property OrcTitleChance = 0 Auto
int property OrcEpithetChance = 1 Auto
int property OrcFamilyPrefixChance = 100 Auto
int property OrcOffRaceNameChance = 0 Auto
int property RedguardFamilyNameChance = 2 Auto
int property RedguardFamilyNameOnlyChance = 0 Auto
int property RedguardTitleChance = 2 Auto
int property RedguardEpithetChance = 2 Auto
int property RedguardOffRaceNameChance = 0 Auto
int property BretonFamilyNameChance = 20 Auto
int property BretonFamilyNameOnlyChance = 0 Auto
int property BretonTitleChance = 0 Auto
int property BretonEpithetChance = 0 Auto
int property BretonOffRaceNameChance = 0 Auto
int property DraugrFamilyNameChance = 75 Auto
int property DraugrTitleChance = 0 Auto
int property DraugrEpithetChance = 50 Auto
int property DraugrPatronymicChance = 100 Auto
int property DremoraFamilyNameChance = 1 Auto
int property DremoraTitleChance = 5 Auto
int property DremoraEpithetChance = 5 Auto
int property BossSirChance = 30 Auto
int property BossNameChance = 30 Auto
int property Dragon3SyllableChance = 70 Auto
int property DragonPriest1SyllableChance = 20 Auto
int property TitlePreference = 16 Auto

;Public Arrays
string[] property GenericNameSearchTermsLst1 Auto
string[] property GenericNameSearchTermsLst2 Auto
string[] property GenericNameSearchTermsLst3 Auto
string[] property GenericBossSearchTermsLst1 Auto
string[] property GenericBossSearchTermsLst2 Auto
string[] property GenericBossSearchTermsLst3 Auto
string[] property BosmerMaleFirstNamesLst1 Auto
string[] property BosmerMaleFirstNamesLst2 Auto
string[] property BosmerMaleFirstNamesLst3 Auto
string[] property BosmerFemaleFirstNamesLst1 Auto
string[] property BosmerFemaleFirstNamesLst2 Auto
string[] property BosmerFemaleFirstNamesLst3 Auto
string[] property BosmerFamilyNamesLst1 Auto
string[] property BosmerFamilyNamesLst2 Auto
string[] property BosmerFamilyNamesLst3 Auto
string[] property BretonMaleFirstNamesLst1 Auto
string[] property BretonMaleFirstNamesLst2 Auto
string[] property BretonMaleFirstNamesLst3 Auto
string[] property BretonFemaleFirstNamesLst1 Auto
string[] property BretonFemaleFirstNamesLst2 Auto
string[] property BretonFemaleFirstNamesLst3 Auto
string[] property BretonFamilyNamesLst1 Auto
string[] property BretonFamilyNamesLst2 Auto
string[] property BretonFamilyNamesLst3 Auto
string[] property DunmerMaleFirstNamesLst1 Auto
string[] property DunmerMaleFirstNamesLst2 Auto
string[] property DunmerMaleFirstNamesLst3 Auto
string[] property DunmerFemaleFirstNamesLst1 Auto
string[] property DunmerFemaleFirstNamesLst2 Auto
string[] property DunmerFemaleFirstNamesLst3 Auto
string[] property DunmerFamilyNamesLst1 Auto
string[] property DunmerFamilyNamesLst2 Auto
string[] property DunmerFamilyNamesLst3 Auto
string[] property AshlanderMaleFirstNamesLst1 Auto
string[] property AshlanderMaleFirstNamesLst2 Auto
string[] property AshlanderMaleFirstNamesLst3 Auto
string[] property AshlanderFemaleFirstNamesLst1 Auto
string[] property AshlanderFemaleFirstNamesLst2 Auto
string[] property AshlanderFemaleFirstNamesLst3 Auto
string[] property AshlanderFamilyNamesLst1 Auto
string[] property AshlanderFamilyNamesLst2 Auto
string[] property AshlanderFamilyNamesLst3 Auto
string[] property ArgonianMaleFirstNamesLst1 Auto
string[] property ArgonianMaleFirstNamesLst2 Auto
string[] property ArgonianMaleFirstNamesLst3 Auto
string[] property ArgonianFemaleFirstNamesLst1 Auto
string[] property ArgonianFemaleFirstNamesLst2 Auto
string[] property ArgonianFemaleFirstNamesLst3 Auto
string[] property ArgonianFamilyNamesLst1 Auto
string[] property ArgonianFamilyNamesLst2 Auto
string[] property ArgonianFamilyNamesLst3 Auto
string[] property AltmerMaleFirstNamesLst1 Auto
string[] property AltmerMaleFirstNamesLst2 Auto
string[] property AltmerMaleFirstNamesLst3 Auto
string[] property AltmerFemaleFirstNamesLst1 Auto
string[] property AltmerFemaleFirstNamesLst2 Auto
string[] property AltmerFemaleFirstNamesLst3 Auto
string[] property AltmerFamilyNamesLst1 Auto
string[] property AltmerFamilyNamesLst2 Auto
string[] property AltmerFamilyNamesLst3 Auto
string[] property RedguardMaleFirstNamesLst1 Auto
string[] property RedguardMaleFirstNamesLst2 Auto
string[] property RedguardMaleFirstNamesLst3 Auto
string[] property RedguardFemaleFirstNamesLst1 Auto
string[] property RedguardFemaleFirstNamesLst2 Auto
string[] property RedguardFemaleFirstNamesLst3 Auto
string[] property RedguardFamilyNamesLst1 Auto
string[] property RedguardFamilyNamesLst2 Auto
string[] property RedguardFamilyNamesLst3 Auto
string[] property OrcMaleFirstNamesLst1 Auto
string[] property OrcMaleFirstNamesLst2 Auto
string[] property OrcMaleFirstNamesLst3 Auto
string[] property OrcFemaleFirstNamesLst1 Auto
string[] property OrcFemaleFirstNamesLst2 Auto
string[] property OrcFemaleFirstNamesLst3 Auto
string[] property OrcFamilyNamesLst1 Auto
string[] property OrcFamilyNamesLst2 Auto
string[] property OrcFamilyNamesLst3 Auto
string[] property NordMaleFirstNamesLst1 Auto
string[] property NordMaleFirstNamesLst2 Auto
string[] property NordMaleFirstNamesLst3 Auto
string[] property NordFemaleFirstNamesLst1 Auto
string[] property NordFemaleFirstNamesLst2 Auto
string[] property NordFemaleFirstNamesLst3 Auto
string[] property NordFamilyNamesLst1 Auto
string[] property NordFamilyNamesLst2 Auto
string[] property NordFamilyNamesLst3 Auto
string[] property KhajiitMaleFirstNamesLst1 Auto
string[] property KhajiitMaleFirstNamesLst2 Auto
string[] property KhajiitMaleFirstNamesLst3 Auto
string[] property KhajiitMalePreficesLst1 Auto
string[] property KhajiitMalePreficesLst2 Auto
string[] property KhajiitMalePreficesLst3 Auto
string[] property KhajiitMaleSufficesLst1 Auto
string[] property KhajiitMaleSufficesLst2 Auto
string[] property KhajiitMaleSufficesLst3 Auto
string[] property KhajiitFemaleFirstNamesLst1 Auto
string[] property KhajiitFemaleFirstNamesLst2 Auto
string[] property KhajiitFemaleFirstNamesLst3 Auto
string[] property KhajiitFemalePreficesLst1 Auto
string[] property KhajiitFemalePreficesLst2 Auto
string[] property KhajiitFemalePreficesLst3 Auto
string[] property KhajiitFemaleSufficesLst1 Auto
string[] property KhajiitFemaleSufficesLst2 Auto
string[] property KhajiitFemaleSufficesLst3 Auto
string[] property KhajiitFamilyNamesLst1 Auto
string[] property KhajiitFamilyNamesLst2 Auto
string[] property KhajiitFamilyNamesLst3 Auto
string[] property ImperialMaleFirstNamesLst1 Auto
string[] property ImperialMaleFirstNamesLst2 Auto
string[] property ImperialMaleFirstNamesLst3 Auto
string[] property ImperialFemaleFirstNamesLst1 Auto
string[] property ImperialFemaleFirstNamesLst2 Auto
string[] property ImperialFemaleFirstNamesLst3 Auto
string[] property ImperialFamilyNamesLst1 Auto
string[] property ImperialFamilyNamesLst2 Auto
string[] property ImperialFamilyNamesLst3 Auto
string[] property WarriorTitlesLst1 Auto
string[] property WarriorTitlesLst2 Auto
string[] property WarriorTitlesLst3 Auto
string[] property ThiefTitlesLst1 Auto
string[] property ThiefTitlesLst2 Auto
string[] property ThiefTitlesLst3 Auto
string[] property MageTitlesLst1 Auto
string[] property MageTitlesLst2 Auto
string[] property MageTitlesLst3 Auto
string[] property GenericTitlesLst1 Auto
string[] property GenericTitlesLst2 Auto
string[] property GenericTitlesLst3 Auto
string[] property WarriorEpithetsLst1 Auto
string[] property WarriorEpithetsLst2 Auto
string[] property WarriorEpithetsLst3 Auto
string[] property ThiefEpithetsLst1 Auto
string[] property ThiefEpithetsLst2 Auto
string[] property ThiefEpithetsLst3 Auto
string[] property MageEpithetsLst1 Auto
string[] property MageEpithetsLst2 Auto
string[] property MageEpithetsLst3 Auto
string[] property GenericEpithetsLst1 Auto
string[] property GenericEpithetsLst2 Auto
string[] property GenericEpithetsLst3 Auto
string[] property BossSirLst1 Auto
string[] property BossSirLst2 Auto
string[] property BossSirLst3 Auto
string[] property BossNameLst1 Auto
string[] property BossNameLst2 Auto
string[] property BossNameLst3 Auto
string[] property DragonNamesLst1 Auto
string[] property DragonNamesLst2 Auto
string[] property DragonNamesLst3 Auto
string[] property DragonOneSyllableLst1 Auto
string[] property DragonOneSyllableLst2 Auto
string[] property DragonOneSyllableLst3 Auto
string[] property DragonTwoSyllableLst1 Auto
string[] property DragonTwoSyllableLst2 Auto
string[] property DragonTwoSyllableLst3 Auto
string[] property DremoraMaleFirstNamesLst1 Auto
string[] property DremoraMaleFirstNamesLst2 Auto
string[] property DremoraMaleFirstNamesLst3 Auto
string[] property DremoraFemaleFirstNamesLst1 Auto
string[] property DremoraFemaleFirstNamesLst2 Auto
string[] property DremoraFemaleFirstNamesLst3 Auto
string[] property DremoraFamilyNamesLst1 Auto
string[] property DremoraFamilyNamesLst2 Auto
string[] property DremoraFamilyNamesLst3 Auto
string[] property GiantNamesLst1 Auto
string[] property GiantNamesLst2 Auto
string[] property GiantNamesLst3 Auto

;Public Booleans
Bool Property bMassRenaming = true Auto
Bool Property bEnabled = true Auto
Bool Property bDraugrExclusion = false Auto
Bool Property bDremoraExclusion = false Auto
Bool Property bDragonExclusion = false Auto
Bool Property bDragonPriestExclusion = false Auto
Bool Property bGiantExclusion = false Auto
Bool Property bHagravenExclusion = false Auto
Bool Property bCorpseExclusion = false Auto
Bool Property bShowRenamingNotification Auto
Bool Property bPeopleAreStrangersInst = false Auto
bool Property bPeopleAreStrangersMode = false Auto
Bool Property bRefreshDisplayedName = true Auto
bool property BosmerAltmerName = false Auto
bool property BosmerDunmerName = false Auto
bool property BosmerNordName = false Auto
bool property BosmerKhajiitName = false Auto
bool property BosmerArgonianName = false Auto
bool property BosmerImperialName = false Auto
bool property BosmerOrcName = false Auto
bool property BosmerRedguardName = false Auto
bool property BosmerBretonName = false Auto
bool property AltmerBosmerName = false Auto
bool property AltmerDunmerName = false Auto
bool property AltmerNordName = false Auto
bool property AltmerKhajiitName = false Auto
bool property AltmerArgonianName = false Auto
bool property AltmerImperialName = false Auto
bool property AltmerOrcName = false Auto
bool property AltmerRedguardName = false Auto
bool property AltmerBretonName = false Auto
bool property DunmerAltmerName = false Auto
bool property DunmerBosmerName = false Auto
bool property DunmerNordName = false Auto
bool property DunmerKhajiitName = false Auto
bool property DunmerArgonianName = false Auto
bool property DunmerImperialName = false Auto
bool property DunmerOrcName = false Auto
bool property DunmerRedguardName = false Auto
bool property DunmerBretonName = false Auto
bool property NordAltmerName = false Auto
bool property NordDunmerName = false Auto
bool property NordBosmerName = false Auto
bool property NordKhajiitName = false Auto
bool property NordArgonianName = false Auto
bool property NordImperialName = false Auto
bool property NordOrcName = false Auto
bool property NordRedguardName = false Auto
bool property NordBretonName = false Auto
bool property KhajiitAltmerName = false Auto
bool property KhajiitDunmerName = false Auto
bool property KhajiitNordName = false Auto
bool property KhajiitBosmerName = false Auto
bool property KhajiitArgonianName = false Auto
bool property KhajiitImperialName = false Auto
bool property KhajiitOrcName = false Auto
bool property KhajiitRedguardName = false Auto
bool property KhajiitBretonName = false Auto
bool property ArgonianAltmerName = false Auto
bool property ArgonianDunmerName = false Auto
bool property ArgonianNordName = false Auto
bool property ArgonianKhajiitName = false Auto
bool property ArgonianBosmerName = false Auto
bool property ArgonianImperialName = false Auto
bool property ArgonianOrcName = false Auto
bool property ArgonianRedguardName = false Auto
bool property ArgonianBretonName = false Auto
bool property ImperialAltmerName = false Auto
bool property ImperialDunmerName = false Auto
bool property ImperialNordName = false Auto
bool property ImperialKhajiitName = false Auto
bool property ImperialArgonianName = false Auto
bool property ImperialBosmerName = false Auto
bool property ImperialOrcName = false Auto
bool property ImperialRedguardName = false Auto
bool property ImperialBretonName = false Auto
bool property OrcAltmerName = false Auto
bool property OrcDunmerName = false Auto
bool property OrcNordName = false Auto
bool property OrcKhajiitName = false Auto
bool property OrcArgonianName = false Auto
bool property OrcImperialName = false Auto
bool property OrcBosmerName = false Auto
bool property OrcRedguardName = false Auto
bool property OrcBretonName = false Auto
bool property RedguardAltmerName = false Auto
bool property RedguardDunmerName = false Auto
bool property RedguardNordName = false Auto
bool property RedguardKhajiitName = false Auto
bool property RedguardArgonianName = false Auto
bool property RedguardImperialName = false Auto
bool property RedguardOrcName = false Auto
bool property RedguardBosmerName = false Auto
bool property RedguardBretonName = false Auto
bool property BretonAltmerName = false Auto
bool property BretonDunmerName = false Auto
bool property BretonNordName = false Auto
bool property BretonKhajiitName = false Auto
bool property BretonArgonianName = false Auto
bool property BretonImperialName = false Auto
bool property BretonOrcName = false Auto
bool property BretonRedguardName = false Auto
bool property BretonBosmerName = false Auto

;Public Strings
string property GenericNameSearchTerms = "" Auto
string property GenericBossSearchTerms = "" Auto
string property BosmerMaleFirstNames = "" Auto
string property BosmerFemaleFirstNames = "" Auto
string property BosmerFamilyNames = "" Auto
string property BretonMaleFirstNames = "" Auto
string property BretonFemaleFirstNames = "" Auto
string property BretonFamilyNames = "" Auto
string property DunmerMaleFirstNames = "" Auto
string property DunmerFemaleFirstNames = "" Auto
string property DunmerFamilyNames = "" Auto
string property AshlanderMaleFirstNames = "" Auto
string property AshlanderFemaleFirstNames = "" Auto
string property AshlanderFamilyNames = "" Auto
string property ArgonianMaleFirstNames = "" Auto
string property ArgonianFemaleFirstNames = "" Auto
string property ArgonianFamilyNames = "" Auto
string property AltmerMaleFirstNames = "" Auto
string property AltmerFemaleFirstNames = "" Auto
string property AltmerFamilyNames = "" Auto
string property RedguardMaleFirstNames = "" Auto
string property RedguardFemaleFirstNames = "" Auto
string property RedguardFamilyNames = "" Auto
string property OrcMaleFirstNames = "" Auto
string property OrcFemaleFirstNames = "" Auto
string property OrcFamilyNames = "" Auto
string property NordMaleFirstNames = "" Auto
string property NordFemaleFirstNames = "" Auto
string property NordFamilyNames = "" Auto
string property KhajiitMaleFirstNames = "" Auto
string property KhajiitMalePrefices = "" Auto
string property KhajiitMaleSuffices = "" Auto
string property KhajiitFemaleFirstNames = "" Auto
string property KhajiitFemalePrefices = "" Auto
string property KhajiitFemaleSuffices = "" Auto
string property KhajiitFamilyNames = "" Auto
string property ImperialMaleFirstNames = "" Auto
string property ImperialFemaleFirstNames = "" Auto
string property ImperialFamilyNames = "" Auto
string property WarriorTitles = "" Auto
string property ThiefTitles = "" Auto
string property MageTitles = "" Auto
string property GenericTitles = "" Auto
string property WarriorEpithets = "" Auto
string property ThiefEpithets = "" Auto
string property MageEpithets = "" Auto
string property GenericEpithets = "" Auto
string property BossSir = "" Auto
string property BossName = "" Auto
string property DragonNames = "" Auto
string property DragonOneSyllable = "" Auto
string property DragonTwoSyllable = "" Auto
string property DremoraMaleFirstNames = "" Auto
string property DremoraFemaleFirstNames = "" Auto
string property DremoraFamilyNames = "" Auto
string property GiantNames = "" Auto
string property Exceptions1 = "" Auto
string property Exceptions2 = "" Auto
string property Exceptions3 = "" Auto
string property ExceptionsINPC = "" Auto
string property PlayerExceptions = "," Auto

;/  RF00.2 -=PRIVATE VARIABLES=-    /;

;Private Integers
int lastentry
int iRaceIndex
int iClassIndex
int kNPC = 43
int kLeveledCharacter = 44
int kCharacter = 62
int kReference = 61

;Private Float
float LatestVersion

;Private Booleans
bool bLocked
bool bGameLoad

;Private Strings
string sGenericNameSearch
string sGenericBossSearch
string sAddedExceptions

;Private Arrays
int[] iActorTypes

;/  RF00.X -=NO LONGER USED=-   /;

;string[] Property sNamesAlignedWithRaceGroups Auto
;string[] Property sNamesINIAlignedWithRaceGroups Auto
;Bool bIniMissingWarned


;/  CH00 -=PLAYER ALIAS=-       /;
;   This portion of the script is derived from the original
;   jaxonznamegenplayeralias script. Some of the events are
;   pulled straight from the source with no changes, but there
;   are several functions which were modified significantly in
;   order to get the mod to function with the new String and Array
;   Method (vice the .INI Method). 

;/  CH00.0 -=EVENTS AND STATES=-    /;

Event OnInit()
    ;iActorTypes = new int[4]
        ;iActorTypes[0] = kNPC
        ;iActorTypes[1] = kCharacter
        ;iActorTypes[2] = kReference
        ;iActorTypes[3] = kLeveledCharacter

    LatestVersion = 1.22
    GotoState("InitializeSearch")

    RegisterForSingleUpdate(4.5)
EndEvent

Event OnCrosshairRefChange(ObjectReference ref)
;Notification("JaxonzNameGenPlayerAlias::OnCrosshairRefChange, ref.GetType():" + ref.GetType())
    If bEnabled && ref && ((ref.GetType() == kNPC) || (ref.GetType() == kCharacter)) && !flstAlreadyEvaluated.HasForm(ref) && ref.GetDisplayName() == (ref as actor).GetLeveledActorBase().GetName() && !bLocked && !bPeopleAreStrangersMode
        If AssignName(ref)
            bLocked = true  ;avoid recursive naming if flashing menu causes new OnCrosshairRefChange event
            RefreshDisplayedName()
            Wait(0.1)
            ;AreaRename()
        EndIf
    EndIf
    bLocked = False
EndEvent

;Commenting out the following events, since the AssignNames function does not perform reliably
;/Event OnCellLoad()
Trace("OnCellLoad called, bLocked:" + bLocked)
    AssignNames()
;   RegisterForSingleUpdate(1.0)
EndEvent

Event OnLocationChange(Location akOldLoc, Location akNewLoc)
Trace("OnLocationChange called, bLocked:" + bLocked)
    AssignNames()
;   RegisterForSingleUpdate(1.0)
EndEvent

Event OnPlayerLoadGame()
Trace("OnPlayerLoadGame called, bLocked:" + bLocked)
    AssignNames()
;   RegisterForSingleUpdate(2.0)
EndEvent/;

;These events should prove more reliable alternatives
Event OnCellLoad()
    if !bMassRenaming || !bEnabled
	;Do Nothing
    else
        RegisterForSingleUpdate(0.1)
    endif
EndEvent

Event OnLocationChange(Location akOldLoc, Location akNewLoc)
    if !bMassRenaming || !bEnabled
	;Do Nothing
    else
        RegisterForSingleUpdate(0.1)
    endif
EndEvent

Event OnPlayerFastTravelEnd(float afTravelTime)
    if !bMassRenaming || !bEnabled
	;Do Nothing
    else
        RegisterForSingleUpdate(0.1)
    endif
EndEvent

Event OnUpdate()
    if !bMassRenaming || !bEnabled
	;Do Nothing
    else
        AreaRename()
        RegisterForSingleUpdate(4.5)
    endif
EndEvent

Event OnPlayerLoadGame()
    ;Prepare to check for updates
    bGameLoad = true
    GotoState("UpdateSettings")
EndEvent

Event OnKeyDown(Int KeyCode)
    ObjectReference kCrosshairRef
    if (KeyCode == GetMappedKey("Activate")) && GetCurrentCrosshairRef() && !IsInMenuMode() && !IsMenuOpen("Dialogue Menu")
        kCrosshairRef = GetCurrentCrosshairRef()
        if !flstAlreadyEvaluated.HasForm(kCrosshairRef) && kCrosshairRef.GetDisplayName() == (kCrosshairRef as actor).GetLeveledActorBase().GetName() && AssignName(kCrosshairRef)
            RefreshDisplayedName()
        endIf
    elseif KeyCode == GetMappedKey("Activate") && bMassRenaming && !IsInMenuMode() && !IsMenuOpen("Dialogue Menu")
        AreaRename()
    elseIf KeyCode == iForceRenameKey && !IsInMenuMode() && !IsMenuOpen("Dialogue Menu")
        kCrosshairRef = GetCurrentCrosshairRef()
        string sCurrentName = kCrosshairRef.GetDisplayName()
        AreaRename()
        int iButton = 0
        ;Add a warning to "protected" NPCs who are detected by the Exceptions String. A response of "No!" returns 1
        if (kCrosshairRef as Actor).GetLeveledActorBase().GetName() == sCurrentName && GetException(sCurrentName)
            iButton = RealNamesSSEPatchException.Show()
        endif
        ;By evaluating iButton first, AssignName is short circuited (does not get called) when Player responsds "No!" to the warning
        if iButton == 0 && AssignName(kCrosshairRef, true)
            RefreshDisplayedName()
        elseif kCrosshairRef.GetDisplayName() == sCurrentName
            Notification("Real Names: Unable to assign a name")
        endif
    elseif KeyCode == iRevertNameKey && !IsInMenuMode() && !IsMenuOpen("Dialogue Menu")
        if (GetCurrentCrosshairRef() as Actor)
            kCrosshairRef = GetCurrentCrosshairRef()
            if (kCrosshairRef as Actor).GetLeveledActorBase().GetName() != kCrosshairRef.GetDisplayName()
                kCrosshairRef.SetDisplayName((kCrosshairRef as Actor).GetLeveledActorBase().GetName())
                RefreshDisplayedName()
		if !flstAlreadyEvaluated.HasForm(kCrosshairRef)
			flstAlreadyEvaluated.AddForm(kCrosshairRef)
		endif
            endIF
        endIf
    endif
endEvent


State UpdateSettings

Event OnBeginState()

;TestNameGenerator()

    if bEnabled ;enable mod functions
        UnregisterForAllKeys()
        ;check for presence of PeopleAreStrangers mod
        if Quest.GetQuest("_SAG_StrangersQuest_JustAddPerk")
            bPeopleAreStrangersInst = True
            bPeopleAreStrangersMode = True
            bMassRenaming = false
        endif

        bool bExceptionsValidated = true
	bool bTribeNames = true
        if bGameLoad && LatestVersion < 1.22
            if LatestVersion < 1.1
                ;Add New Terms to Arrays and Rewrite Strings
                Notification("Real Names: Current Version is " + LatestVersion + " :: Updating to Version 1.103")

                string NewGenericTitles = ",Able,Affable,Barefoot,Careless,Charming,Cross-Eyed,Debonaire,Drunk,Eager,Famous,Fickle,Fortunate,Good,Handsome,Haughty,Homeless,Hot-Headed,Indolent,Lisping,Long-Winded,Merry,Mild,Mild-Mannered,Proud,Resourceful,Simple,Thoughtful,Vain"
                string NewWarriorTitles = ",Armsman,Fighter,Hardy,Ironclad,Peacekeeper,Peerless,Protector,Shining,Soldier,Swaggering,Tireless,Valiant"
                string NewThiefTitles = ",Acrobat,Cunning,Discreet,Eloquent,Honest,Naughty,Nervous,Ranger,Shifty,Slippery,Sneaky,Tricky,Twitchy,Watchful"
                string NewMageTitles = ",Astrologer,Bookish,Brilliant,Charmed,Enlightened,Illustrious,Mage,Magician,Philosopher,Sage,Seer,Sorcerer,Wise"
                string NewGenericEpithets = ",of the Forest,of the Hills,of the Mountains,of the Plains,of the Sea,of the Valley,the Able,the Adopted,the Affable,the Ambitious,the Barefoot,the Bastard,the Blowhard,the Brash,the Careless,the Changeling,the Charming,the Cross-Eyed,the Debonaire,the Determined,the Diplomat,the Drunk,the Drunkard,the Eager,the Famed,the Fawn,the Fickle,the Foreigner,the Fortunate,the Good,the Great,the Handsome,the Haughty,the Homeless,the Hot-Headed,the Ill-Tempered,the Indolent,the Lamb,the Last,the Lisping,the Long-Winded,the Lucky,the Merry,the Mild,the Mild-Mannered,the Proud,the Resourceful,the Simple,the Thoughtful,the Vain,the Young"
                string NewWarriorEpithets = ",the Aegis,the Armsman,the Battler,the Booming,the Fighter,the Glorious,the Hardy,the Hart,the Iron,the Ironclad,the Lion,the Lionheart,the Peacekeeper,the Peerless,the Powerful,the Protector,the Ram,the Shining,the Soldier,the Steed,the Swaggering,the Thundering,the Tireless,the Tower,the Valiant,the Warden,the Warrior,the Wolf"
                string NewThiefEpithets = ",the Acrobat,the Agent,the Cunning,the Discreet,the Eloquent,the Honest,the Naughty,the Nervous,the Ranger,the Robber,the Rogue,the Shifty,the Sneaky,the Thief,the Tricky,the Twitchy,the Watchful,the Wily,the Wit"
                string NewMageEpithets = ",the Adept,the Apprentice,the Astrologer,the Brilliant,the Bookish,the Charmed,the Conjurer,the Enlightened,the Illustrious,the Learned,the Mage,the Magician,the Philosopher,the Sage,the Seer,the Sorcerer,the Spellslinger,the Wise,the Wizard"
                string NewBossSir = ",Accursed,Apostate,Blood-Thirsty,Cruel,Daedra-Blooded,Damned,Exiled,Ferocious,God-Like,Grim,Terrible,Treacherous,Tyrant"
                string NewBossName = ",of the Wastes,the Accursed,the Apostate,the Black,the Black Prince,the Blood-Thirsty,the Bloody,the Craven,the Crazy,the Cruel,the Contemptible,the Daedra-Blooded,the Daedric,the Damned,the Darkened,the Devil,the Executioner,the Exiled,the Ferocious,the Flayer,the Forgotten,the God-Like,the Grim,the Mad,the Outlaw,the Reaper,the Ripper,the Serpent,the Terrible,the Treacherous,the Tyrant,the Wretched"

                Notification("Real Names: Adding more Titles and Epithets ...")
                UpdateList(GenericTitlesLst1, GenericTitles, NewGenericTitles)
                UpdateList(GenericTitlesLst2, GenericTitles, NewGenericTitles)
                UpdateList(GenericTitlesLst3, GenericTitles, NewGenericTitles)
                lastentry = 0

                UpdateList(WarriorTitlesLst1, WarriorTitles, NewWarriorTitles)
                UpdateList(WarriorTitlesLst2, WarriorTitles, NewWarriorTitles)
                UpdateList(WarriorTitlesLst3, WarriorTitles, NewWarriorTitles)
                lastentry = 0

                UpdateList(ThiefTitlesLst1, ThiefTitles, NewThiefTitles)
                UpdateList(ThiefTitlesLst2, ThiefTitles, NewThiefTitles)
                UpdateList(ThiefTitlesLst3, ThiefTitles, NewThiefTitles)
                lastentry = 0

                UpdateList(MageTitlesLst1, MageTitles, NewMageTitles)
                UpdateList(MageTitlesLst2, MageTitles, NewMageTitles)
                UpdateList(MageTitlesLst3, MageTitles, NewMageTitles)
                lastentry = 0

                UpdateList(GenericEpithetsLst1, GenericEpithets, NewGenericEpithets)
                UpdateList(GenericEpithetsLst2, GenericEpithets, NewGenericEpithets)
                UpdateList(GenericEpithetsLst3, GenericEpithets, NewGenericEpithets)
                lastentry = 0

                UpdateList(WarriorEpithetsLst1, WarriorEpithets, NewWarriorEpithets)
                UpdateList(WarriorEpithetsLst2, WarriorEpithets, NewWarriorEpithets)
                UpdateList(WarriorEpithetsLst3, WarriorEpithets, NewWarriorEpithets)
                lastentry = 0

                UpdateList(ThiefEpithetsLst1, ThiefEpithets, NewThiefEpithets)
                UpdateList(ThiefEpithetsLst2, ThiefEpithets, NewThiefEpithets)
                UpdateList(ThiefEpithetsLst3, ThiefEpithets, NewThiefEpithets)
                lastentry = 0

                UpdateList(MageEpithetsLst1, MageEpithets, NewMageEpithets)
                UpdateList(MageEpithetsLst2, MageEpithets, NewMageEpithets)
                UpdateList(MageEpithetsLst3, MageEpithets, NewMageEpithets)
                lastentry = 0

                UpdateList(BossSirLst1, BossSir, NewBossSir)
                UpdateList(BossSirLst2, BossSir, NewBossSir)
                UpdateList(BossSirLst3, BossSir, NewBossSir)
                lastentry = 0

                UpdateList(BossNameLst1, BossName, NewBossName)
                UpdateList(BossNameLst2, BossName, NewBossName)
                UpdateList(BossNameLst3, BossName, NewBossName)
                lastentry = 0
            endif

            if LatestVersion < 1.101
                Notification("Real Names: Rewriting updated strings ...")

                WarriorTitles = "" + AssembleString(WarriorTitlesLst1) + AssembleString(WarriorTitlesLst2) + AssembleString(WarriorTitlesLst3)

                ThiefTitles = "" + AssembleString(ThiefTitlesLst1) + AssembleString(ThiefTitlesLst2) + AssembleString(ThiefTitlesLst3)

                MageTitles = "" + AssembleString(MageTitlesLst1) + AssembleString(MageTitlesLst2) + AssembleString(MageTitlesLst3)

                GenericTitles = "" + AssembleString(GenericTitlesLst1) + AssembleString(GenericTitlesLst2) + AssembleString(GenericTitlesLst3)

                WarriorEpithets = "" + AssembleString(WarriorEpithetsLst1) + AssembleString(WarriorEpithetsLst2) + AssembleString(WarriorEpithetsLst3)

                ThiefEpithets = "" + AssembleString(ThiefEpithetsLst1) + AssembleString(ThiefEpithetsLst2) + AssembleString(ThiefEpithetsLst3)

                MageEpithets = "" + AssembleString(MageEpithetsLst1) + AssembleString(MageEpithetsLst2) + AssembleString(MageEpithetsLst3)

                GenericEpithets = "" + AssembleString(GenericEpithetsLst1) + AssembleString(GenericEpithetsLst2) + AssembleString(GenericEpithetsLst3)

                BossSir = "" + AssembleString(BossSirLst1) + AssembleString(BossSirLst2) + AssembleString(BossSirLst3)

                BossName = "" + AssembleString(BossNameLst1) + AssembleString(BossNameLst2) + AssembleString(BossNameLst3)
            endif

            if LatestVersion < 1.102
                Notification("Real Names: Correcting Generic Name Search Terms ...")
                string cGenericNameSearchTerms = ",Addict,Adept,Adventurer,Afflicted,Agent,Alchemist,Alik'r,Apprentice,Archer,Arch,Argonian,Ascendant,Assassin,Assistant,Bandit,Barbarian,Bard,Barkeep,Beggar,Bodyguard,Boy,Breton,Caitiff,Captain,Captive,Centurion,Challenger,Champion,Chef,Chief,Child,Churl,Citizen,Collector,College,Commander,Commoner,Companion,Conjurer,Corpse,Corsair,Courier,Cow,Cow Hand,Cryomancer,Cultist,Dawnguard,Deathlord,Default,Devotee,Dignitary,Dishonored,Dockmaster,Dockworker,Dragon,Dremora,Driver,Drunk,Electromancer,Elf,Envoy,Escaped,Farmer,Fan,Fellow,Field,Legate,First Mate,Fisherman,Fishwife,Forager,Forsworn,Fugitive,Furrier,Gambler,General,Gent,Giant,Girl,Gravedigger,Groundskeeper,Guardian,Guard,Hag,Hagraven,Headsman,Hero,Highwayman,Hunter,Imperial,Initiate,Invader,Jailor,Justiciar,Khajiit,Knight,Kynreeve,Kynval,Laborer,Labourer,Leader,Leper,Lookout,Looter,Lord,Lowlife,Lumberjack,Madwoman,Mage,Man,Marauder,Markynaz,Master,Mercenary,Merchant,Militia,Miner,Monk,Necromage,Necromancer,Noble,Nobleman,Noblewoman,Nord,Northwatch,Novice,Orc,Orphan,Outlaw,Overlord,Patron,Peddler,Penitus,Pilgrim,Pillager,Pirate,Plunderer,Poacher,Priest,Priestess,Prisoner,Prospector,Prostitute,Quartermaster,Ravager,Redguard,Refugee,Researcher,Reveler,Rogue,Sailor,Scavenger,Scholar,Scout,Scribe,Sellsword,Servant,Shaman,Shieldmaiden,Silver,Hand,Smuggler,Soldier,Spellsword,Stormcloak,Student,Thalmor,Thane,Thief,Thug,Torturer,Town,Crier,Trapper,Travel,Summerset,Shadow,Urchin,Vagrant,Valkynaz,Vampire,Veteran,Victim,Vigilant,Visiting,Wandering,Warden,Warlord,Warrior,Wayfinder,Wench,Witch,Wizard,Woman,Wood,Cutter,Worker,Worshipper"
                CorrectList(GenericNameSearchTermsLst1, GenericNameSearchTerms, cGenericNameSearchTerms)
                CorrectList(GenericNameSearchTermsLst2, GenericNameSearchTerms, cGenericNameSearchTerms)
                CorrectList(GenericNameSearchTermsLst3, GenericNameSearchTerms, cGenericNameSearchTerms)
                Notification("Real Names: Rewriting Generic Name Search Terms string ...")
                GenericNameSearchTerms = "" + AssembleString(GenericNameSearchTermsLst1) + AssembleString(GenericNameSearchTermsLst2) + AssembleString(GenericNameSearchTermsLst3)
            endif

            if LatestVersion < 1.103
                Notification("Real Names: Adding New Search Terms ...")
                UpdateList(GenericNameSearchTermsLst1, GenericNameSearchTerms, ",Spy")
                UpdateList(GenericNameSearchTermsLst2, GenericNameSearchTerms, ",Spy")
                UpdateList(GenericNameSearchTermsLst3, GenericNameSearchTerms, ",Spy")
                lastentry = 0
                GenericNameSearchTerms = "" + AssembleString(GenericNameSearchTermsLst1) + AssembleString(GenericNameSearchTermsLst2) + AssembleString(GenericNameSearchTermsLst3)
            endif

            if bTribeNames && LatestVersion < 1.103
                Notification("Real Names: Adding Argonian and Khajiit Tribe Names ...")
                if ArgonianFamilyNames == ""
                    ArgonianFamilyNames = ","
                endif
                if KhajiitFamilyNames == ""
                    KhajiitFamilyNames = ","
                endif
                string cArgonianFamilyNames = ",Caith,Caleel,Hajiin,Hejleel,Ixtaxh,Jasaiin,Jee,Jeke,Jekiul,Julan,Juleel,Kajmeht,Lukiul,Meedish,Rajith,Rormasu,Shapmasu,Teeba,Theiliin,Thonashap,Thotaiin,Thuxleel,Waith,Waleel,Wamasu,Wasaxh,Xajhuthi,Xaleel"
                string cKhajiitFamilyNames = ",Ahzii,Amanni,ap'Anequina,ap'Corinthe,ap'Keirgo,ap'Khaj,ap'Kolthis,ap'Leyawiin,ap'Makapi,ap'Malapi,ap'Maor,ap'Mistral,ap'Ne Quin-al,ap'Niben,ap'Orcrest,ap'Pelletine,ap'Quin'rawl,ap'Rawl'kha,ap'Rimmen,ap'Saimisil,ap'Senchal,ap'Tenmar,ap'Topal,ap'Torval,ap'Zelinin,Barakar,Brae,Budiit,Gzalziit,Hammurabi,Jahan,Keirgiit,Khahir,Kioni,Leyawiit,Lhajiit,Maasziit,Mahtani,Makapiit,Malapiit,Maoriit,Nibeniit,Saimisiit,Skoomiit,Tenmariit,Topaliit,Torvaliit,Zeliniit,za'Alfiq,za'Alfiq-raht,za'Cathay,za'Cathay-raht,za'Dagi,za'Dagi-raht,za'Darloc,za'Jode,za'Jone,za'Ohmes,za'Ohmes-raht,za'Pahmar,za'Pahmar-raht,za'Rajhin,za'Senche,za'Senche-raht,za'Suthay,za'Suthay-raht,za'Tojay,za'Tojay-raht,za'Topal"

                UpdateList(ArgonianFamilyNamesLst1, ArgonianFamilyNames, cArgonianFamilyNames)
                UpdateList(ArgonianFamilyNamesLst2, ArgonianFamilyNames, cArgonianFamilyNames)
                UpdateList(ArgonianFamilyNamesLst3, ArgonianFamilyNames, cArgonianFamilyNames)
                lastentry = 0

                UpdateList(KhajiitFamilyNamesLst1, KhajiitFamilyNames, cKhajiitFamilyNames)
                UpdateList(KhajiitFamilyNamesLst2, KhajiitFamilyNames, cKhajiitFamilyNames)
                UpdateList(KhajiitFamilyNamesLst3, KhajiitFamilyNames, cKhajiitFamilyNames)
                lastentry = 0

                Notification("Real Names: Rewriting updated strings ...")
                ArgonianFamilyNames = "" + AssembleString(ArgonianFamilyNamesLst1) + AssembleString(ArgonianFamilyNamesLst2) + AssembleString(ArgonianFamilyNamesLst3)
                KhajiitFamilyNames = "" + AssembleString(KhajiitFamilyNamesLst1) + AssembleString(KhajiitFamilyNamesLst2) + AssembleString(KhajiitFamilyNamesLst3)           
            endif

            if LatestVersion < 1.21
                Notification("Real Names: Correcting Imperial Family Names ...")
                string cImperialFamilyNames = ",Caius,Maro,Metilius,Tyranus,Verulus,Atrius,Cadiusus,Faustus,Flaccus,Herennia,Olcinius,Pullia,Tullius,Abitius,Acicius,Acilius,Aerius,Afronia,Agrudilius,Albarnian,Albuttian,Alfena,Alleius,Amatius,Amiulusus,Amnis,Amphia,Angius,Antabolis,Apinia,Artoria,Artorius,Aurrus,Autrus,Baenius,Berne,Blonia,Burtilius,Caerellius,Callonus,Carbo,Caro,Carius,Catanius,Catius,Catraso,Cedus,Celata,Ceno,Cerunia,Cines,Claevius,Clanler,Closcius,Cnisia,Colollius,Conciatius,Conician,Cosades,Crunus,Cullian,Curio,Delitian,Denian,Dergius,Douar,Duronia,Egnatius,Entius,Esdrecus,Essagan,Facian,Famalius,Fauseius,Felannus,Flaeus,Flarugrius,Flavonius,Fulbenus,Fulcinius,Furotis,Gabenagus,Gabinia,Galenus,Gloriosus,Gratius,Gratus,Gravius,Hanotepelus,Harmevus,Harsinia,Hateria,Herennius,Hers,Hertarian,Horatius,Iulus,Jannus,Jucanis,Jullalian,Laecinnius,Laenius,Lalelius,Leontiulonus,Liore,Lusius,Macrina,Macro,Magia,Magius,Mantedius,Maraennius,Maria,Marius,Maro,Matius,Maximus,Menanius,Mercius,Mero,Mevureius,Mico,Mido,Muco,Munia,Musilchiotus,Muspidius,Mussillius,Nigilius,Nuccius,Nuccusius,Nuncius,Olcinius,Oranius,Ostorius,Otiustiris,Palenix,Pelelius,Petilia,Pevengius,Plalocius,Platorius,Plebo,Ponius,Pontanian,Popillius,Posuceius,Pundus,Puruseius,Quarra,Quaspus,Rato,Roscius,Rulician,Saccus,Scerius,Scinia,Secunia,Sialius,Sibassius,Siruliulus,Sosia,Spurius,Statlilius,Talanian,Tiragrius,Truptor,Tunifus,Urgelian,Urgusiso,Urtius,Uulentanus,Valerius,Valius,Valodius,Vandacia,Vant,Vantinius,Varian,Varo,Varro,Velvus,Viciulus,Vinipter,Vitellia,Vitellius,Vunnis"
                CorrectList(ImperialFamilyNamesLst1, ImperialFamilyNames, cImperialFamilyNames)
                CorrectList(ImperialFamilyNamesLst2, ImperialFamilyNames, cImperialFamilyNames)
                CorrectList(ImperialFamilyNamesLst3, ImperialFamilyNames, cImperialFamilyNames)
                Notification("Real Names: Rewriting Imperial Family Names string ...")
                ImperialFamilyNames = "" + AssembleString(ImperialFamilyNamesLst1) + AssembleString(ImperialFamilyNamesLst2) + AssembleString(ImperialFamilyNamesLst3)
            endif

            ;Reinitiate Exceptions list, to update from the old list
            Notification("Real Names: Validating Exceptions")
            if Exceptions1 != "" ;/Check to make sure Exceptions are not in a cleared state, otherwise do nothing/;
                sAddedExceptions = PlayerExceptions
                bExceptionsValidated = false
            endif

            LatestVersion = 1.22
            Notification("Real Names: Update Complete. Current Version is " + LatestVersion)
        endif 

        if bPeopleAreStrangersMode || bMassRenaming      
            RegisterForKey(GetMappedKey("Activate"))
        endif
        if !bPeopleAreStrangersMode && !bMassRenaming
            RegisterForCrosshairRef()
        EndIf
;Trace("bPeopleAreStrangersMode:" + bPeopleAreStrangersMode)
    
        ;RegisterForSingleUpdate(1.0)
        RegisterForKey(iForceRenameKey)
        RegisterForKey(iRevertNameKey)

        ;bIniMissingWarned = false
        sGenericNameSearch = GenericNameSearchTerms
        sGenericBossSearch = GenericBossSearchTerms

        if !bExceptionsValidated
             GoToState("ResetExceptions")
        endif

        if bGameLoad
            ;Clean up flstAlreadyEvaluated
;Notification("Cleaning AlreadyEvaluated formlist")
;int iRemoved = 0
;/            int index = 0
            while flstAlreadyEvaluated.GetAt(Index)
                Form kForm = flstAlreadyEvaluated.GetAt(index)
                if Find(kForm, "(FF") >= 0
                        flstAlreadyEvaluated.RemoveAddedForm(kForm)
;iRemoved += 1
                else
                    index += 1
                endif
            endwhile
;Notification(iRemoved + " unused RefIDs purged from AlreadyEvaluated formlist")
            bGameLoad = false /;
            flstAlreadyEvaluated.Revert()
        endif

        AreaRename()
        if bMassRenaming
            RegisterForSingleUpdate(4.5)
        endif

    Else
        UnregisterForAllKeys()
        UnregisterForCrosshairRef()
        ;UnregisterForUpdate()
    EndIf

    GotoState("")

EndEvent

EndState


State AssigningNames
    ;this state prevents duplicate assignment of names in a cell if called by OnPlayerLoadGame, OnCellLoad, or MCM menu.
    Function AssignNames(bool bForce = false)
;Trace("AssignNames called in AssigningNames state = no op")
        ;do nothing
    EndFunction
EndState


;/  CH00.1 -=FUNCTIONS=-        /;

function AreaRename()

;   This function makes use of a scripted Spell to facilitate
;   renaming large numbers of NPCs unobserved.
	
    if bEnabled && bMassRenaming && !bPeopleAreStrangersMode
        (GetRef() as actor).AddSpell(RealNamesSSEPatchCloakAbility, false)
        (GetRef() as actor).RemoveSpell(RealNamesSSEPatchCloakAbility)
    endif
endfunction

Bool Function AssignName(ObjectReference objNPC, bool bForce = false)

;   This is the overhead name assignment function which gets
;   called by the PlayerAlias Events and determines which other
;   functions also need to be called.
;
;   Returns TRUE upon successful name assignment.

;Trace("AssignName called on objNPC:" + objNPC + ", GetDisplayName:" + objNPC.GetDisplayName() + ", GetType:" + objNPC.GetType())
    String sNewName
    String sNamesList
    
    if !(objNPC as Actor)
        return false
    elseIf !bForce && flstAlreadyEvaluated.HasForm(objNPC) && (objNPC.GetDisplayName() != (objNPC as Actor).GetLeveledActorBase().GetName() || GetException(objNPC.GetDisplayName()))
;Trace("AssignName, objNPC:" + objNPC + " already evaluated")
        return false
    elseIf bCorpseExclusion && (objNPC as Actor).IsDead()
        return false
    else
        flstAlreadyEvaluated.AddForm(objNPC)
    endIf

    ;Don't even attempt to rename NPCs in the Exception list, unless ForceRename function is used
    if !bForce && GetException(objNPC.GetDisplayName())
        return false
    endif

    ;string sRaceNameKey = GetRaceIniKey(objNPC as Actor)
    int iRaceGroupIndex = GetRaceGroupIndex(objNPC as Actor)
    
;Trace("objNPC:" + objNPC + ", GetDisplayName:" + objNPC.GetDisplayName() + ", iRaceGroupIndex:" + iRaceGroupIndex)
    if (iRaceGroupIndex != -1) && (bForce || DetectGenericName(objNPC.GetDisplayName()))
;Trace("iRaceIndex:" + iRaceIndex + ", Sex:" + iNPCSex + ", sRaceNameKey:" + sRaceNameKey + "=" + sNamesList)

        ;fetch an appropriate name
        sNewName = GenerateNameforActor (objNPC as Actor)

        ;assign the name
        if sNewName != ""
Trace("Real Names renaming " + objNPC + " from '" + objNPC.GetDisplayName() + "' to '" + sNewName + "'")
            if iGenericName == 0
                return objNPC.SetDisplayName(sNewName, true)
            elseIf iGenericName == 1
                objNPC.SetDisplayName((objNPC as Actor).GetLeveledActorBase().GetName())
                return objNPC.SetDisplayName(sNewName + " [" + objNPC.GetDisplayName() + "]", true)
            elseIf iGenericName == 2
                objNPC.SetDisplayName((objNPC as Actor).GetLeveledActorBase().GetName())
                return objNPC.SetDisplayName(sNewName + "\n[" + objNPC.GetDisplayName() + "]", true)
            elseIf iGenericName == 3
                objNPC.SetDisplayName((objNPC as Actor).GetLeveledActorBase().GetName())
                return objNPC.SetDisplayName(sNewName + ", " + objNPC.GetDisplayName(), true)
            elseIf iGenericName == 4
                objNPC.SetDisplayName((objNPC as Actor).GetLeveledActorBase().GetName())
                return objNPC.SetDisplayName(sNewName + "\n" + objNPC.GetDisplayName(), true)
            endIf
        endIf
    elseif iRaceGroupIndex != -1
        flstAlreadyEvaluated.RemoveAddedForm(objNPC) ; Allow RealNames to try again, should changes be made to SearchLists or the NPC's Display Name
    endif
;Trace("AssignName returning false, objNPC:" + objNPC + ", Old Name:" + objNPC.GetDisplayName())
    return false
endFunction


String Function GenerateNameforActor (Actor akActor)

;   This function gets called after the AssignName function
;   determines that the detected/targeted NPC is a valid
;   candidate for renaming. This is the function which
;   actually calls the NameGenerator Functions to pull names
;   from the Name Strings and construct new names.

;trace("JaxonzNameGenPlayerAlias::GenerateNameforActor, akActor:" + akActor + ", GetName():" + akActor.GetName() + ", akActor.GetActorBase().GetRace():" + akActor.GetActorBase().GetRace() + ", akActor.GetRace():" + akActor.GetRace() + ", akActor.GetActorBase().GetSex():" + akActor.GetActorBase().GetSex())
    ActorBase kActorBase = akActor.GetLeveledActorBase()
    Race racNPC = kActorBase.GetRace()
    Int iNPCSex = kActorBase.GetSex()    ;0=male, 1=female
    Class clasNPC = kActorBase.GetClass()

    bool bNPCisBoss = DetectGenericBoss(kActorBase.GetName())

    iClassIndex = 0
    bool bClassIdentified = false
    While (iClassIndex < flstClassGroups.GetSize()) && !bClassIdentified
        if (flstClassGroups.GetAt(iClassIndex) as FormList).HasForm(clasNPC)
            bClassIdentified = True
        else
            iClassIndex += 1
        endif
    endwhile

    iRaceIndex = 0
    Bool bRaceIdentified = false
    While (iRaceIndex < flstRaceGroups.GetSize()) && !bRaceIdentified
        if (flstRaceGroups.GetAt(iRaceIndex) as FormList).HasForm(racNPC)
            bRaceIdentified = True
        Else
            iRaceIndex += 1
        EndIf
    EndWhile

    string sActor = akActor
    ;Special treatment of Torturer to give him an Imperial name
    if Find(sActor, "000B79BA") >= 0 ;/Torturer RefID/;
        iRaceIndex = 5
    endif

    if iRaceIndex == 12 ;/Dragon/;
        if bDragonExclusion
            return ""
        else
            return GenerateMonsterName(iRaceIndex, bNPCisBoss)
        endIf
    elseif iRaceIndex == 13 ;/Dragon Priest/;
        if bDragonPriestExclusion
            return ""
        else
            return GenerateMonsterName(iRaceIndex, bNPCisBoss)
        endIf
    elseif iRaceIndex == 14 ;/Giant/;
        if bGiantExclusion
            return ""
        else
            return GenerateMonsterName(iRaceIndex, bNPCisBoss)
        endIf
    elseif iRaceIndex == 15 ;/Hagraven/;
        if bHagravenExclusion
            return ""
        else
            return GenerateMonsterName(iRaceIndex, bNPCisBoss)
        endIf
    elseif bRaceIdentified
        if iRaceIndex == 10 && bDraugrExclusion
            return ""
        elseif iRaceIndex == 11 && bDremoraExclusion
            return ""
        endif

        ;Special cases for "anonymous" NPCs with contextual names
        if Find(sActor, "00039FE4") >= 0 ;Arvel the Swift's Crew, Male watch
            return "Harknir the Stout"
        elseif Find(sActor, "00039FE2") >= 0 ;Arvel the Swift's Crew, Female watch
            return "Soling Dead-Eye"
        elseif Find(sActor, "00039FE1") >= 0 ;Arvel the Swift's Crew, Puzzle-Trap victim
            return "Ill-Fated Bjorn"
        elseif Find(sActor, "000977FC") >= 0 ;Agent Sanyon, Thalmor Soldier at Talos Shrine near Helgen
            return "Agent Sanyon"
        elseif Find(sActor, "000812FD") >= 0 ;Rodulf, Bandit outside White River Watch
            return "Rodulf Red-Nose"
        elseif Find(akActor.GetBaseObject(), "00090C87") >= 0 && Find(sActor, "000FE54B") == -1 ;Dead Bouncer, Random Encounter
            return "Adril the Handler"
        elseif Find(sActor, "00F496F") >= 0 ;Savos Aren's Companion in Labyrinthian
            return "Hafnar Ice-Fist"
        elseif Find(sActor, "00F4927") >= 0 ;Savos Aren's Companion in Labyrinthan
            return "Atmah"
        endif

        return GenerateName(iRaceIndex, iClassIndex, iNPCSex as bool, bNPCisBoss)
    Else
        return ""
    EndIf

EndFunction


String Function GetNameFromString(String strNameList, int iNameIndex = -1)

;   This function is unused. Its original purpose seems to have
;   been a Debug Tool, to be used to verify that the script was
;   pulling substrings (i.e. Names) from the Name Strings as
;   expected.
;
;   Alternatively, it's possible that this function was intended
;   to be the primary means of Name Generation, but when it failed
;   to return the desired result, the NameGenerator script was
;   developed instead.

;returns individual name at index, random if index -1
;Trace("GetNameFromString, strNameList:" + strNameList + ", iNameIndex:" + iNameIndex)
    int iIteration = 0
    int iOffset = 0

    if iNameIndex != -1
        While iIteration < iNameIndex
            iOffset = Find(strNameList, ",", iOffset) + 1
            iIteration += 1
            if iOffset == 0
                return ""   ;nothing found
            EndIf
        EndWhile
    Else
        iOffset = Find(strNameList, ",", RandomInt(0, GetLength(strNameList))) + 1      
        if iOffset == -1
            iOffset == 0
        EndIf
    EndIf
    
;Trace("GetNameFromString, before return, iOffset:" + iOffset)
    return Substring(strNameList, iOffset, Find(strNameList, ",", iOffset) - iOffset)
EndFunction


Bool Function DetectGenericName(string ActorName)

;   This function gets called as part of the process to determine
;   an NPCs candidacy for renaming. It compares the base objects
;   name with the Generic Search Terms String. Specifically, it
;   crawls through the Generic Search Terms and checks to see if
;   any of the listed terms is contained in the base name.

;Trace("DetectGenericName, ActorName:" + ActorName )
    ;exit if already renamed
    if Find(ActorName, "[") > 0 || Find(ActorNAme, "\n") > 0
        return false
    EndIf

;Trace("DetectGenericName, before get sGenericNameSearch from Script, ActorName:" + ActorName )

    if sGenericNameSearch == ""
        ;Original coding, for reference
        ;sGenericNameSearch = FetchSetting("General", "GenericNameSearchTerms")
        sGenericNameSearch = GenericNameSearchTerms
;Trace("sGenericNameSearch:" + sGenericNameSearch)
    EndIf

;Trace("DetectGenericName, before sSearchTerm loop, ActorName:" + ActorName )
    string sSearchTerm

    int iOffset = 0
    bool bEndOfString = false
    While (iOffset < GetLength(sGenericNameSearch)) && !bEndOfString
        iOffset = Find(sGenericNameSearch, ",", iOffset) + 1
;Trace("iOffset:" + iOffset)
        if iOffset == 0 ;nothing found
;           iOffset = GetLength(sGenericNameSearch) + 10
            bEndOfString = true
        EndIf
        sSearchTerm = Substring(sGenericNameSearch, iOffset, Find(sGenericNameSearch, ",", iOffset) - iOffset)
;Trace("ActorName:" + ActorName + ", sSearchTerm:" + sSearchTerm + ", iOffset:" + iOffset)
        if Find(ActorName, sSearchTerm) != -1
;Trace("DetectGenericName found match, '" + sSearchTerm + "' in '" + ActorName + "'")
            return true
        EndIf
    EndWhile
;Trace("DetectGenericName, none found, ActorName:" + ActorName + ", sGenericNameSearch:" + sGenericNameSearch)
    return false
EndFunction


bool Function DetectGenericBoss(string ActorName)

;   One of my new functions. Almost the same as DetectGenericName
;   except that this function compares the base name to the
;   Generic Boss Search Terms. It is used as part of the logic
;   to determine whether a Boss Moniker should be assigned.

;Trace("DetectGenericBoss, ActorName:" + ActorName )

;Trace("DetectGenericBoss, before get sGenericBossSearch from Script, ActorName:" + ActorName )

    if sGenericBossSearch == ""
        sGenericBossSearch = GenericNameSearchTerms
;Trace("sGenericBossSearch:" + sGenericBossSearch)
    EndIf

;Trace("DetectGenericBoss, before sSearchTerm loop, ActorName:" + ActorName )
    string sSearchTerm

    int iOffset = 0
    bool bEndOfString = false
    While (iOffset < GetLength(sGenericBossSearch)) && !bEndOfString
        iOffset = Find(sGenericBossSearch, ",", iOffset) + 1
;Trace("iOffset:" + iOffset)
        if iOffset == 0 ;nothing found
;           iOffset = GetLength(sGenericBossSearch) + 10
            bEndOfString = true
        EndIf
        sSearchTerm = Substring(sGenericBossSearch, iOffset, Find(sGenericBossSearch, ",", iOffset) - iOffset)
;Trace("ActorName:" + ActorName + ", sSearchTerm:" + sSearchTerm + ", iOffset:" + iOffset)
        if Find(ActorName, sSearchTerm) != -1
;Trace("DetectGenericBoss found match, '" + sSearchTerm + "' in '" + ActorName + "'")
            return true
        EndIf
    EndWhile
;Trace("DetectGenericBoss, none found, ActorName:" + ActorName + ", sGenericBossSearch:" + sGenericBossSearch)
    return false
EndFunction


bool Function GetException(string ActorName)

;   Another of my functions. This one functions as a counter to
;   DetectGenericName, as a failsafe to prevent pre-Named NPCs
;   whose names happen to contain terms from the Generic Name
;   Search Terms (e.g. Ulfric Stormcloak) from being renamed.
;
;   Instead of crawling through the list searching the ActorName
;   for each term, this function searches the Exceptions List(s)
;   for each term in the ActorName. This reverse comparison helps
;   to prevent any false positives from occurring.

    int iOffset = 0
    string sFullName = "," + ActorName + ","
    if Find(Exceptions1, sFullName) > -1 || Find(Exceptions2, sFullName) > -1 || Find(Exceptions3, sFullName) > -1 || Find(ExceptionsINPC, sFullName) > -1 || Find(PlayerExceptions, sFullName) > -1
        return true
    else
        iOffset = Find(ActorName, " ")
        if iOffset > 0
            string sFirstWord = "," + Substring(ActorName, 0, iOffset) + ","
            if Find(Exceptions1, sFirstWord) > -1 || Find(Exceptions2, sFirstWord) > -1 || Find(Exceptions3, sFirstWord) > -1 || Find(ExceptionsINPC, sFirstWord) > -1 || Find(PlayerExceptions, sFirstWord) > -1
                return true
            else
                string sSecondWord
                int iOffset2 = Find(ActorName, " ", iOffset + 1)
                if iOffset2 > 0
                    sSecondWord = "," + Substring(ActorName, iOffset + 1, iOffset2 - iOffset - 1) + ","
                else
                    sSecondWord = "," + Substring(ActorName, iOffset + 1) + ","
                endif
                if Find(Exceptions1, sSecondWord) > -1 || Find(Exceptions2, sSecondWord) > -1 || Find(Exceptions3, sSecondWord) > -1 || Find(ExceptionsINPC, sSecondWord) > -1 || Find(PlayerExceptions, sSecondWord) > -1
                    return true
                endif
            endif
        endif
    endif

    ;if no other valid return
    return false

endfunction

Int Function GetRaceGroupIndex(Actor akActor)

;   This function is called by AssignName as part of candidacy
;   logic. It is only used as a check to ensure that the querried
;   Actor belongs to one of the pre-selected races before moving
;   on to name generation.
;
;   If you wanted to, you could remove this function from the
;   logic chain, and the NameGenerator would be able to rename
;   any NPC, regardless of Race.

    Race racNPC = akActor.GetLeveledActorBase().GetRace()
    Int iNPCSex = akActor.GetLeveledActorBase().GetSex()    ;0=male, 1=female

    iRaceIndex = 0
    Bool bRaceIdentified = false
    while (iRaceIndex < flstRaceGroups.GetSize()) && !bRaceIdentified
        if (flstRaceGroups.GetAt(iRaceIndex) as FormList).HasForm(racNPC)
            bRaceIdentified = True
        else
            iRaceIndex += 1
        endIf
    endWhile

    if !bRaceIdentified
        return -1
    endIf
    
    ;make sure monster names index selected if appropriate
    if iRaceIndex >= 12
        iNPCSex = 1
    endIf
    
    return iRaceIndex + (12*iNPCSex)
EndFunction


Function RefreshDisplayedName()

;   If enabled in the MCM, this function quickly opens and closes
;   the menu in order to force the Game Engine to redraw the
;   display name. There used to be a very brief (0.005) pause
;   between the two InvokeString() calls, but I removed it, and
;   in doing so have only observed improvement (flickering went
;   from measurable and annoying to imperceptible). Adding the 
;   IsMenuOpen condition prevents the mod from interrupting
;   dialogue.

    If bRefreshDisplayedName && IsMenuOpen("Dialogue Menu") == false && IsInMenuMode() == false
        ;quickly open and close the menu to cause name to refresh
        SetFloat("TweenMenu", "_root.TweenMenu_mc._alpha", 0.0)
        InvokeString("HUD Menu", "_global.skse.OpenMenu", "TweenMenu")
        InvokeString("HUD Menu", "_global.skse.CloseMenu", "TweenMenu")
    EndIf
EndFunction


Function AssignNames(bool bForce = false)

;   It appears that this function is intended to assign names to
;   NPCs even before they've been interacted with. It gets called
;   by OnPlayerLoadGame and OnCellLoad, but it doesn't always
;   produce the expected results.

;Trace("AssignNames called, bLocked:" + bLocked)
    if !bEnabled
        return
    EndIf
    
    if bPeopleAreStrangersMode
        return
    EndIf
    
    GoToState("AssigningNames")
    if !bLocked
        bLocked = true

        InitializeProcess()
        
        ;find all actors in the current cell
;       Wait(0.2)
        Cell celCurrent = Game.GetPlayer().GetParentCell()
        int iActorTypeIndex = iActorTypes.Length
        While iActorTypeIndex
            iActorTypeIndex -= 1
            int iActorType  = iActorTypes[iActorTypeIndex]
            Int iIndex = celCurrent.GetNumRefs(iActorType)
;           Int iIndex = celCurrent.GetNumRefs(0)
;Trace("AssignNames before While kCharacter loop, iActorTypeIndex:" + iActorTypeIndex + ", iActorType:" + iActorType + ", GetNumRefs:" + iIndex)
            While iIndex        ;for each NPC
                iIndex -= 1
;               ObjectReference objNPC = celCurrent.GetNthRef(iIndex, 0)
                ObjectReference objNPC = celCurrent.GetNthRef(iIndex, iActorType)
                AssignName(objNPC, bForce)
    ;Trace("AssignNames in kCharacter While loop, iIndex:" + iIndex)
            EndWhile
        EndWhile
        bLocked = false
    EndIf
    GoToState("")
;Trace("AssignNames ending, bLocked:" + bLocked)
EndFunction


Function InitializeProcess()
    ;Called by the AssignNames function.
    ;clear lists and flags to initiate process
    flstAlreadyEvaluated.Revert()
    ;bIniMissingWarned = false
    sGenericNameSearch = ""
EndFunction


Function TestNameGenerator()

;   Does what it says on the Tin.

    Notification("Real Names: Beginning Name Generation Test")

    int iRace
    int iClass
    int iSex
    int iBoss
    int i
    While iRace < 16
        iClass = 0
        while iClass < 7
            iSex = 0
            While iSex < 2
                iBoss = 0
                While iBoss < 2
                    i = 0
                    While i < 1000
                        Trace("GenerateName(" + iRace + ", " + iClass + ", " + (iSex as Bool) + ", " + (iBoss as Bool) + "):" + GenerateName(iRace, iClass, (iSex as Bool), (iBoss as Bool)))
                        i += 1
                    EndWhile
                    iBoss += 1
                EndWhile
                iSex += 1
            EndWhile
            iClass += 1
        EndWhile
        iRace += 1
    Notification("Real Names: iRace:" + iRace)
    EndWhile
    Notification("Real Names: Name Generation Test Complete")

EndFunction


;/  CH01 -=NAME GENERATOR=-     /;
;   This portion of the script is derived from the original
;   jaxonznamegenerator script. Each of these functions was
;   modified to some extent in order to get the mod to function
;   with the new String and Array Method (vice the .INI Method).

;/  CH01.0 -=EVENTS AND STATES=-    /;
;   None. The original script was simply a set of functions.

;/  CH01.1 -=FUNCTIONS=-        /;

String Function GenerateName (int iRace, int iClass = 3, bool bSex, bool bBoss = false) ;0=male, 1=female

;   This function is much, much longer than the original script
;   merely because all of the checks which the .INI method was
;   able to perform with string building (e.g. "race" + "sex" + ...)
;   now needs to be performed by scripted logic, and there are 12
;   races (including Draugr and Dremora) which need to be checked.
;
;   Ultimately, what the function does is after checking the NPC's
;   race and all proscribed probabilites, a name is construceted
;   by adding substrings pulled from the Name Strings. If all
;   probabilities hit, the end result is FirstName + LastName + Epithet.

    ;/String[] sRaceNames = new string[10]  ;array of race names used for reference
    sRaceNames[0] = "Altmer"
    sRaceNames[1] = "Argonian"
    sRaceNames[2] = "Bosmer"
    sRaceNames[3] = "Breton"
    sRaceNames[4] = "Dunmer"
    sRaceNames[5] = "Imperial"
    sRaceNames[6] = "Kajiit"
    sRaceNames[7] = "Nord"
    sRaceNames[8] = "Orc"
    sRaceNames[9] = "Redguard"
    sRaceNames[10] = "Draugr"
    sRaceNames[11] = "Dremora"/;

    
    ;check for Ashlander
    bool bAshlander = false
    if iRace == 4 ;/Dunmer/;
        if DunmerAshlanderChance >= RandomInt(1,100)
            bAshlander = true
        endif
    endif

    ;Check for Off-Race Name Chance
    string oRaceList = ""
    int oRace = -1
    if iRace == 0  && AltmerOffRaceNameChance >= Utility.RandomInt(1,100)
        if AltmerArgonianName
            oRaceList += "1"
        endif
        if AltmerBosmerName
            oRaceList += "2"
        endif
        if AltmerBretonName
            oRaceList += "3"
        endif
        if AltmerDunmerName
            oRaceList += "4"
        endif
        if AltmerImperialName
            oRaceList += "5"
        endif
        if AltmerKhajiitName
            oRaceList += "6"
        endif
        if AltmerNordName
            oRaceList += "7"
        endif
        if AltmerOrcName
            oRaceList += "8"
        endif
        if AltmerRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 1  && ArgonianOffRaceNameChance >= Utility.RandomInt(1,100)
        if ArgonianAltmerName
            oRaceList += "0"
        endif
        if ArgonianBosmerName
            oRaceList += "2"
        endif
        if ArgonianBretonName
            oRaceList += "3"
        endif
        if ArgonianDunmerName
            oRaceList += "4"
        endif
        if ArgonianImperialName
            oRaceList += "5"
        endif
        if ArgonianKhajiitName
            oRaceList += "6"
        endif
        if ArgonianNordName
            oRaceList += "7"
        endif
        if ArgonianOrcName
            oRaceList += "8"
        endif
        if ArgonianRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 2  && BosmerOffRaceNameChance >= Utility.RandomInt(1,100)
        if BosmerAltmerName
            oRaceList += "0"
        endif
        if BosmerArgonianName
            oRaceList += "1"
        endif
        if BosmerBretonName
            oRaceList += "3"
        endif
        if BosmerDunmerName
            oRaceList += "4"
        endif
        if BosmerImperialName
            oRaceList += "5"
        endif
        if BosmerKhajiitName
            oRaceList += "6"
        endif
        if BosmerNordName
            oRaceList += "7"
        endif
        if BosmerOrcName
            oRaceList += "8"
        endif
        if BosmerRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 3  && BretonOffRaceNameChance >= Utility.RandomInt(1,100)
        if BretonAltmerName
            oRaceList += "0"
        endif
        if BretonArgonianName
            oRaceList += "1"
        endif
        if BretonBosmerName
            oRaceList += "2"
        endif
        if BretonDunmerName
            oRaceList += "4"
        endif
        if BretonImperialName
            oRaceList += "5"
        endif
        if BretonKhajiitName
            oRaceList += "6"
        endif
        if BretonNordName
            oRaceList += "7"
        endif
        if BretonOrcName
            oRaceList += "8"
        endif
        if BretonRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 4  && DunmerOffRaceNameChance >= Utility.RandomInt(1,100)
        if DunmerAltmerName
            oRaceList += "0"
        endif
        if DunmerArgonianName
            oRaceList += "1"
        endif
        if DunmerBosmerName
            oRaceList += "2"
        endif
        if DunmerBretonName
            oRaceList += "3"
        endif
        if DunmerImperialName
            oRaceList += "5"
        endif
        if DunmerKhajiitName
            oRaceList += "6"
        endif
        if DunmerNordName
            oRaceList += "7"
        endif
        if DunmerOrcName
            oRaceList += "8"
        endif
        if DunmerRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 5  && ImperialOffRaceNameChance >= Utility.RandomInt(1,100)
        if ImperialAltmerName
            oRaceList += "0"
        endif
        if ImperialArgonianName
            oRaceList += "1"
        endif
        if ImperialBosmerName
            oRaceList += "2"
        endif
        if ImperialBretonName
            oRaceList += "3"
        endif
        if ImperialDunmerName
            oRaceList += "4"
        endif
        if ImperialKhajiitName
            oRaceList += "6"
        endif
        if ImperialNordName
            oRaceList += "7"
        endif
        if ImperialOrcName
            oRaceList += "8"
        endif
        if ImperialRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 6  && KhajiitOffRaceNameChance >= Utility.RandomInt(1,100)
        if KhajiitAltmerName
            oRaceList += "0"
        endif
        if KhajiitArgonianName
            oRaceList += "1"
        endif
        if KhajiitBosmerName
            oRaceList += "2"
        endif
        if KhajiitBretonName
            oRaceList += "3"
        endif
        if KhajiitDunmerName
            oRaceList += "4"
        endif
        if KhajiitImperialName
            oRaceList += "5"
        endif
        if KhajiitNordName
            oRaceList += "7"
        endif
        if KhajiitOrcName
            oRaceList += "8"
        endif
        if KhajiitRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 7  && NordOffRaceNameChance >= Utility.RandomInt(1,100)
        if NordAltmerName
            oRaceList += "0"
        endif
        if NordArgonianName
            oRaceList += "1"
        endif
        if NordBosmerName
            oRaceList += "2"
        endif
        if NordBretonName
            oRaceList += "3"
        endif
        if NordDunmerName
            oRaceList += "4"
        endif
        if NordImperialName
            oRaceList += "5"
        endif
        if NordKhajiitName
            oRaceList += "6"
        endif
        if NordOrcName
            oRaceList += "8"
        endif
        if NordRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 8  && OrcOffRaceNameChance >= Utility.RandomInt(1,100)
        if OrcAltmerName
            oRaceList += "0"
        endif
        if OrcArgonianName
            oRaceList += "1"
        endif
        if OrcBosmerName
            oRaceList += "2"
        endif
        if OrcBretonName
            oRaceList += "3"
        endif
        if OrcDunmerName
            oRaceList += "4"
        endif
        if OrcImperialName
            oRaceList += "5"
        endif
        if OrcKhajiitName
            oRaceList += "6"
        endif
        if OrcNordName
            oRaceList += "7"
        endif
        if OrcRedguardName
            oRaceList += "9"
        endif
    elseif iRace == 9  && RedguardOffRaceNameChance >= Utility.RandomInt(1,100)
        if RedguardAltmerName
            oRaceList += "0"
        endif
        if RedguardArgonianName
            oRaceList += "1"
        endif
        if RedguardBosmerName
            oRaceList += "2"
        endif
        if RedguardBretonName
            oRaceList += "3"
        endif
        if RedguardDunmerName
            oRaceList += "4"
        endif
        if RedguardImperialName
            oRaceList += "5"
        endif
        if RedguardKhajiitName
            oRaceList += "6"
        endif
        if RedguardNordName
            oRaceList += "7"
        endif
        if RedguardOrcName
            oRaceList += "8"
        endif
    endif

    if oRaceList != ""
        oRace = GetNthChar(oRaceList, RandomInt(0, GetLength(oRaceList) - 1)) as int
    endif

    ;Check if iRace has Family Names
    bool biNoTribe = false
    if (iRace == 0 && AltmerFamilyNames == "") || (iRace == 1 && ArgonianFamilyNames == "") || (iRace == 2 && BosmerFamilyNames == "") \
|| (iRace == 3 && BretonFamilyNames == "") || (iRace == 4 && DunmerFamilyNames == "") || (iRace == 5 && ImperialFamilyNames == "") || \
(iRace == 6 && KhajiitFamilyNames == "") || (iRace == 7 && NordFamilyNames == "") || (iRace == 8 && OrcFamilyNames == "") || (iRace == 9 && RedguardFamilyNames == "")
        biNoTribe = true
    endif

    ;Check if oRace has Family Names
    bool boNoTribe = false
    if (oRace == 0 && AltmerFamilyNames == "") || (oRace == 1 && ArgonianFamilyNames == "") || (oRace == 2 && BosmerFamilyNames == "") \
|| (oRace == 3 && BretonFamilyNames == "") || (oRace == 4 && DunmerFamilyNames == "") || (oRace == 5 && ImperialFamilyNames == "") || \
(oRace == 6 && KhajiitFamilyNames == "") || (oRace == 7 && NordFamilyNames == "") || (oRace == 8 && OrcFamilyNames == "") || (oRace == 9 && RedguardFamilyNames == "")
        boNoTribe = true
    endif

    if bAshlander
        biNoTribe = false
        if AshlanderFamilyNames == ""
            biNoTribe = true
        endif
    endif

    ;Check for Family Name Only Chance
    bool LastNameOnly = false
    if oRace != -1 && boNoTribe == false
        if oRace == 0  ;/Altmer/;
            if AltmerFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 1  ;/Argonian/;
            if ArgonianFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 2  ;/Bosmer/;
            if BosmerFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 3  ;/Breton/;
            if BretonFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 5  ;/Imperial/;
            if ImperialFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 6  ;/Khajiit/;
            if KhajiitFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 7  ;/Nord/;
            if NordFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 8  ;/Orc/;
            if OrcFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif oRace == 9  ;/Redguard/;
            if RedguardFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        endif
    endif
    if LastNameOnly == false && ((oRace != -1 && boNoTribe == false) || biNoTribe == false)
        if iRace == 0  ;/Altmer/;
            if AltmerFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 1  ;/Argonian/;
            if ArgonianFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 2  ;/Bosmer/;
            if BosmerFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 3  ;/Breton/;
            if BretonFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 4  ;/Dunmer/;
            if DunmerFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 5  ;/Imperial/;
            if ImperialFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 6  ;/Khajiit/;
            if KhajiitFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 7  ;/Nord/;
            if NordFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 8  ;/Orc/;
            if OrcFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        elseif iRace == 9  ;/Redguard/;
            if RedguardFamilyNameOnlyChance >= Utility.RandomInt(1,100)
                LastNameOnly = true
            endif
        endif
    endif


    int aRace
    string sFirstName
    if LastNameOnly == false
        if oRace != -1
            if boNoTribe
                aRace = oRace
            elseif Utility.RandomInt(1,2) == 1
                aRace = oRace
            else
                aRace = iRace
            endif
        else
            aRace = iRace
        endif
        if aRace == 0  ;/Altmer/;
            if !bSex
                sFirstName = AltmerMaleFirstNames
            elseif bSex
                sFirstName = AltmerFemaleFirstNames
            endif
        elseif aRace == 1  ;/Argonian/;
            if !bSex
                sFirstName = ArgonianMaleFirstNames
            elseif bSex
                sFirstName = ArgonianFemaleFirstNames
            endif
        elseif aRace == 2  ;/Bosmer/;
            if !bSex
                sFirstName = BosmerMaleFirstNames
            elseif bSex
                sFirstName = BosmerFemaleFirstNames
            endif
        elseif aRace == 3  ;/Breton/;
            if !bSex
                sFirstName = BretonMaleFirstNames
            elseif bSex
                sFirstName = BretonFemaleFirstNames
            endif
        elseif aRace == 4  ;/Dunmer/;
            if !bAshlander
                if !bSex
                    sFirstName = DunmerMaleFirstNames
                elseif bSex
                    sFirstName = DunmerFemaleFirstNames
                endif
            elseif bAshlander
                if !bSex
                    sFirstName = AshlanderMaleFirstNames
                elseif bSex
                    sFirstName = AshlanderFemaleFirstNames
                endif
            endif
        elseif aRace == 5  ;/Imperial/;
            if !bSex
                sFirstName = ImperialMaleFirstNames
            elseif bSex
                sFirstName = ImperialFemaleFirstNames
            endif
        elseif aRace == 6  ;/Khajiit/;
            if !bSex
                sFirstName = KhajiitMaleFirstNames
            elseif bSex
                sFirstName = KhajiitFemaleFirstNames
            endif
        elseif aRace == 7 || aRace == 10 ;/Nord or Draugr/;
            if !bSex
                sFirstName = NordMaleFirstNames
            elseif bSex
                sFirstName = NordFemaleFirstNames
            endif
        elseif aRace == 8  ;/Orc/;
            if !bSex
                sFirstName = OrcMaleFirstNames
            elseif bSex
                sFirstName = OrcFemaleFirstNames
            endif
        elseif aRace == 9  ;/Redguard/;
            if !bSex
                sFirstName = RedguardMaleFirstNames
            elseif bSex
                sFirstName = RedguardFemaleFirstNames
            endif
        elseif aRace == 11 ;/Dremora/;
            if !bSex
                sFirstName = DremoraMaleFirstNames
            elseif bSex
                sFirstName = DremoraFemaleFirstNames
            endif
        endif

        sFirstName = RandomStringFromCSV(sFirstName)

	;special treatment for Khajiit honorifics
	if sFirstName != "" && (aRace == 6 || iRace == 6) ;/Khajiit/;
		string sHonorificPre = ""
		string sHonorificSuf = ""
		if !bSex
			if KhajiitMalePrefixChance >= RandomInt(1,100)
				sHonorificPre = KhajiitMalePrefices
			endif
			if KhajiitMaleSuffixChance >= RandomInt(1,100)
				sHonorificSuf = KhajiitMaleSuffices
			endif
		elseif bSex
			if KhajiitFemalePrefixChance >= RandomInt(1,100)
				sHonorificPre = KhajiitFemalePrefices
			endif
			if KhajiitFemaleSuffixChance >= RandomInt(1,100)
				sHonorificSuf = KhajiitFemaleSuffices
			endif
		endif
		if sHonorificPre != ""
			sHonorificPre = RandomStringFromCSV(sHonorificPre)
		endif
		if sHonorificSuf != ""
			sHonorificSuf = RandomStringFromCSV(sHonorificSuf)
			if sHonorificPre != "" && KhajiitHonorificPreference == 2
				while Substring(sHonorificSuf, 0, 1) == Substring(sHonorificPre, 0, 1)
					if !bSex
						sHonorificSuf = KhajiitMaleSuffices
						sHonorificSuf = RandomStringFromCSV(sHonorificSuf)
					elseif bSex
						sHonorificSuf = KhajiitFemaleSuffices
						sHonorificSuf = RandomStringFromCSV(sHonorificSuf)
					endif
				endwhile
			endif
		endif
		if sHonorificPre != "" && (KhajiitHonorificPreference != 1 || sHonorificSuf == "")
			sFirstName = sHonorificPre + "'" + sFirstName
		endif
		if sHonorificSuf != "" && (KhajiitHonorificPreference != 0 || sHonorificPre == "")
			sFirstName += "-" + sHonorificSuf
		endif
	endif

    endif

    ;Check for Family Name
    bool LastName = false
    if LastNameOnly == false
        if oRace != -1
            if aRace != oRace
                aRace = oRace
                LastName = true
            elseif boNoTribe
                aRace = iRace
            elseif Utility.RandomInt(1,2) == 1
                aRace = oRace
            else
                aRace = iRace
            endif
        endif
	if boNoTribe && biNoTribe
		;skip this step
        elseif aRace == 0  && !LastName ;/Altmer/;
            if AltmerFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 1  && !LastName ;/Argonian/;
            if ArgonianFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 2  && !LastName ;/Bosmer/;
            if BosmerFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 3  && !LastName ;/Breton/;
            if BretonFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif (aRace == iRace && bAshlander) || (aRace == 4 && !LastName)  ;/Dunmer/;
            if DunmerFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 5 && !LastName ;/Imperial/;
            if ImperialFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 6  && !LastName ;/Khajiit/;
            if KhajiitFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 7 && !LastName ;/Nord/;
            if NordFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 8 && !LastName ;/Orc/;
            if OrcFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 9 && !LastName ;/Redguard/;
            if RedguardFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 10 && NordFamilyNames != "" ;/Draugr/;
            if DraugrFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        elseif aRace == 11 && DremoraFamilyNames != "" ;/Dremora/;
            if DremoraFamilyNameChance >= Utility.RandomInt(1,100)
                LastName = true
            endif
        endif
    elseif LastNameOnly
        if oRace != -1
            aRace = oRace
        else
            aRace = iRace
        endif
    endif

    string sFamilyName
    bool bPatronym = false
    if LastNameOnly || LastName
        if aRace == 0  ;/Altmer/;
            sFamilyName = AltmerFamilyNames
        elseif aRace == 1 ;/Argonian/;
            sFamilyName = ArgonianFamilyNames
        elseif aRace == 2  ;/Bosmer/;
            sFamilyName = BosmerFamilyNames
        elseif aRace == 3  ;/Breton/;
            sFamilyName = BretonFamilyNames
        elseif aRace == 4  ;/Dunmer/;
            if !bAshlander
                sFamilyName = DunmerFamilyNames
            elseif bAshlander
                sFamilyName = AshlanderFamilyNames
            endif
        elseif aRace == 5  ;/Imperial/;
            sFamilyName = ImperialFamilyNames
        elseif aRace == 6  ;/Khajiit/;
            sFamilyName = KhajiitFamilyNames
        elseif aRace == 7 || aRace == 10 ;/Nord or Draugr/;
            if aRace == 7
                bPatronym = (NordPatronymicChance >= RandomInt(1,100))
            elseif aRace == 10
                bPatronym = (DraugrPatronymicChance >= RandomInt(1,100))
            endif

            if !bPatronym
                sFamilyName = NordFamilyNames
            elseif bSex
                sFamilyName = NordFemaleFirstNames
            else
                sFamilyName = NordMaleFirstNames
            endif
        elseif aRace == 8  ;/Orc/;
            sFamilyName = OrcFamilyNames
        elseif aRace == 9  ;/Redguard/;
            sFamilyName = RedguardFamilyNames
        elseif aRace == 11 ;/Dremora/;
            sFamilyName = DremoraFamilyNames
        endif

        sFamilyName = RandomStringFromCSV(sFamilyName)
    
        ;special treatment of orc last names
        If (sFamilyName != "") && (aRace == 8 || iRace == 8) && (OrcFamilyPrefixChance >= RandomInt(1,100))
            If bSex
                sFamilyName = "gra-"+ sFamilyName
            Else
                sFamilyName = "gro-"+ sFamilyName
            EndIf
        EndIf

        ;special treatment of nord and draugr last names
        If (sFamilyName != "") && (aRace == 7 || aRace == 10) && bPatronym
            If bSex
                sFamilyName += "sdottir"
            Else
                sFamilyName += "son"
            EndIf
        EndIf

    endif

    ;Check for Boss
    bool bMonikerTi = false
    bool bMonikerEp = false
    if bBoss
        if BossSirChance >= Utility.RandomInt(1,100)
            bMonikerTi = true
        endif
        if BossNameChance >= Utility.RandomInt(1,100)
            bMonikerEp = true
        endif
    endif

    ;Check for Title
    bool bTitle = false
    if (TitlePreference == 1 || TitlePreference == 5 || TitlePreference == 8 \
|| TitlePreference == 14 || TitlePreference == 15 || TitlePreference == 17) && sFamilyName != ""
        ;Skip this step
    elseif bMonikerTi == false
        if oRace != -1
            if oRace == 0  ;/Altmer/;
                if AltmerTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 1  ;/Argonian/;
                if ArgonianTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 2  ;/Bosmer/;
                if BosmerTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 3  ;/Breton/;
                if BretonTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 4  ;/Dunmer/;
                if DunmerTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 5  ;/Imperial/;
                if ImperialTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 6  ;/Khajiit/;
                if KhajiitTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 7  ;/Nord/;
                if NordTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 8  ;/Orc/;
                if OrcTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 9  ;/Redguard/;
                if RedguardTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            endif
        endif
        if bTitle == false
            if iRace == 0  ;/Altmer/;
                if AltmerTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif oRace == 1  ;/Argonian/;
                if ArgonianTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 2  ;/Bosmer/;
                if BosmerTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 3  ;/Breton/;
                if BretonTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 4  ;/Dunmer/;
                if DunmerTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 5  ;/Imperial/;
                if ImperialTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 6  ;/Khajiit/;
                if KhajiitTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 7 ;/Nord/;
                if NordTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 8  ;/Orc/;
                if OrcTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 9  ;/Redguard/;
                if RedguardTitleChance >= Utility.RandomInt(1,100)
                    bTitle = true
            elseif iRace == 10 ;/Draugr/;
                if DraugrTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
            elseif iRace == 11 ;/Dremora/;
                if DremoraTitleChance >= RandomInt(1,100)
                    bTitle = true
                endif
                endif
            endif
        endif
    endif

    string sTitle
    if bMonikerTi
        sTitle = RandomStringFromCSV(BossSir)
    elseif bTitle
        if iClass == 0 ;/Warrior/;
            sTitle = WarriorTitles + GenericTitles
        elseif iClass == 1 ;/Warrior-Thief/;
            sTitle = WarriorTitles + ThiefTitles + GenericTitles
        elseif iClass == 2 ;/Warrior-Mage/;
            sTitle = WarriorTitles + MageTitles + GenericTitles
        elseif iClass == 3 ;/Thief/;
            sTitle = ThiefTitles + GenericTitles
        elseif iClass == 4 ;/Thief-Mage/;
            sTitle = ThiefTitles + MageTitles + GenericTitles
        elseif iClass == 5 ;/Mage/;
            sTitle = MageTitles + GenericTitles
        else ;/Catch-All/;
            sTitle = WarriorTitles + ThiefTitles + MageTitles + GenericTitles
        endif
        sTitle = RandomStringFromCSV(sTitle)
    endif

    ;Check for Epithet
    bool bEpithet = false
    if ((TitlePreference == 0 || TitlePreference == 7 || TitlePreference == 13) && sTitle != "") \
|| ((TitlePreference == 1 || TitlePreference == 9 || TitlePreference == 15) && sFamilyName != "") \
|| ((TitlePreference == 3 || TitlePreference == 12 || TitlePreference == 14) && sTitle != "" && sFamilyName != "")
        ;Skip this step
    elseif bMonikerEp == false
        if oRace != -1
            if oRace == 0  ;/Altmer/;
                if AltmerEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 1  ;/Argonian/;
                if ArgonianEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 2  ;/Bosmer/;
                if BosmerEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 3  ;/Breton/;
                if BretonEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 4  ;/Dunmer/;
                if DunmerEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 5  ;/Imperial/;
                if ImperialEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 6  ;/Khajiit/;
                if KhajiitEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 7  ;/Nord/;
                if NordEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 8  ;/Orc/;
                if OrcEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 9  ;/Redguard/;
                if RedguardEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            endif
        endif
        if bEpithet == false
            if iRace == 0  ;/Altmer/;
                if AltmerEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif oRace == 1  ;/Argonian/;
                if ArgonianEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 2  ;/Bosmer/;
                if BosmerEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 3  ;/Breton/;
                if BretonEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 4  ;/Dunmer/;
                if DunmerEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 5  ;/Imperial/;
                if ImperialEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 6  ;/Khajiit/;
                if KhajiitEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 7 ;/Nord/;
                if NordEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 8  ;/Orc/;
                if OrcEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 9  ;/Redguard/;
                if RedguardEpithetChance >= Utility.RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 10 ;/Draugr/;
                if DraugrEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            elseif iRace == 11 ;/Dremora/;
                if DremoraEpithetChance >= RandomInt(1,100)
                    bEpithet = true
                endif
            endif
        endif
    endif

    string sEpithet
    if bMonikerEp
        sEpithet = RandomStringFromCSV(BossName)
        if sTitle != "" && (TitlePreference == 1 || TitlePreference == 4 || TitlePreference == 6 \
|| TitlePreference == 8 || TitlePreference == 9 || TitlePreference ==  11 || TitlePreference ==  18)
            while Find(sEpithet, Substring(sTitle, 0, 6)) != -1
                sEpithet = RandomStringFromCSV(BossName)
            endwhile
        endif
    elseif bEpithet
        if iClass == 0 ;/Warrior/;
            sEpithet = WarriorEpithets + GenericEpithets
        elseif iClass == 1 ;/Warrior-Thief/;
            sEpithet = WarriorEpithets + ThiefEpithets + GenericEpithets
        elseif iClass == 2 ;/Warrior-Mage/;
            sEpithet = WarriorEpithets + MageEpithets + GenericEpithets
        elseif iClass == 3 ;/Thief/;
            sEpithet = ThiefEpithets + GenericEpithets
        elseif iClass == 4 ;/Thief-Mage/;
            sEpithet = ThiefEpithets + MageEpithets + GenericEpithets
        elseif iClass == 5 ;/Mage/;
            sEpithet = MageEpithets + GenericEpithets
        else ;/Catch-All/;
            sEpithet = WarriorEpithets + ThiefEpithets + MageEpithets + GenericEpithets
        endif
        sEpithet = RandomStringFromCSV(sEpithet)
        if sTitle != "" && (TitlePreference == 1 || TitlePreference == 4 || TitlePreference == 6 \
|| TitlePreference == 8 || TitlePreference == 9 || TitlePreference ==  11 || TitlePreference ==  18)
            while Find(sEpithet, Substring(sTitle, 0, 6)) != -1
                if iClass == 0 ;/Warrior/;
                    sEpithet = WarriorEpithets + GenericEpithets
                elseif iClass == 1 ;/Warrior-Thief/;
                    sEpithet = WarriorEpithets + ThiefEpithets + GenericEpithets
                elseif iClass == 2 ;/Warrior-Mage/;
                    sEpithet = WarriorEpithets + MageEpithets + GenericEpithets
                elseif iClass == 3 ;/Thief/;
                    sEpithet = ThiefEpithets + GenericEpithets
                elseif iClass == 4 ;/Thief-Mage/;
                    sEpithet = ThiefEpithets + MageEpithets + GenericEpithets
                elseif iClass == 5 ;/Mage/;
                    sEpithet = MageEpithets + GenericEpithets
                else ;/Catch-All/;
                    sEpithet = WarriorEpithets + ThiefEpithets + MageEpithets + GenericEpithets
                endif
                sEpithet = RandomStringFromCSV(sEpithet)
            endwhile
	endif
    endif

    if LastNameOnly
        sFirstName = sFamilyName
        sFamilyName = ""
    endif

    string sFinalName
    if sFirstName != ""
        sFinalName = sFirstName
    endif

    if sFamilyName != "" && ((TitlePreference != 0 && TitlePreference != 6 && TitlePreference != 12) || sTitle == "") \
&& ((TitlePreference != 2 && TitlePreference != 11 && TitlePreference != 17) || sEpithet == "") \
&& ((TitlePreference != 4 && TitlePreference != 13 && TitlePreference != 16) || (sTitle == "" && sEpithet == ""))
        if sFinalName != ""
            sFinalName += " " + sFamilyName
        else
            sFinalName = sFamilyName
        endif
    endif

    if sFinalName != ""
        if sTitle != "" && ((TitlePreference != 1 && TitlePreference != 8 && TitlePreference != 14) || sFamilyName == "") \
&& ((TitlePreference != 2 && TitlePreference != 10 && TitlePreference != 16) || sEpithet == "") \
&& ((TitlePreference != 5 && TitlePreference != 15 && TitlePreference != 17) || (sFamilyName == "" && sEpithet == ""))
            sFinalName = sTitle + " " + sFinalName
        endif
        if sEpithet != "" && ((TitlePreference != 0 && TitlePreference != 7 && TitlePreference != 13) || sTitle == "") \
&& ((TitlePreference != 1 && TitlePreference != 9 && TitlePreference != 15) || sFamilyName == "") \
&& ((TitlePreference != 3 && TitlePreference != 12 && TitlePreference != 14) || (sTitle == "" && sFamilyName == ""))
            sFinalName += " " + sEpithet
        endif
    endif

    return sFinalName
EndFunction


String Function GenerateMonsterName (int iRace, bool bBoss)

;   When the detected NPC is a Monster (Dragon, Dragon Priest,
;   Giant, or Hagraven) this is the function which gets called to
;   generate a name, since these Races have special treatment
;   which needs to be performed.

    ;/Race Values for Reference:
        [12]: Dragon
        [13]: Dragon Priest
        [14]: Giant
        [15]: Hagraven/;        

    string sFinalName = ""
    if iRace == 12 || iRace == 13 ;/Dragon or Dragon Priest/;
        string sFirstWord = DragonNames
        sFirstWord = RandomStringFromCSV(sFirstWord)
        sFinalName = sFirstWord

        if iRace == 12 ;/Dragon/;
            If Dragon3SyllableChance >= Utility.RandomInt(1,100)
                string sDragonSyllable = DragonOneSyllable
                string sSecondWord = RandomStringFromCSV(sDragonSyllable)
                While sSecondWord == sFirstWord
                    sSecondWord = RandomStringFromCSV(sDragonSyllable)
                EndWhile
                string sThirdWord = RandomStringFromCSV(sDragonSyllable)
                While sThirdWord == sSecondWord || sThirdWord == sFirstWord
                    sThirdWord = RandomStringFromCSV(sDragonSyllable)
                EndWhile
                sFinalName += sSecondWord + sThirdWord
            Else
                string sSecondWord = DragonTwoSyllable
                sSecondWord = RandomStringFromCSV(sSecondWord)
                sFinalName += sSecondWord
            EndIf
        elseif iRace == 13 ;/DragonPriest/;
            string sSecondWord = ""
            if DragonPriest1SyllableChance < Utility.RandomInt(1,100)
                if Utility.RandomInt(1,2) == 1
                    string sDragonSyllable = DragonOneSyllable
                    sSecondWord = RandomStringFromCSV(sDragonSyllable)
                    while sSecondWord == sFirstWord
                        sSecondWord = RandomStringFromCSV(sDragonSyllable)
                    endwhile
                else
                    sSecondWord = DragonTwoSyllable
                    sSecondWord = RandomStringFromCSV(sSecondWord)
                endif
            endif
            if sSecondWord != ""
                sFinalName += sSecondWord
            endif
        endif

    elseif iRace == 14 ;/Giant/;
        sFinalName = GiantNames
        sFinalName = RandomStringFromCSV(sFinalName)

    elseif iRace == 15 ;/Hagraven/;
        sFinalName = BretonFemaleFirstNames
        sFinalName = RandomStringFromCSV(sFinalName)
    endif

    string sMonikerTi = ""
    if bBoss && BossSirChance >= Utility.RandomInt(1,100)
        sMonikerTi = BossSir
        sMonikerTi = RandomStringFromCSV(sMonikerTi)
    endif

    string sMonikerEp = ""
    if bBoss && BossNameChance >= RandomInt(1,100)
        sMonikerEp = BossName
        sMonikerEp = RandomStringFromCSV(sMonikerEp)
        if sMonikerTi != "" && (TitlePreference == 1 || TitlePreference == 4 || TitlePreference == 6 \
|| TitlePreference == 8 || TitlePreference == 9 || TitlePreference ==  11 || TitlePreference ==  18)
            while Find(sMonikerTi, Substring(sMonikerEp, 0, 6)) != -1
                sMonikerEp = BossName
                sMonikerEp = RandomStringFromCSV(sMonikerEp)
            endwhile
        endif
    endif
    
    if sFinalName != ""
        if sMonikerTi != "" && ((TitlePreference != 2 && TitlePreference != 10 \
&& TitlePreference != 16 && TitlePreference != 5 && TitlePreference != 15 \
&& TitlePreference != 17) || sMonikerEp == "")
            sFinalName = sMonikerTi + " " + sFinalName
        endif
        if sMonikerEp != "" && ((TitlePreference != 0 && TitlePreference != 7 \
&& TitlePreference != 13 && TitlePreference != 3 && TitlePreference != 12 \
&& TitlePreference != 14) || sMonikerTi == "")
            sFinalName += " " + sMonikerEp
        endif
    endif

    return sFinalName
    
EndFunction

String Function RandomStringFromCSV (string sCSV)

;   This function is almost unchanged from the original script,
;   with the exception of a small change which prevents the mod
;   from pulling the entire Name String as a valid Name value.
;   The bug occurred rarely, but by its nature was painfully
;   obvious when it came up.

;   New version: To even out the probability distribution. The
;   old model, by its nature, had probability skewed by the
;   length of the entries.

    int iComma = 0
    bool EndOfString = false
    int iOffset = 0
    while !EndOfString
        iOffset = Find(sCSV, ",", iOffset) + 1
        if iOffset > 0
            iComma += 1
        else
            EndOfString = true
        endif
    endwhile

    iOffset = 0
    EndOfString = false
    int iRando = Utility.RandomInt(0,iComma - 1)
    while iRando > 0 && !EndOfString
        iRando -= 1
        int oOffset = iOffset
        iOffset = Find(sCSV, ",", iOffset) + 1
        if iOffset == 0
            iOffset = oOffset
            EndOfString = true
        endif
    endwhile

    ;/ This is the old randomizer
    int iOffset = Find(sCSV, ",", RandomInt(0, GetLength(sCSV))) + 1
    if iOffset == -1
        iOffset = 0
    endif/;

    string sName = Substring(sCSV, iOffset, Find(sCSV, ",", iOffset) - iOffset)

    ;This logic prevents the mod from pulling the entire Name String
    if Find(sName, ",") == -1
        return sName
    else
        return RandomStringFromCSV(sName)
    endif
EndFunction


;/  CH02 -=LIST MANAGER=-       /;
;   This is the single longest portion of the script, just due
;   to repetitiveness. There are 38 strings which get translated
;   into a series of arrays. The setup of the MCM limits each
;   string to 378 entries.

;/  CH02.0 -=EVENTS AND STATES=-    /;

State InitializeSearch

Event OnBeginState()
;/EXCEPTIONS LIST/;
;   This list is too large to translate into a manageable set of
;   arrays and was even too long to fit into one string. But it
;   is a very important part of the mod. This string is used by
;   the DetectGenericName function to ensure that Named NPCs do
;   not get inadvertently renamed.

    Exceptions1 = ",Hrothmund,Aicantar,Ancano,Ancarion,Aringoth,Arondil,Calcelmo,Estormo,Fasendil,Kornalus,Linwe,Lorcalin,Melaran,Muril,Naris,Nelacar,Nerien,Nurelion,Ondolemar,Orthorn,Quaranir,Rulindil,Runil,Sanyon,Tandil,Ulundil,Valmir,Viarmo,Vingalmo,Arivanya,Curwe,Elenwen,Endarie,Faralda,Minorne,Nenya,Niranye,Nirya,Reldith,Taarie,Valie,Deekus,Derkeethus,Madesi,Neetrenaza,Usha,Veezara,Beem-Ja,Brand-Shei,Gajul-Lei,Gulum-Ei,Ilas-Tei,Jaree-Ra,Talen-Jei,Teeba-Ei,Deep-In-His-Cups,Scouts-Many-Marshes,Stands-In-Shallows,Watches-The-Roots,Deeja,Keerava,Shahvee,Wujeeta,From-Deepest-Fathoms,Takes-In-Light,Anoriath,Dervenin,Edorfin,Elrindir,Endrast,Enthir,Faendal,Gadnor,Gelebros,Girduin,Gwilin,Malborn,Niruin,Orchendor,Ronthil,Syndus,Thaer,Ungrien,Valindor,Anuriel,Ardwen,Beleval,Brelas,Galathil,Nimphaneth,Nimriel,Nivenor,Wylandriah,Morven,Adeber,Ainethach,Alain,Amaund,Anton,Arniel,Banning,Belchimac,Belethor,Bendt,Blaise,Braig,Brandish,Cedran,Celann,Clinton,Cosnach,Cynric,Dalan,Degaine,Delacourt,Delvin,Donnel,Dorian,Dryston,Duach,Eltrys,Emmanuel,Enmon,Ennoc,Erj,Esmond,Etienne,Francois,Garvey,Gauldur,Giraud,Glover,Hamelyn,Hathrasil,Herluin,Imedhnain,Jouane,Krag,Louis,Madanach,Malkoran,Maurice,Mercer,Moric,Nepos,Octieve,Odvan,Omluag,Perth,Phinis,Rahd,Rondach,Sam,Sebastian,Staubin,Stromm,Thomas,Tuthul,Tynan,Uraccen,Weylin,Willem,Angeline,Babette,Bothela,Cairine,Carcette,Colette,Daighre,Delphine,Eola,Erith,Fianna,Isabelle,Julienne,Kaie,Laelette,Lisette,Madena,Mena,Michel,Mirabelle,Modhna,Muiri,Nana,Rhiada,Sabine,Senna,Sorine,Sybille,Uaile,Vivienne,Voada,Adril,Ambarys,Arvel,Athis,Aval,Belyn,Bradyn,Casimir,Daynas,Dravin,Drelas,Drevis,Drovas,Erandur,Evul,Falas,Faldrus,Faryl,Feran,Fethis,Galdrus,Garan,Garyn,Geldis,Indaryn,Jiub,Lleril,Malthyr,Malur,Maluril,Malyn,Meden,Mithorpa,Modyn,Naris,Neloth,Orini,Othreloth,Raleth,Ralis,Ravam,Ravyn,Revus,Revyn,Rirns,Romlyn,Saden,Sarthis,Savos,Servos,Sevan,Slitter,Sondas,Talvas,Talvur,Taron,Teldryn,Tolendos,Tythis,Ulves,Ulyn,Valin,Vals,Vanryth,Vendil,Veren,Wyndelius,Aduri,Aphia,Aranea,Avrusa,Bralsa,Brelyna,Cindiri,Dinya,Dravynea,Dreyla,Elvali,Elynea,Gabriella,Idesa,Ildari,Irileth,Jenassa,Karliah,Luaffyn,Marise,Medresi,Merilar,Milore,Mireli,Mirri,Niluva,Selveni,Suvaris,Synda,Tilisu,Varona,Voldsea,Adonato,Adventus,Aerin,Agrius,Alethius,Aquillius,Aventus,Bassianus,Caius,Calixto,Cicero,Constantius,Corpulus,Crescius,Dexion,Dirge,Ennodius,Falx,Florentius,Gaius,Gallus,Gavros,Gian the Fist,Leontius,Lucan,Mallus,Marcurio,Maro,Metilius,Noster,Orthus,Paratus,Pavo,Plautis,Proventus,Quentin,Quintus,Reburrus,Rexus,Rogatus,Salvianus,Samuel,Septimus,Sergius,Severio,Silus,Sorex,Sulla,Tacitus,Taurinus,Telrav,Terek,Titus,Tyranus,Vantus,Varnius,Venarus,Verulus,Xander,Tullius,"

    Exceptions2 = ",Salonia,Adelaisa,Adrianne,Aia,Alessandra,Alexia,Arcadia,Camilla,Carlotta,Constance,Edda,Ganna,Gemma,Gianna,Gilfre,Illia,Indara,Larina,Leonara,Lucia,Mila,Pantea,Raen,Ria,Safia,Silana,Silvia,Sosia,Valga,Vex,Vigdis,Viola,Vittoria,Dro'marash,J'darr,J'datharr,J'Kier,J'zargo,J'zhar,Kharjo,Kesh,Ma'dran,Ma'jhad,Ma'randru-jo,Ma'tasarr,Ma'zaka,M'aiq,Ra'jirr,Ra'kheran,Ri'saad,Vasha,Ahkari,Ahjisi,Atahbah,Khayla,Ra'zhinda,Shavari,Tsavani,Tsrasuna,Zaynabi,Barknar,Engar,Fultheim,Gunjar,Kjeld,Nikulas,Sond,Torkild,Adalvald,Addvar,Addvild,Agmaer,Akar,Alding,Aldis,Alfarinn,Alvor,Andurs,Angrenor,Angvid,Argis,Arnbjorn,Arngeir,Arnskar,Arrald,Asbjorn,Asgeir,Aslfur,Assur,Atar,Aventus,Avulstein,Badnir,Balbus,Baldor,Balgruuf,Balimund,Bassianus,Beirand,Belrand,Benkum,Benor,Bersi,Bjorlam,Bjornolfr,Bolgeir,Bolli,Bolund,Borgny,Borri,Borvir,Bottar,Briehl,Brill,Brond,Brunwulf,Brynjolf,Bulfrek,Calder,Christer,Dagur,Dengeir,Deor,Drahff,Drennen,Eimar,Einarth,Elgrim,Elmus,Embry,Eorlund,Erik,Erikur,Erlendr,Esbern,Falk,Farengar,Farkas,Felldir,Fenrig,Filnjar,Frodnar,Frofnir,Froki,Frorkmar,Frothar,Galmar,Garthar,Garuk,Geimund,Geirlund,Gestur,Gissur,Gjak,Gjalund,Gjuk,Gloth,Golldir,Gonnar,Gorm,Gort,Gralnach,Gregor,Grimvar,Grisvar,Gunding,Gunmar,Gunnar,Guthrum,Hadring,Hadvar,Hafnar,Hajvarr,Hakar,Haknir,Hakon,Halbarn,Haming,Hargar,Harkon,Harlaug,Harrald,Heddic,Heimskr,Heimvar,Helvard,Hemming,Henrik,Heratar,Hern,Hewnon,Hjalfar,Hjornskar,Hjorunn,Hod,Hofgrir,Hogni,Holgeir,Horgeir,Horik,Horm,Hreinn,Hroar,Hrodulf,Hroggar,Hrollod,Hrolmir,Hrongar,Hrothmund,Hunroor,Idolaf,Igmund,Irlof,Irnskar,Istar,Jenssen,Jervar,Jesper,Jod,Jofthor,Jolf,Jon,Jorgen,Joric,Jorleif,Jorn,Jurgen,Kai,Karl,Kibell,Kjar,Kleppr,Klimmek,Knjakr,Knud,Kodlak,Kodrir,Korir,Kottir,Kraldar,Kust,Kuvar,Kvenel,Kyr,Lars,Leifur,Leigelf,Lemkil,Llewellyn,Lod,Lodi,Lodvar,Logrolf,Lokil,Lokir,Lond,Lorenz,Lortheim,Lund,Lygrleid,Lynoit,Majni,Markus,Mathies,Maul,Metilius,Mikael,Miraak,Molgrom,Mralki,Narfi,Nelkir,Niels,Nils,Odar,Odfel,Oengul,Ogmund,Olaf,Olfrid,Ollrod,Onmund,Orgnar,Orthjolf,Oslaf,Pactur,Raerek,Ragnar,Ralof,Ranmir,Rargal,Reves,Rhorlak,Rigel,Rissing,Rogen,Roggi,Roggvir,Rolff,Rorik,Rorlund,Rundi,Rustleif,Sabjorn,Saerlund,Sibbi,Siddgeir,Sifnar,Sigaar,Sigar,Sigurd,Sild,Sinding,Sinmir,Sirgar,Sirkjorg,Skaggi,Skald,Skeggr,Skjor,Skuli,Skulnar,Skulvar,Snilf,Snilling,Snorreid,Sogrlaf,Solaf,Stalf,Steirod,Stenvar,Stig,Storn,Styrr,Sulvar,Svaknir,Sven,Talsgar,Thadgeir,Thaer,Thalin,Thjollod,Thonar,Thongvor,Thonjolf,Thonnir,Thorald,Thorek,Thoring,Thorygg,Thorgar,Thrynn,Tolan,Tolfdir,Torbjorn,Torolf,Torsten,Torvar,High King Torygg,Torygg,Trilf,Tsun,Tulvur,Ulfberth,Ulfgar,Ulfr,Ulfric,Unmid,Vald,Valdar,Valdimar,Valdr,Vanik,Vekel,Verner,Vidgrod,Viding,Vidrald,Vignar,Vilkas,Vilod,Vipir,Virkmund,Volk,Vorstag,Vulwulf,Wilhelm,Wilmuth,Wulf,Wulfgar,Wuunferth,Yngvar,Yrsarald,Eydis,Idgrod,Karita,Liesl,"

    Exceptions3 = ",Abelone,Adisla,Aela,Aeri,Aeta,Agna,Agni,Agnis,Alea,Alfhild,Alva,Alvide,Annekke,Angi,Anise,Anska,Argi,Ari,Asta,Astrid,Beitild,Bergritte,Betrid,Birna,Bjartur,Bodil,Bolfrida,Boti,Breya,Brina,Britte,Bryling,Bujold,Dagny,Danica,Dorthe,Drifa,Edith,Edla,Eirid,Eisa,Elda,Elisif,Erdi,Eris,Evette,Faida,Fanari,Fastred,Finna,Fjori,Fjola,Fjotra,Frabbi,Fralia,Frea,Freir,Frida,Fridrika,Fruki,Fura,Gerda,Gerdur,Gisli,Gormlaith,Grelka,Grelod,Greta,Grete,Gretta,Grosta,Gwendolyn,Haelga,Hafjorg,Hamal,Haran,Hefid,Helgi,Helgird,Herkja,Hermir,Hert,Hestla,Hilde,Hillevi,Hilund,Hjordis,Hoddreid,Hrefna,Hroki,Huki,Hulda,Iddra,Illdi,Inge,Ingjard,Ingrid,Ingun,Iona,Irgnir,Jala,Jora,Jordis,Katla,Katria,Laila,Lami,Lillith,Lisbet,Lydia,Lynly,Margret,Matlara,Maven,Minette,Mjoll,Morwen,Narri,Nilsine,Njada,Nura,Olava,Olda,Olfina,Oriella,Orla,Rakel,Rigel,Rikke,Ruki,Runa,Sapphire,Serana,Sigrid,Silda,Sissel,Sofie,Sonir,Sorli,Susanna,Svana,Svari,Swanhvir,Sylgja,Tekla,Temba,Thaena,Threki,Tilde,Tilma,Tormir,Tova,Una,Uthgerd,Valerica,Vori,Yrsa,Ysolda,Mogrul,Balagog,Bashnag,Borkul,Burguk,Durak,Dushnamub,Gadba,Gat,Ghamorz,Ghorbash,Ghunzul,Grogmar,Grushnag,Gularzob,Kharag,Larak,Lob,Lurbuk,Mahk,Makhel,Malkus,Mauhulakh,Moth,Mul,Mulush,Nagrub,Oglub,Ogol,Olur,Ulag,Umurn,Urag,Yamarz,Yar,Arob,Atub,Bagrak,Batum,Bolar,Bor,Borgakh,Dulug,Garakh,Ghak,Gharol,Ghorza,Gul,Lash,Mogdurz,Murbul,Sharamph,Shel,Shuftharz,Ugor,Urog,Urzoga,Yag,Yatul,Ahtar,Alesan,Amren,Ataf,Avidius,Azzada,Azzadal,Baral,Brenuin,Endon,Ennis,Falion,Fihada,Firir,Haldyn,Isran,Jawanan,Kayd,Kematu,Majid,Mani,Maramal,Namasur,Nazeem,Nazir,Razelan,Shadr,Talib,Adara,Ahlam,Anwen,Atmah,Braith,Eriana,Faleen,Iman,Jayri,Jonna,Kerah,Lu'ah,Niyya,Ramati,Rayya,Rochelle,Saadia,Saffir,Saliah,Salma,Sayma,Seren,Sudi,Tonilia,Umana,Yisra,Zaria,Alduin,Durnehviir,Krosulhah,Kruziikrel,Mirmulnir,Naaslaarum,Nahagliiv,Odahviing,Paarthurnax,Relonikiv,Sahloknir,Sahrotaar,Viinturuth,Voslaarum,Vuljotnaak,Vulthuryol,Velehk,Drascua,Esmerelda,Ettiene,Fallaise,Isobel,Lucerne,Melka,Moira,Petra,Curalmil,Saerek,Halldir,Jyrik,Mikrul,Red Eagle,Sigdis,Gathrik,Hevnoraak,Krosis,Morokei,Nahkriin,Otar,Rahgot,Vokun,Volsung,Ahzdial,Dukaan,Miraak,Vahlok,Zahkriisos,Grok,Vyrthur,Gelebor,Ghost,Anger,Confidence,Self Doubt,Nightingale Sentinel,Captain Wayfinder,Captain Lonely-Gale,"

    ExceptionsINPC = ",Alassea,Among-The-Hist,Arghus,Brakh,Callen,Cassock,Daenlyn,Duraz,Erevan,Fa'nar,Fontaine,Froa,Gorr,Helsig,Hjoromir,Jade,Jilkmar,Lajjan,Larkspur,Lurgok,Meresine,Raynes,Skjarn,Tikrid,Vardath,Vaughn,Vawkes,Ylgyne,Amalee,Anum-La,Dagri'Lon,Qa'Dojo,Rumarin,Valgus,Zora,Ortheim,Amras,Lamki,Rosalind,Jorrvid,Bilbus,Jilkmar,Sixby,Kjen,Sven,Maren,Fig,Whig,Kyra,Jaysen,Mowgrol,Ises,Melea,Lyrik,Jerulith,Isobel,Thrice-Bitten,Fjona,Carmella,Lleyara,Glathriel,S'vashni,Olivia,Eleanor,Mugnor,Vigram,Kianna,Al'Hassan,Ingarte,Arelthor,Juryk,Iorel,Jaspar,Frik,Thorolf,Berg,Viranya,Moris,Sylphiron,Jeerah-Nur,Jolene,Lundvar,Clario,Beatrice,Urzub,Two-Tails,Okapi,Tellevi,Gerrod,Asteria,Yseld,Ynvar,Y'tharil,Mithril,Fironet,Orondil,Norgrof,Morndas,Nythriel,Veralene,K'avald,Yggleif,Edwayne,Salty-Throat,Halcar,Darcy,Gwyvane,Nitter,Gilsi,Orenius,Misha,Fjord,Zarlak,Cassius,Iria,Eldawyn,Eldar,Averna,Sonja,Zajhirra,Yarbrough,Ogdul,Leifan,Rongeir,Shelur,Morviah,Rinori,Wander-Lust,Haakon,Valla,Efram,Azzarian,Geel-Jah,Torveld,Fareloth,Fjona's,Gromash,Iris,Griffith,Morrigan,Tanita,Sadrin,Peragorn,Gnives,Olette,Dirassi,Yushari,Caylene,Amicus,Korrilan,Kjoli,Dar'Rakki,Hagravi,Fuki,Feldi,Vukeera,Batheru,Dravos,Ureni,Dro'bar,Offryn,Helsig,Felena,Vinnius,Flin,Puck,Elenthil,Marigoth,Bodan,Dalum-Ei,The Fair Lady,J'Sharr,Valenor,Ignatius,Arilwaen,Telthar,Hiram,Gabania,Joselyn,Theron,Aldi,Camaron,Alendril,Herran,Utei,Peracles,Blue-Throat,An-Ru,Asgeld,Foromir,Thorngar,Frygg,Hrongar,Hjolm,Falatild,Larsdon,Karras,Signus,Henna,Frenna,Gren,Helcyon,Uldvar,Olfrim,Anja,Brun,Cyrus,Sevrilla,Vardath,Ralothi,Fassius,Amsien,The Captain,Prompt-But-Bashful,Sergius,Gancielo,Lorne,Soren,Deneris,Varyan,Renmar,Deirdre,Hrokkar,Parellius,Nasrin,Krillo,Miros,Grenier,Roggstad,Bettany,Erwen,Sahlene,Godrod,Devran,Vartheim,Pelgrut,Benild,Florentus,Bette,Frodmar,Renni,Gormleif,Grotta,Ulrund,Lattimore,Merissa,Julius,Primei,Ester,Carah,Fenrald,Marla,Ignar,Valencia,Horker,Wolf,Skeever,Sweetroll,Ange,Adius,Aera,Afanna,Alammu,Alavva,Alina,Almo,Ambric,Ananril,Androlius,Arentius,Arenus,Aretino,Armion,Arnoth,Arilus,Astor,Atienne,Atul,Aubert,Aurelia,Avar,Avring,Bathes-In-Murky-Waters,Balius,Bentior,Benuni,Bjarni,Bolar,Botram,Brecius,Brenna,Byretus,Caius,Cadius,Cedus,Centos,Cerendil,Cornelius,Corvon,Crecius,Crextus,Crouches-In-Crevices,Dar'seel,Dar,taqto,Dar'zorben,Desilus,Drusus,Edwarin,Elintius,Encarnus,Eranir,Ereia,Erlus,Fadel,Falegus,Faleria,Famalius,Farsvar,Galarynn,Garceus,Garridan,Gavros,Gort,Gracius,Grecus,gro-Morrg,Grushnag,Gryfard,Gurnjar,Harald,Harsvar,Harthir,Hulgard,Hun-Wei,Ila,Ionalla,Iron-Heart,Ja'malgo,Jiriium,Jucani,Kalfimbria,Leo,Lone-Stead,Malumea,Marc,Marcin,Marcius,Mathias,Mirich,Neldam,Neremus,Nurenor,Odrevan,Olaf,Olgvir,Oreej-Tai,Ossi,Ovacca,Pinter,Precilius,Quinus,Raaslan,Ravil,Razzada,Reln-Tei,Reman,Renod,Rentin,Rignar,Rilja,Roland,Sellus,Senhilde,Serulius,Snipes-Many-Innocents,Servus,Silver-Plow,Simund,Sinja,Snorre,Stenar,Surilaron,Svenja,Tedral,Telenis,Terinius,Thorina,Ulchevan,Ulgruff,Ungveir,Urma,Vilenoldil,Yzzira,Zurikki,"

;/GENERIC NAME SEARCH TERMS/;
    GenericNameSearchTerms = ",Addict,Adept,Adventurer,Afflicted,Agent,Alchemist,Alik'r,Apprentice,Archer,Arch,Argonian,Ascendant,Assassin,Assistant,Bandit,Barbarian,Bard,Barkeep,Beggar,Bodyguard,Boy,Breton,Caitiff,Captain,Captive,Centurion,Challenger,Champion,Chef,Chief,Child,Churl,Citizen,Collector,College,Commander,Commoner,Companion,Conjurer,Corpse,Corsair,Courier,Cow,Cow Hand,Cryomancer,Cultist,Dawnguard,Deathlord,Default,Devotee,Dignitary,Dishonored,Dockmaster,Dockworker,Dragon,Dremora,Driver,Drunk,Electromancer,Elf,Envoy,Escaped,Farmer,Fan,Fellow,Field,Legate,First Mate,Fisherman,Fishwife,Forager,Forsworn,Fugitive,Furrier,Gambler,General,Gent,Giant,Girl,Gravedigger,Groundskeeper,Guardian,Guard,Hag,Hagraven,Headsman,Hero,Highwayman,Hunter,Imperial,Initiate,Invader,Jailor,Justiciar,Khajiit,Knight,Kynreeve,Kynval,Laborer,Labourer,Leader,Leper,Lookout,Looter,Lord,Lowlife,Lumberjack,Madwoman,Mage,Man,Marauder,Markynaz,Master,Mercenary,Merchant,Militia,Miner,Monk,Necromage,Necromancer,Noble,Nobleman,Noblewoman,Nord,Northwatch,Novice,Orc,Orphan,Outlaw,Overlord,Patron,Peddler,Penitus,Pilgrim,Pillager,Pirate,Plunderer,Poacher,Priest,Priestess,Prisoner,Prospector,Prostitute,Quartermaster,Ravager,Redguard,Refugee,Researcher,Reveler,Rogue,Sailor,Scavenger,Scholar,Scout,Scribe,Sellsword,Servant,Shaman,Shieldmaiden,Silver,Hand,Smuggler,Soldier,Spellsword,Spy,Stormcloak,Student,Thalmor,Thane,Thief,Thug,Torturer,Town,Crier,Trapper,Travel,Summerset,Shadow,Urchin,Vagrant,Valkynaz,Vampire,Veteran,Victim,Vigilant,Visiting,Wandering,Warden,Warlord,Warrior,Wayfinder,Wench,Witch,Wizard,Woman,Wood,Cutter,Worker,Worshipper"

    GenericNameSearchTermsLst1 = new string[127]
        GenericNameSearchTermsLst1[126] = "END OF LIST"

    GenericNameSearchTermsLst2 = new string[127]
        GenericNameSearchTermsLst2[126] = "END OF LIST"

    GenericNameSearchTermsLst3 = new string[127]
        GenericNameSearchTermsLst3[126] = "END OF LIST"

;/GENERIC BOSS SEARCH TERMS/;
    GenericBossSearchTerms = ",Champion of Boethiah,Bandit Chief,Master Vampire,Volkihar Master Vampire,Nightmaster Vampire,Forsworn Briarheart,Reaver Lord,Master Necromancer,Arch Necromancer,Master Conjurer,Arch Conjurer,Master Pyromancer,Arch Pyromancer,Arch Cryomancer,Arch Electromancer,Ritual Master,Northwatch Interrogator,First Mate,Hagraven,Draugr Overlord,Draugr Wight Lord,Draugr Scourge Lord,Draugr Death Overlord,Hagraven,Dragon Priest,Dremora Lord"

    GenericBossSearchTermsLst1 = new string[127]
        GenericBossSearchTermsLst1[126] = "END OF LIST"

    GenericBossSearchTermsLst2 = new string[127]
        GenericBossSearchTermsLst2[126] = "END OF LIST"

    GenericBossSearchTermsLst3 = new string[127]
        GenericBossSearchTermsLst3[126] = "END OF LIST"

;/BOSMER MALE FIRST NAMES/;
    BosmerMaleFirstNames = ",Sylch,Erasmo,Aengoth,Agarond,Allimir,Alveleg,Amring,Anglalos,Anruin,Arannir,Arathor,Baradras,Berengeval,Bolrin,Bragor,Brallion,Brithroth,Brolmir,Celegorn,Cingor,Clendil,Cun,Dangor,Denegor,Dirding,Dondir,Elberoth,Elegal,Eloroth,Endring,Engaer,Eradan,Erradan,Erval,Faldan,Fargoth,Faulgor,Fillin,Findulain,Foronir,Gaeldol,Gaenor,Galmir,Galthragoth,Gazalem,Gerrilgor,Glaum,Glonagoth,Godros,Gorchalas,Hingor,Meldor,Menelras,Minedhel,Minglos,Monthadan,Morth,Nalion,Nedhelas,Nedhelorn,Pegasai,Peragon,Rithrannir,Tarhiel,Thoromlallor,Thoronor,Tuundir,Ulwaen,Ungeleb,Neargoth,Aengvir,Amragor,Anglor,Anguilon,Athragar,Athrelor,Baenlin,Baeralorn,Brellin,Brithaur,Brodras,Bolwing,Caenlin,Caenlorn,Cingaer,Cingor,Daenlin,Dervenin,Engorm,Enilroth,Eridor,Erthor,Fimmion,Fithragaer,Foroch,Gelephor,Glarthir,Gloorolros,Gundalas,Gwinas,Hagaer,Hundolin,Maenlorn,Maglir,Nedhelfin,Niraegaer,Nordinor,Rindir,Thadon,Thaurron,Thoronir,Thrangirfin,Thurindil,Ungolim,Urunil,Uungor,Uuras,Eliras,Draeg,Fenton,Thorongil,Basth,Haymon,Mankar,Orben,Vanirion"

    BosmerMaleFirstNamesLst1 = new string[127]
        BosmerMaleFirstNamesLst1[126] = "END OF LIST"

    BosmerMaleFirstNamesLst2 = new string[127]
        BosmerMaleFirstNamesLst2[126] = "END OF LIST"

    BosmerMaleFirstNamesLst3 = new string[127]
        BosmerMaleFirstNamesLst3[126] = "END OF LIST"

;/BOSMER FEMALE FIRST NAMES/;
    BosmerFemaleFirstNames = ",Selene,Br'itsa,Falicia,Aerin,Aglaril,Anrel,Aradraen,Arangaer,Aranwen,Ardhil,Aredhel,Bauril,Baurin,Belwen,Berwen,Borwen,Celegil,Cirwedh,Cuunel,Distel,Dondreth,Dothiel,Dothruviel,Eindel,Elegnan,Elphiron,Emelin,Eraldil,Estinan,Falion,Fara,Filbeneth,Galbedir,Galdiir,Gelduin,Gildan,Giningil,Glathel,Hyna,Iingail,Indrel,Irwaen,Kirsty,Liette,Lorchel,Mara,Menelin,Milbereth,Nael,Natesse,Nathien,Nilioniel,Nona,Penglithil,Radras,Samia,Thaeril,Adanrel,Angrond,Alawen,Althel,Andragil,Anedhel,Boroneth,Bothiel,Carwen,Ciindil,Dagail,Dairihill,Degil,Dennilwen,Dredhwen,Elragail,Elsynia,Faurinthil,Ganredhel,Githriian,Hasathil,Huurwen,Idrolian,Laenafil,Laralthir,Mandil,Methredhel,Naarifin,Nilawen,Parwen,Syl,Telaendril,Thaedil,Thalfin,Thamriel,Thiirchel,Uurwen,Lywen,Gosellaneth,Serenarth,Allena,Fanriel,Kaalys"

    BosmerFemaleFirstNamesLst1 = new string[127]
        BosmerFemaleFirstNamesLst1[126] = "END OF LIST"

    BosmerFemaleFirstNamesLst2 = new string[127]
        BosmerFemaleFirstNamesLst2[126] = "END OF LIST"

    BosmerFemaleFirstNamesLst3 = new string[127]
        BosmerFemaleFirstNamesLst3[126] = "END OF LIST"

;/BOSMER FAMILY NAMES/;
    BosmerFamilyNames = ",Dorn'ke,Drurel,Garnona,Marvos,Ramothran,Vayn,Camoran,Balfix,Benoch"

    BosmerFamilyNamesLst1 = new string[127]
        BosmerFamilyNamesLst1[126] = "END OF LIST"

    BosmerFamilyNamesLst2 = new string[127]
        BosmerFamilyNamesLst2[126] = "END OF LIST"

    BosmerFamilyNamesLst3 = new string[127]
        BosmerFamilyNamesLst3[126] = "END OF LIST"

;/BRETON MALE FIRST NAMES/;
    BretonMaleFirstNames = ",Crendal,Alodie,Andre,Arnand,Astien,Barnand,Bereditte,Birard,Ciel,Cirges,Danders,Daric,Debentien,Detritus,Dilborn,Elbert,Ernand,Ernard,Faric,Fenas,Ferarilie,Francois,Frizkav,Frostien,Gaban,Geon,Geor,Idhdean,Inwold,Irbran,Jadier,Jeanciele,Jerian,Jocien,Joncis,Juillen,Lanie,Listien,Louis,Manis,Marcel,Mebestien,Merard,Merthierry,Myn,Noleon,Ormax,Orrent,Paur,Perien,Phane,Relien,Rerlas,Roberto,Ruran,Simine,Socucius,Thetrard,Varnis,Verick,Ylbert,Eduard,Gaston,Guilbert,Achille,Aelwin,Alberic,Aleron,Alix,Ambroise,Amiable,Ancus,Andre,Antoine,Astav,Baragon,Barthel,Baurion,Beran,Bradon,Brucetus,Christophe,Claude,Colin,Corrick,Damian,Davide,Didier,Edgar,Ernest,Eugal,Ferrum,Floyd,Francois,Gan,Gaspar,Gepard,Gerard,Grantham,Gregory,Hannibal,Hastrel,Istirus,Jakben,Jauffre,Jean-Pierre,Jhared,Jim,Kastus,Lazare,Lewin,Marc,Marcel,Mathieu,Matthias,Norbert,Ogier,Philip,Renald,Reynald,Roderic,Roland,Rowley,Salomon,Samuel,Sam,Traelius,Theranis,Thorley,Timothee,Ulrich,Vicente,Vincent,Alain,Amaund,Anton,Arniel,Clinton,Cynric,Dalan,Delvin,Emmanuel,Esmond,Etienne,Francois,Giraud,Glover,Herluin,Jouane,Louis,Maurice,Mercer,Moric,Nepos,Octieve,Phinis,Sam,Sebastian,Edgard,Gargrell,Riente,Taig,Vignac,Alain,Cortoran,Faolan,Gallivere,Gilbard,Gryf,Gyron,Hennabear,Horstle,Lywel,Sylon,Wergital,Wynster,Lucien"

    BretonMaleFirstNamesLst1 = new string[127]
        BretonMaleFirstNamesLst1[126] = "END OF LIST"

    BretonMaleFirstNamesLst2 = new string[127]
        BretonMaleFirstNamesLst2[126] = "END OF LIST"

    BretonMaleFirstNamesLst3 = new string[127]
        BretonMaleFirstNamesLst3[126] = "END OF LIST"

;/BRETON FEMALE FIRST NAMES/;
    BretonFemaleFirstNames = ",Abelle,Aditte,Ales,Ama,Amarie,Arbene,Arielle,Asciene,Aurane,Aurnie,Belene,Bovkinna,Cienne,Dabienne,Darene,Derelle,Edre,Edwinna,Emusette,Fasele,Fasile,Frelene,Gulitte,Helviane,Heniele,Janand,Jeanne,Jeberilie,Joslin,Leles,Lielle,Lirielle,Mabrelle,Malielle,Maline,Maranique,Marelle,Marielle,Marthe,Masalinie,Maurrie,Melie,Milie,Mitanne,Muriel,Pierlette,Piernette,Relie,Rianciene,Ronerelie,Sosia,Surane,Synette,Vienne,Virene,Marie,Evangeline,Adrienne,Alyssa,Annabelle,Antoinetta,Anya,Arielle,Beatrice,Bernadette,Bernice,Biene,Blanche,Callia,Candice,Caroline,Chana,Claudette,Elisa,Erline,Estelle,Francine,Irene,Isabeau,Jeanne,Jolie,Julienne,Luciana,Marguerite,Mariette,Matilde,Melisande,Mirabelle,Nanette,Raelynn,Rochelle,Roxanne,Roxy,Sabine,Phebe,Ursanne,Yvara,Zoe,Angeline,Carcette,Colette,Isabelle,Julienne,Laelette,Michel,Mirabelle,Nana,Sabine,Sorine,Sybille,Vivienne,Amelia,Annaig,Arzhela,Amelie,Cairine,Cayleigh,Edana,Gyna,Jyllia,Malvina,Muriel,Prolyssa,Riane,Rosalyn,Sorcha,Talara,Frieda"

    BretonFemaleFirstNamesLst1 = new string[127]
        BretonFemaleFirstNamesLst1[126] = "END OF LIST"

    BretonFemaleFirstNamesLst2 = new string[127]
        BretonFemaleFirstNamesLst2[126] = "END OF LIST"

    BretonFemaleFirstNamesLst3 = new string[127]
        BretonFemaleFirstNamesLst3[126] = "END OF LIST"

;/BRETON FAMILY NAMES/;
    BretonFamilyNames = ",Geles,Stoine,Acques,Adrard,Adrognese,Amedee,Ancois,Andre,Arthe,Auline,Aurelie,Aurilie,Aurmine,Beauchamp,Beluelle,Bielle,Bienne,Bierles,Birian,Bracques,Broles,Brutya,Caria,Caristiana,Charascel,Charien,Chriditte,Cienne,Dalomax,Desele,Dolbanitte,Elbert,Ence,Endre,Ephine,Erelie,Ergalla,Farr,Fralinie,Frenck,Frernis,Gemain,Genis,Geontene,Gernis,Geves,Gilelle,Hastien,Jastal,Jeannie,Jeline,Jes,Jodoin,Jolvanne,Julalanie,Kirbatha,Laelippe,Leoriane,Limax,Liric,Macile,Madach,Mannick,Marquardt,Masolaude,Masoriane,Maston,Matreinace,Maul,Maulinie,Maurard,Mene,Merian,Meric,Milielle,Mon,Moniel,Mornardl,Nermarc,Nestal,Nin,Oges,Panoit,Phiencel,Philulanie,Phirrienele,Rane,Retene,Rielle,Rirne,Riscel,Rolston,Rostorard,Sele,Sette,Sintieve,Stieve,Sylbenitte,Tailas,Thierry,Vanne,Vette,Viralaine,Virmaulese,Ysciele,Yvienne,Amelion,Bantien,Jemane,Lirran,Loche,Pierrane,Retiene,Surilie,Aethelred,Afranius,Alouette,Aric,Arne,Aumilie,Beanique,Belette,Bellamont,Berene,Bienne,Bincal,Blakeley,Branck,Brigette,Brolus,Canne,Channitte,Corgine,Denile,Diel,Don,Draconis,Eardwulf,Ernarde,Fanis,Fanriene,Festinius,Franc,Frasoric,Galena,Galien,Gene,Geonette,Georick,Gernand,Gulitte,Herrick,Imbel,Jeanard,Jend,Jenseric,Jurard,Labouche,LaRouche,Laul,Leland,Lelles,Lemonds,Lencolia,Litte,Luseph,Maborel,Magius,Malene,Manis,Marane,Maric,Marie,Mastien,Melie,Merowald,Metrick,Milvan,Mona,Monet,Montrose,Motierre,Nathans,Northwode,Ottus,Palielle,Peneles,Perrick,Petit,Renault,Renoit,Rielle,Selone,Stedrine,Stegine,Tilwald,Traven,Tussaud,Valtieri,Vautrine,Velain,Viernis,Wirich,Lylvieve,Mallory,Admand,Beaufort,Dufont,Endell,Ervine,Frey,Gane,Gemane,Gestor,Guevenne,Jondrelle,Jurard,Letrush,Lort,Lothaire,Manette,Marence,Merchad,Morrard,Motierre,Nytte,Onis,Rarnis,Rolaine,San,Sidrey,Stentor,Stroud,Tyne,Virane,Hoinart,Antieve,Sorick,Thenephan,Raze,Bontecou,Lariat,Larocque,Mallon,Montrose,Vardengroet,Lachance"

    BretonFamilyNamesLst1 = new string[127]
        BretonFamilyNamesLst1[126] = "END OF LIST"

    BretonFamilyNamesLst2 = new string[127]
        BretonFamilyNamesLst2[126] = "END OF LIST"

    BretonFamilyNamesLst3 = new string[127]
        BretonFamilyNamesLst3[126] = "END OF LIST"

;/DUNMER MALE FIRST NAMES/;
    DunmerMaleFirstNames = ",Dram,Ano,Galos,Anes,Arns,Avon,Avus,Balen,Balis,Breyns,Daris,Direr,Dranas,Draren,Drathas,Dravil,Dreyns,Enar,Eno,Ervis,Fedris,Fendel,Gaden,Galar,Galtis,Gilas,Gils,Ienas,Llaalam,Llerar,Manos,Novor,Ralen,Ralyn,Talis,Taren,Ulms,Vaval,Dranas,Fathis,Aldos,Alval,Andel,Aron,Arvin,Avrus,Aymar,Banus,Belmyne,Bolor,Bongond,Cylben,Delos,Diram,Dovyn,Drels,Eris,Eronor,Erver,Farwil,Felas,Felen,Ferul,Frathen,Gilen,Gureryne,Hlanas,Hloval,Ilvel,Kiliban,Kovan,Kylius,Mels,Modryn,Mondrar,Nilphas,Nivan,Olyn,Othrelos,Ralsa,Ralvas,Raynil,Redas,Rendil,Rythe,Slythe,Soris,Syndelius,Tovas,Ulen,Valen,Varel,Varon,Adril,Ambarys,Arvel,Aval,Belyn,Daynas,Dravin,Drevis,Drovas,Evul,Falas,Faryl,Feran,Fethis,Galdrus,Garan,Garyn,Geldis,Lleril,Malthyr,Malur,Malyn,Meden,Mithorpa,Modyn,Neris,Orini,Raleth,Ralis,Ravam,Ravyn,Revus,Revyn,Rirns,Romlyn,Sarthis,Savos,Servos,Sevan,Sondas,Talvas,Taron,Teldryn,Tolendos,Tythis,Ulves,Valin,Vals,Vanryth,Vendil,Veren,Wyndelius,Balir,Naris,Varlyn,Elhul,Hleryn,Ilver,Llynth,Malkur,Ozul,Rolves,Sul,Tnaval,Vuhon,Deros,Drallin,Grendis,Lyrin,Lythelus,Missun,Modyn,Moraelyn,Revus,Saldus,Taren,Ulvul,Vilur,Xiomara"

    DunmerMaleFirstNamesLst1 = new string[127]
        DunmerMaleFirstNamesLst1[126] = "END OF LIST"

    DunmerMaleFirstNamesLst2 = new string[127]
        DunmerMaleFirstNamesLst2[126] = "END OF LIST"

    DunmerMaleFirstNamesLst3 = new string[127]
        DunmerMaleFirstNamesLst3[126] = "END OF LIST"

;/DUNMER FEMALE FIRST NAMES/;
    DunmerFemaleFirstNames = ",Karethys,Mehra,Daynasa,Daynila,Felara,Galsa,Mivanu,Thelama,Velsa,Verasa,Adosi,Alves,Arvena,Boderi,Bralsa,Davela,Dervera,Dovesi,Dralora,Drarana,Dredena,Falanu,Gadeneri,Glistel,Llathasa,Llensi,Llevana,Malyani,Melisi,Mensa,Mirili,Mivryna,Myvryna,Nardhil,Norasa,Noveni,Ralsa,Relmyna,Satha,Saveri,Tadrose,Tanasa,Tilse,Tivela,Tolisi,Tolvasa,Ulene,Undena,Uravasa,Urnsi,Aduri,Aphia,Aranea,Avrusa,Bralsa,Brelyna,Cindiri,Dinya,Dravynea,Dreyla,Elvali,Elynea,Idesa,Ildari,Marise,Medresi,Merilar,Milore,Mirri,Niluva,Selveni,Suvaris,Synda,Tilisu,Varona,Voldsea,Laaneth,Aramea,Bathrone,Belderi,Nirai,Urili,Anara,Drelliane,Iachilla,Inzoliah,Katariah,Neria,Peliah,Sendrasa,Turala"

    DunmerFemaleFirstNamesLst1 = new string[127]
        DunmerFemaleFirstNamesLst1[126] = "END OF LIST"

    DunmerFemaleFirstNamesLst2 = new string[127]
        DunmerFemaleFirstNamesLst2[126] = "END OF LIST"

    DunmerFemaleFirstNamesLst3 = new string[127]
        DunmerFemaleFirstNamesLst3[126] = "END OF LIST"

;/DUNMER FAMILY NAMES/;
    DunmerFamilyNames = ",Andrano,Andrethi,Hlaalu,Ienith,Arvel,Dralor,Llervu,Sadri,Saren,Andas,Dren,Faryon,Girith,Hleran,Indaram,Aryon,Indaren,Indrano,Sarano,Seran,Andarys,Andus,Arenim,Athren,Farano,Fyr,Gilnith,Hlaren,Llethri,Maryon,Omayn,Relas,Salas,Saryon,Uvayn,Vendu,Andaram,Aralas,Arelas,Arobar,Helas,Hlervu,Llaren,Lleran,Maren,Nerethi,Omoran,ORan,Othravel,Othril,Rathryon,Rethan,Retheran,Salvani,Saram,Sarethi,Seloth,Senim,Teran,Tharen,Ules,Velas,Venim,Veran,Verethi,Virith,Adas,Adren,Alen,Andalas,Andalen,Andoril,Andules,Arano,Aravel,Aren,Arethan,Arethi,Beleth,Berano,Dral,Dralas,Drelas,Drenim,Drethan,Drothan,Helvi,Henim,Ieneth,Indalas,Irano,Llaram,Llarys,Llenim,Llethan,Lloran,Mandas,Nethan,Omalen,Omavel,Oren,Orethi,Oril,Othan,Othran,Othren,Radas,Ralen,Rendas,Sadus,Sarendas,Sarys,Seleth,Serethi,Tedas,Tenim,Thelas,Thirandus,Thirano,Uvaren,Uvenim,Uveran,Varen,Varis,Alver,Andaren,Andavel,Andoren,Andrelo,Aralen,Areleth,Arinith,Arothan,Athram,Athrys,Balen,Belas,Berendas,Berethi,Dalen,Darethi,Daryon,Darys,Dran,Drom,Falas,Falen,Faram,Faren,Farethi,Farys,Favelnim,Gilvayn,Heleran,Heran,Hlaalo,Hlan,Hlas,Hlen,Hler,Hloran,Indalen,Indarys,Llendu,Marvel,Marys,Moren,Morvayn,Mothryon,Nelas,Nerano,Nirith,Norvayn,Norvayne,Omalas,Omalor,Omoril,Omothan,Othralen,Othrelas,Othreleth,Othreloth,Othrenim,Ralas,Raledran,Ravel,Releth,Romoren,Romori,Rothandus,Samori,Sarandas,Sarothil,Savel,Savil,Sedas,Selaren,Selas,Selvilo,Sendas,Sethan,Telas,Thendas,Tures,Ulven,Uvelas,Uvirith,Varam,Vari,Varyon,Varys,Vedas,Vedran,Veleth,Verano,Vules,Andalor,Baram,Drath,Dreloth,Dres,Drinith,Ginith,Omaren,Randas,Redas,Reloth,Rothan,Sadryon,Sandus,Serano,Thalas,Thiralas,Veloth,Alas,Aran,Arys,Beran,Dareleth,Dulo,Fadathram,Favel,Gimothran,Helan,Hlervi,Llando,Marvani,Raviro,Redoran,Rethandus,Salothan,Salothran,Salvel,Samarys,Sandas,Telvayn,Tharys,Vandus,Verelnim,Dalvilu,Ulen,Telvanni,Sathil,Indoril,Sadras"

    DunmerFamilyNamesLst1 = new string[127]
        DunmerFamilyNamesLst1[126] = "END OF LIST"

    DunmerFamilyNamesLst2 = new string[127]
        DunmerFamilyNamesLst2[126] = "END OF LIST"

    DunmerFamilyNamesLst3 = new string[127]
        DunmerFamilyNamesLst3[126] = "END OF LIST"

;/ASHLANDER MALE FIRST NAMES/;
    AshlanderMaleFirstNames = ",Mausur,Ainab,Maeonius,Manat,Missun,Mubdan,Odairan,Zallit,Ababael,Abassel,Abibaal,Abishpulu,Adairan,Adaishah,Addammus,Addarnat,Adibael,Ahasour,Ahaz,Ainab,Ainat,Anasour,Anit,Asha-Ammu,Ashibaal,Ashu-Ahhe,Ashuma-Nud,Ashur-Dan,Assaba-Bentus,Assallit,Assamanut,Assamma-Idan,Assantus,Assatlit,Assemmus,Assurdan,Balur,Beden,Conoon,Dakin,Dralas,Dun-Ilu,Dunsalipal,Dutadalk,Elibael,Emul-Ran,Erur-Dan,Esar-Don,Esib-Nummu,Hainab,Hairan,Han-Ammu,Hannat,Hassour,Hemus,Hentus,Hibaddar,Hinammu,Hirarend,Ibarnadad,Ibasour,Ilasour,Kanat,Kanit,Kanud,Kashtes,Kaushad,Kausi,Kind,Kitbael,Kummi-Namus,Kund,Lassour,Mabarrabael,Maesat,Mal,Malay,Mansilamat,Massarapal,Massour,Mibdinahaz,Minassour,Minisun,Minnibi,Musan,Mut,Nind,Ninimilk,Ninirrasour,Nirait,Nund,Odaishah,Ohibaal,Patababi,Patus,Pilun,Punibi,Ranabi,Sakin,Sakulerib,Sal,Salattanat,Salay,Salmat,Salmus,Sannit,Sargon,Seldus,Selk,Selkirnemus,Shabinbael,Shanat,Shanit,Shannat,Shanud,Shargon,Shilipuran,Shimsun,Shin,Shipal,Shirerib,Shullay,Subenend,Sul-Matuul,Talkin,Teshmus,Tibdan,Tinti,Tis,Truan,Tubilalk,Tussurradad,Ulabael,Ulath-Pal,Yahaz,Yakin,Yakum,Yan,Yanit,Yantus,Yapal,Yassour,Yeherradad,Yen,Yenammu,Zabamund,Zabarbael,Zainat,Zairan,Zalabelk,Zalit,Zallay,Zanat,Zanmulk,Zebba,Zebdusipal,Zelaku,Zelay,Zenabi,Zennammu,Thanethen"

    AshlanderMaleFirstNamesLst1 = new string[127]
        AshlanderMaleFirstNamesLst1[126] = "END OF LIST"

    AshlanderMaleFirstNamesLst2 = new string[127]
        AshlanderMaleFirstNamesLst2[126] = "END OF LIST"

    AshlanderMaleFirstNamesLst3 = new string[127]
        AshlanderMaleFirstNamesLst3[126] = "END OF LIST"

;/ASHLANDER FEMALE FIRST NAMES/;
    AshlanderFemaleFirstNames = ",Mamaea,Manirai,Manu,Nummu,Shara,Zebba,Addut-Lamanu,Adusamsi,Ahanabi,Ansu,Antu,Ashamanu,Asharapli,Ashumanu,Assamanu,Assi,Assimusa,Assirari,Assumanu,Benudni,Berapli,Dissu,Elitlaya,Elumabi,Hainab,Hairan,Hanarai,Hannabi,Hansi,Hinummu,Ibanammu,Kammu,Kausha,Kummu,Kurapli,Kushishi,Lanabi,Maela,Maeli,Mamusa,Manabi,Man-Ilu,Mi-Ilu,Midnabi,Milynea,Mimanu,Minabibi,Missamsi,Munbebi,Musa,Nammu,Nibani,Nirai,Peakstar,Pilu,Raishi,Rasamsi,Rawia,Sakiran,Samsi,Santinti,Seba,Sen,Senipu,Shali,Shallath-Piremus,Shinammu,Shishi,Shulki,Shullay,Sinnammu,Sonummu,Tashpi,Tissamsi,Tunipy,Tussi,Ulibabi,Ulisamsi,Urshamusa,Vabbar,Yan-Ahhe,Yanabani,Yanibi,Yenabi,Zaba,Zabi,Zanummu,Zennammu,Zula"

    AshlanderFemaleFirstNamesLst1 = new string[127]
        AshlanderFemaleFirstNamesLst1[126] = "END OF LIST"

    AshlanderFemaleFirstNamesLst2 = new string[127]
        AshlanderFemaleFirstNamesLst2[126] = "END OF LIST"

    AshlanderFemaleFirstNamesLst3 = new string[127]
        AshlanderFemaleFirstNamesLst3[126] = "END OF LIST"

;/ASHLANDER FAMILY NAMES/;
    AshlanderFamilyNames = ",Vabdas,Almu,Man-llu,Mantiti,Yahaz,Zainsubani,Zama-Rasour,Ababael,Abalkala,Abelmawia,Addarari,Addaribantes,Addinibi,Addunipu,Adidshina,Adur-Dan,Ahalkalun,Ahanidiran,Akin,Anurnudai,Ashalkimallit,Ashananapal,Ashapaladdon,Asharapli,Ashar-Dad,Ashar-Don,Asharnalit,Ashibael,Ashi-Iddan,Ashirbibi,Ashishpalirdan,Ashumallit,Ashummi-Ammus,Ashunbabi,Ashun-Idantus,Ashurnasaddas,Ashushshirari,Assalatammis,Assannabalit,Assantinalit,Assaplit,Assarbeberib,Assardarainat,Assardidairan,Assarnibani,Assarnuridan,Assarrapanat,Assarrimisun,Assattadaishah,Assebiriddan,Assemmus,Asserbassalit,Asserrumusa,Assillariran,Assinabi,Assintashiran,Assirnarari,Assonirishpal,Assudiraplit,Assudnilamat,Assullinbanud,Assumanallit,Assunbahanammu,Assunudadnud,Assurnarairan,Assurnipilu,Assurnumanit,Assurnumausur,Assutladainab,Assutlanipal,Atinsabia,Bael,Benamamat,Chodala,Darirnaddunumm,Dinadad,Dudnebisun,Dun-Ahhe,Dunsamsi,Elarrapal,Enturnabaelul,Eraishah,Eramarellaku,Erarbadon,Erushara,Esatliballit,Esurarnat,Esurnadarpal,Giladren,Gilu,Hainnabibi,Hainterari,Hairshashishi,Hairshumusa,Hansar,Ilabael,Ilath-Pal,Ilurnubishpal,Kaushad,Kaushmamanu,Kaushminipu,Kil,Kummimmidan,Kuntarnammu,Kutebani,Lalansour,Lasamsi,Laserbillahemmi,Maesa,Maessabibi,Maliiran,Malman-Ammu,Mannanalit,Mantashpi,Massitisun,Matluberib,Mibishanit,Mirathrernenum,Mirpal,Mirshamammu,Muddumummu,Nerendus,Niladon,Odimabesser,Odin-Ahhe,Odirnapal,Pansamsi,Pudashara,Puntumisun,Radansour,Ranarbatus,Rapli,Saddarnuran,Saharnatturapli,Saladnius,Salkatanat,Salvu,Samma,Sammalamus,Samsi,Sanammasour,Santumatus,Sehabani,Selitbael,Selkin-Adda,Senatbael,Serdimapal,Serimilk,Sershurrapal,Shalarnetus,Shamirbasour,Shand,Shashipal,Shashmassamsi,Shilansour,Shimmabadas,Shin-Ilu,Shinirbael,Shishara,Siddurnanit,Sobbinisun,Sobdishapal,Sonnerralit,Subaddamael,Tansumiran,Tibashipal,Timmiriran,Timsar-Dadisun,Ududnabia,Ulannanit,Ularshanentus,Ulirbabi,Urshan-Adairan,Urshummarnamus,Urshumusa,Varnan-Adda,Yansurnummu,Yanumibaal,Yasalmibaal,Yassabisun,Yentimilal,Yessur-Disadon,Zabamat,Zabynatus,Zainab,Zaintashara,Zansatanit,Zeba-Adad,Zelma-Alit,Zenammu,Rotheloth"

    AshlanderFamilyNamesLst1 = new string[127]
        AshlanderFamilyNamesLst1[126] = "END OF LIST"

    AshlanderFamilyNamesLst2 = new string[127]
        AshlanderFamilyNamesLst2[126] = "END OF LIST"

    AshlanderFamilyNamesLst3 = new string[127]
        AshlanderFamilyNamesLst3[126] = "END OF LIST"

;/ARGONIAN MALE NAMES/;
    ArgonianMaleFirstNames = ",Dreekius,Asum,Bunish,Busheeus,Chalureel,Chiwish,Chulz,Chuna,Haran,Hathei,Heedul,Huleeya,Huzei,Inee,Itan,Meer,Milos,Neetinei,Okaw,Peeradeeh,Rasha,Reemukeeus,Reesa,Seewul,Skeetul,Tanan,Teegla,Tul,Ukawei,Ula,Utadeek,Weeltul,Weer,An-Zaw,Bun-Teemeeta,Dan-Ru,Effe-Tei,Eleedal-Lei,Gah Julan,Gam-Kur,Geel-Lah,Haj-Ei,Han-Tulm,Heem-La,Heir-Zish,Im-Kilaya,Jeelus-Tei,Jeer-Maht,Ram-Dar,Junal-Lei,Keerasa-Tan,Miun-Gei,Mush-Mere,Okan-Shei,Oleen-Gei,Olink-Nur,Reeh-Jah,Silm-Dar,Tee-Lan,Tim-Jush,Vistha-Kai,Wanan-Dum,Wih-Eius,Wud-Neeus,Wuleen-Shei,Also-He-Washes,Basks-In-The-Sun,Big Head,Dreaded-Water,Fine-Mouth,Grey-Throat,Hides-His-Eyes,Hides-His-Foot,High-Heart,Morning-Star-Steals-Away-Clouds,Nelix Fly-Breath,Nine-Toes,Only-He-Stands-There,Skink-in-Tree's-Shade,Smart-Snake,Smokeskin-Killer,Stream-Murk,Swims-In-Swells,Ten-Tongues,Weerhat,Tongue-Toad,Twice-Bitten,Wind-In-His-Hair,Sissithik,Yelnicin,Amusei,Deeh,Geel,Jeelius,Mahei,Otumeel,Pajeen,Reenum,Sakeepa,Saliith,Teekeeus,Teinaava,Usheeja,Wumeek,Yinz'r,Ah-Malz,Ajum-Kajin,Beem-Kiurz,Dar Jee,Deetum-Ja,Dreet-Lai,Er-Teeus,Geem Jasaiin,Gin-Wulm,Jee-Tah,Jeetum-Ze,Oleed-Ei,Pad-Ei,Tun-Zeeus,Weebam-Na,Big Head,Cat-Face,Grey-Throat,Hauls-Ropes-Faster,Hears-Voices-In-The-Air,Hides-His-Heart,Hunting Tail,Raven Biter,Right-Wind,Scar-Tail,Sings-Like-Thunder,Tooth-in-the-Sea,Swims-at-Night,Wuthilul,Ixtah-Nasha,Meeleeh-Een,Mere-Glim,Xhon-Mehl,Footfalls-in-Snow,Lonely-Spines,Binyaar,Deerkaza,Jaraleet,Juunei,Nomu,Shehs,Schiavas,Tsleeixth,Ullis,Croon-Tail,Dives-From-Below,Drawing-Flame,Furl-Of-Fresh-Leaves,Gash-Tail,Right-Foot-Rock,Scale-Song"

    ArgonianMaleFirstNamesLst1 = new string[127]
        ArgonianMaleFirstNamesLst1[126] = "END OF LIST"

    ArgonianMaleFirstNamesLst2 = new string[127]
        ArgonianMaleFirstNamesLst2[126] = "END OF LIST"

    ArgonianMaleFirstNamesLst3 = new string[127]
        ArgonianMaleFirstNamesLst3[126] = "END OF LIST"

;/ARGONIAN FEMALE NAMES/;
    ArgonianFemaleFirstNames = ",Ahaht,Akish,Banalz,Beekatan,Eutei,Gilm,Gish,Hul,Kasa,Milah,Nakuma,Neesha,Nuralg,Nush,Okur,Onasha,Shatalg,Tasha,Wusha,Ah-Meesei,Am-Ra,An-Deesei,Chanil-Lee,Cheesh-Meeus,Deesh-Meeus,El-Lurasha,Ereel-Lei,Gih-Ja,Jeed-Ei,Kal-Ma,Keel-Raniur,Meeh-Mei,Meen-Sa,Mim-Jeen,Muz-Ra,Nam-La,Olank-Neeus,On-Wan,On-Wazei,Seen-Rei,Breech-Star,Snail-Tail,Tern-Feather,Travelling-New-Woman,Beelei,Beewos,Bejeen,Deetsan,Druja,Marz,Numeen,Ocheeva,Pasha,Rana,Shaleez,Skaleel,Witseidutsei,Bur-Meema,Dar-Ma,Hal-Liurz,Kud-Ei,Mach-Na,Seed-Neeus,Sheer Meedish,Tar-Meena,Weedum-Ja,City-Swimmer,On-Staya,Quill-Weave,Runs-in-Circles,Tall-Trees-Falling,Wide-Eye,Sails-Through-Storms,Sees-All-Colors,Heita-Meen,Awas,Sanax,Sigerthe,Zollassa,Seif-ij Hidja,Xil-Go,Lifts-Her-Tail,Makes-Many-Waves"

    ArgonianFemaleFirstNamesLst1 = new string[127]
        ArgonianFemaleFirstNamesLst1[126] = "END OF LIST"

    ArgonianFemaleFirstNamesLst2 = new string[127]
        ArgonianFemaleFirstNamesLst2[126] = "END OF LIST"

    ArgonianFemaleFirstNamesLst3 = new string[127]
        ArgonianFemaleFirstNamesLst3[126] = "END OF LIST"

;/ARGONIAN FAMILY NAMES/;
    ArgonianFamilyNames = ",Caith,Caleel,Hajiin,Hejleel,Ixtaxh,Jasaiin,Jee,Jeke,Jekiul,Julan,Juleel,Kajmeht,Lukiul,Meedish,Rajith,Rormasu,Shapmasu,Teeba,Theiliin,Thonashap,Thotaiin,Thuxleel,Waith,Waleel,Wamasu,Wasaxh,Xajhuthi,Xaleel"

    ArgonianFamilyNamesLst1 = new string[127]
        ArgonianFamilyNamesLst1[126] = "END OF LIST"

    ArgonianFamilyNamesLst2 = new string[127]
        ArgonianFamilyNamesLst2[126] = "END OF LIST"

    ArgonianFamilyNamesLst3 = new string[127]
        ArgonianFamilyNamesLst3[126] = "END OF LIST"

;/ALTMER MALE FIRST NAMES/;
    AltmerMaleFirstNames = ",Gerrick,Aldaril,Anarenen,Andil,Angoril,Armion,Aronil,Arrille,Athellor,Carecalmo,Caryarel,Earmil,Eraamion,Erundil,Fainertil,Falanaamo,Fanildil,Fiiriel,Gladroon,Hecerinde,Hyarnarenquar,Ilmiril,Inganar,Iroroon,Itermerel,Kardryn,Landorume,Meanen,Meryaran,Mollimo,Moranarg,Mororurg,Mossanon,Nande,Nelacar,Norionil,Olquar,Qorwynn,Rimintil,Rumare,Sanyon,Seanwen,Sinyaramen,Sorcalin,Tauryon,Telinturco,Tragrim,Tunengore,Tusamircil,Tyermaillin,Umbacano,Undil,Uulernil,Volanaro,Yakov,Yanniss,Yarnar,Ancotar,Angalmo,Aranath,Areldil,Areldur,Arkved,Arterion,Calindil,Carandial,Celedaen,Cirion,Earil,Eldamil,Elidor,Enrion,Errandil,Faelian,Falcar,Gilgondorin,Halion,Henantier,Hindaril,Hirtel,Honditar,Lithnilian,Mankar,Merandil,Merildor,Ocato,Ohtimbar,Orintur,Ormil,Raven,Salmo,Seridur,Sinderion,Suurootan,Tilmo,Tumindil,Umbacano,Ungarion,Volanaro,Voranil,Lorcalin,Naris,Valmir,Arenthil,Aronil,Falandamil,Falarel,Naemon,Telenger,Vanus,Aiden,Asliel,Beredalmo,Corvus,Fenlil,Gyrnasse,Henantier,Hidellith,Iachesis,Jovron,Kael,Lylim,Malvasian,Nuulion,Pelladil,Ryain,Trechtus,Vorian"

    AltmerMaleFirstNamesLst1 = new string[127]
        AltmerMaleFirstNamesLst1[126] = "END OF LIST"

    AltmerMaleFirstNamesLst2 = new string[127]
        AltmerMaleFirstNamesLst2[126] = "END OF LIST"

    AltmerMaleFirstNamesLst3 = new string[127]
        AltmerMaleFirstNamesLst3[126] = "END OF LIST"

;/ALTMER FEMALE FIRST NAMES/;
    AltmerFemaleFirstNames = ",Medora,Anirne,Ardarume,Calmaninde,Camandil,Celria,Ciralinde,Culumaire,Cumanya,Dhaunayne,Elanande,Elante,Eldafire,Elenwen,Erissare,Erranil,Estalenya,Estirdalin,Estoril,Faire,Fistelle,Helende,Hession,Iirenoore,Imare,Iniel,Irinwe,Lorurmend,Mirkrand,Nalcarya,Siltalaure,Sirilonwe,Sondaale,Taarie,Tarerane,Tenyeminwe,Termanwe,Viraninde,Alwaen,Angalsama,Aranalda,Ardaline,Ariente,Arquen,Aryarie,Astante,Atraena,Aurelinwae,Caminalda,Carahil,Caranya,Cindanwe,Earana,Eilonwy,Elante,Erissare,Hlidara,Karinnarre,Mirie,Muurine,Nerussa,Ohtesse,Palonirya,Ranaline,Ruma,Sontaire,Tandilwe,Termanwe,Varulae,Kianra,Ayrenn,Estre,Silhansa,Tuinden,Arannelya,Camilonwe,Gialene,Tuinden"

    AltmerFemaleFirstNamesLst1 = new string[127]
        AltmerFemaleFirstNamesLst1[126] = "END OF LIST"

    AltmerFemaleFirstNamesLst2 = new string[127]
        AltmerFemaleFirstNamesLst2[126] = "END OF LIST"

    AltmerFemaleFirstNamesLst3 = new string[127]
        AltmerFemaleFirstNamesLst3[126] = "END OF LIST"

;/ALTMER FAMILY NAMES/;
    AltmerFamilyNames = ",Direnni,Aundae,Camoran,Mothril,Rilis,Aliaria,Ateldil,Duure,Galerion,Lloderion,Elsinor,Wethrin"

    AltmerFamilyNamesLst1 = new string[127]
        AltmerFamilyNamesLst1[126] = "END OF LIST"

    AltmerFamilyNamesLst2 = new string[127]
        AltmerFamilyNamesLst2[126] = "END OF LIST"

    AltmerFamilyNamesLst3 = new string[127]
        AltmerFamilyNamesLst3[126] = "END OF LIST"

;/REDGUARD MALE FIRST NAMES/;
    RedguardMaleFirstNames = ",Camaron,Greklith,K'avar,Lhotun,Provlith,Thaik,Vhosek,A'tor,Avik,Basil,Brennan,Coyle,Cyrus,Duadeen,Favis,Hakan,Hayle,Jaganvir,Kotaro,Lakene,Maiko,Prnell,Trithik,Vander,Voa,Achel,Alusaron,Bodean,Camas,Chark,Chrelm,Cluson,Daracy,Daron,Din,Dinok,Domalen,Dortodd,Dudley,Ennah,Farmin,Frink,Gancolm,Garry,Glannison,Gor,Hasell,Hickim,Hinald,Ian,Ilden,Iminda,Jart,Jathlanie,Jeleen,Jelin,Jim,Jine,Jon,Jonis,Kalorter,Karrod,Kaye,Lanald,Lesley,Lidebras,Nachael,Nannithon,Nistacey,Penald,Raccan,Raig,Rinina,Rissinia,Sader,Sason,Shenk,Silasson,Sorian,Soscean,Stargel,Stenet,Sterdecan,Sticey,Therdon,Torline,Tralan,Trasteve,Vilamon,Waylas,Wayn,Alonzo,Amir,Armand,Athon,Azani,Azzan,Baurus,Boldon,Borissean,Casimir,Casnar,Cirroc,Curtis,Cyrus,Delmar,Demetrius,Dorian,Ernest,Gogan,Haekwon,Hillod,Isleif,Jair,Jalbert,Juncan,Kellen,Kelvyn,Kewan,Khafiz,Kinther,Kithlan,Lathon,Limark,Neville,Orinthal,Orrin,Owyn,Phintias,Pranal,Pyke,Relan,Rellian,Rhano,Roderick,Ruslan,Shamar,Shelden,Stanet,Talan,Taran,Thedret,Trayvond,Varnado,Wallace,Wilbur,Willet,Winson,Avidius,Azzada,Baral,Fahara'jad,Namasur,Uwafa,Wisr,Aaban,Abdul-Mujib,Afa-Saryat,Akamon,Arthago,Cirroc,Derik,Destri,Divad,Frandar,Hallin,Hubalajad,Kalam,Mamireh,Abah,Boshek,Hasar,Huzar,Rada,Rajmahar,Razul,Rok'dun,Roshadon,Sameq,Thassad,Volag,Yaghoub,Zakhin"

    RedguardMaleFirstNamesLst1 = new string[127]
        RedguardMaleFirstNamesLst1[126] = "END OF LIST"

    RedguardMaleFirstNamesLst2 = new string[127]
        RedguardMaleFirstNamesLst2[126] = "END OF LIST"

    RedguardMaleFirstNamesLst3 = new string[127]
        RedguardMaleFirstNamesLst3[126] = "END OF LIST"

;/REDGUARD FEMALE FIRST NAMES/;
    RedguardFemaleFirstNames = ",Akorithi,Aubk-i,Charvek-si,Dh'emka,Whitka,Iszara,Krisandra,Mariah,Saban,Siona,Yaeli,Alusannah,Ancola,Anora,Balan,Caminda,Chalone,Dandsa,Davina,Domba,Dulian,Elone,Ena,Enjine,Erla,Ery,Gwen,Hussonia,Jaline,Jamie,Jeanne,Jolda,Kanet,Karlirah,Kati,Katie,Kirginia,Laire,Landa,Lette,Malexa,Manicky,Margonet,Miniel,Mirisa,Monsey,Nanine,Neminda,Pemenie,Relinda,Rithleen,Senyndie,Shardie,Sinia,Sulesa,Tiela,Todwendy,Tanyin,Angelie,Ashanta,Branwen,Carmen,Chanel,Clesa,Dahlia,Dhola,Eletta,Isolde,Jearl,Kiara,Lashana,Maelona,Minerva,Mirisa,Neesha,Oleta,Rasheda,Rhiannon,Rienna,Rohssan,Rona,Shafaye,Shameer,Shelley,Sherina,Tamika,Tavia,Tenville,Tierra,Trevaia,Lu'ah,Rochelle,Taviah,Barahar,Delia,Hecua,Lakana,Radhasa,Sariah,Azadiyeh,Cegila,Corda,Haballa,Hafseta,Julia,Kati,Makela,Memyireh,Patia,Perizada,Rijja,Zell"

    RedguardFemaleFirstNamesLst1 = new string[127]
        RedguardFemaleFirstNamesLst1[126] = "END OF LIST"

    RedguardFemaleFirstNamesLst2 = new string[127]
        RedguardFemaleFirstNamesLst2[126] = "END OF LIST"

    RedguardFemaleFirstNamesLst3 = new string[127]
        RedguardFemaleFirstNamesLst3[126] = "END OF LIST"

;/REDGUARD FAMILY NAMES/;
    RedguardFamilyNames = ",K'Elmar,Mirel,Ruuz,Aldwyr,Litte,al-Skaven,al-Bergama,al-Satakalaam,al-Sentinel,al-Kozanset,al-Tarin,af-Perah,al-Ragath,al-Rihad,at-Owynok,at-Tura,af-Jahannif,af-Jamadin,af-Nimr,af-Ozalan,at-Amil,at-Areshu,at-Ariya,at-Aswala,at-Elett,at-Lehiel,at-Phintos,at-Ramdud,at-Toura,al-Thir,at-Hamisham,al-Saran,af-Ahmus,af-Angilia,af-Armin,af-Ashora,af-Bazra,af-Davood,af-Dometri,af-Dushana,af-Ebdoh,af-Ganet,af-Ghada,af-Guyeline,af-Haba,af-Hafza,af-Hazifah,af-Hisham,af-Islah,af-Jahi,af-Kardyn,af-Karra,af-Katayoun,af-Kimiya,af-Mahala,af-Mubdi,af-Naifa,af-Nazila,af-Owlin,af-Qerik,af-Ramand,af-Rasha,af-Tasa,af-Whyrdh,al-Antiphyllos,al-Azif,al-Belkarth,al-Glessa,al-Hallen,al-Hallin,al-Hallin's,al-Hallins,al-Hareem,al-Hllins,al-Masri,al-Miran,al-Morwha,al-Myrkwasa,al-Natedan,al-Ojwambu,al-Tahud,al-Tava,Alielle,at-Abbas,at-Ajwan,at-Amjad,at-Baqar,at-Dolmir,at-Dorcolm,at-Ebdoh,at-Fada,at-Fara,at-Gamati,at-Ganet,at-Gar,at-Ginal,at-Glina,at-Halma,at-Hassef,at-Hawraz,at-Hollus,at-Housel,at-Jafra,at-Jehat,at-Kalan,at-Karim,at-Khorajah,at-Makela,at-Mardeen,at-Marimah,at-Morad,at-Naruk,at-Nazdar,at-Nazim,at-Ordzan,at-Pawan,at-Pykel,at-Renazh,at-Rusa,at-Sadil,at-Sal,at-Salan,at-Sentinel,at-Shadal,at-Tamina,at-Wardiya,at-Warhpal,at-Werzhan,at-Wildur,at-Yashar"

    RedguardFamilyNamesLst1 = new string[127]
        RedguardFamilyNamesLst1[126] = "END OF LIST"

    RedguardFamilyNamesLst2 = new string[127]
        RedguardFamilyNamesLst2[126] = "END OF LIST"

    RedguardFamilyNamesLst3 = new string[127]
        RedguardFamilyNamesLst3[126] = "END OF LIST"

;/ORC MALE FIRST NAMES/;
    OrcMaleFirstNames = ",Moghakh,Atulg,Azuk,Bagamul,Bakh,Baronk,Bashag,Bazgulub,Bogakh,Bologra,Borug,Both,Bugdul,Bugharz,Bugrash,Bugrol,Bumbub,Burul,Dular,Duluk,Duma,Dumbuk,Dumburz,Dur,Durbul,Durgash,Durz,Durzol,Durzub,Durzum,Garothmuk,Garzonk,Gashna,Ghamborz,Ghamonk,Ghoragdush,Ghorlorz,Glush,Grat,Gruzgob,Guarg,Gurak,Khadba,Khagra,Khargol,Koffutto,Largakh,Lorbumol,Lorzub,Lugdum,Lugrub,Lurog,Mash,Matuk,Mauhul,Mazorn,Mol,Morbash,Mug,Mugdul,Muk,Murag,Murkub,Murzol,Muzgonk,Nag,Nar,Nash,Ogrul,Ogrumbu,Olfin,Olumba,Orakh,Rogdul,Shakh,Shamar,Shamob,Shargam,Sharkub,Shat,Shazgob,Shulong,Shura,Shurkul,Shuzug,Snaglak,Snakha,Snat,Ugdumph,Ughash,Ulam,Umug,Uram,Urim,Urul,Urzog,Ushamph,Ushat,Yadba,Yagak,Yak,Yam,Yambagorn,Yambul,Yargol,Yashnarz,Yatur,Makor,Olpac,Agronak,Bat,Bazur,Brugo,Bogrum,Brag,Brokil,Bugak,Buramog,Burz,Dubok,Dul,Dulfish,Dulphumph,Dumag,Gaturn,Ghola,Ghorub,Gogron,Gorgo,Graklak,Graman,Grommok,Gul,Hanz,Krognak,Kurdan,Kurz,Lum,Lumdum,Luronk,Magra,Magub,Maknok,Orok,Rugdumph,Shagol,Shagrol,Shobob,Shum,Ulmug,Urbul,Ushnar,Uzul,Balagog,Borkul,Gadba,Gat,Grogmar,Kharag,Moth,Mul,Mulush,Urag,Yar,Garnag,Rogthun,Bashnag,Godrun,Gudrig,Gushagub,Joun,Kurog,Loghorz,Urmuk,Durgob,Grul,Mog,Othmash,Ramolg,Shelakh,Uzgakh"

    OrcMaleFirstNamesLst1 = new string[127]
        OrcMaleFirstNamesLst1[126] = "END OF LIST"

    OrcMaleFirstNamesLst2 = new string[127]
        OrcMaleFirstNamesLst2[126] = "END OF LIST"

    OrcMaleFirstNamesLst3 = new string[127]
        OrcMaleFirstNamesLst3[126] = "END OF LIST"

;/ORC FEMALE FIRST NAMES/;
    OrcFemaleFirstNames = ",Agrob,Badbog,Bashuk,Bogdub,Bugdurash,Bula,Bulak,Bulfim,Bum,Burub,Burzob,Dura,Durgat,Durz,Gashnakh,Ghob,Glasha,Glob,Gluronk,Gonk,Grat,Grazob,Gulfim,Kharzug,Lagakh,Lambug,Lazgar,Mogak,Morn,Murob,Murzush,Nargol,Orbul,Ragash,Rolfish,Rulfim,Shadbak,Shagar,Shagdub,Sharn,Sharog,Shelur,Sloomalah,Uloth,Ulumpha,Urzoth,Urzul,Ushug,Yazgash,Batul,Borba,Bumph,Homraz,Mazoga,Mog,Mor,Oghash,Orag,Rogbut,Rogmesh,Snak,Ugak,Umar,Umog,Batum,Ghorza,Lash,Urzoga,Yag,Mazgar,Voltha"

    OrcFemaleFirstNamesLst1 = new string[127]
        OrcFemaleFirstNamesLst1[126] = "END OF LIST"

    OrcFemaleFirstNamesLst2 = new string[127]
        OrcFemaleFirstNamesLst2[126] = "END OF LIST"

    OrcFemaleFirstNamesLst3 = new string[127]
        OrcFemaleFirstNamesLst3[126] = "END OF LIST"

;/ORC FAMILY NAMES/;
    OrcFamilyNames = ",Nagorm,Bagdub,Bulfim,Ulfish,Agadbu,Aglakh,Agum,Atumph,Azorku,Badbu,Bagrat,Bagul,Bamog,Bar,Bargamph,Bashnag,Bat,Batul,Boga,Bogamakh,Bogharz,Bogla,Boglar,Bogrol,Boguk,Bol,Bolak,Borbog,Borbul,Bug,Bugarn,Bulag,Bularz,Bulfish,Burbug,Burish,Burol,Buzga,Dugul,Dul,Dula,Dulob,Dumul,Dumulg,Durga,Durog,Durug,Dush,Gar,Gashel,Gat,Ghash,Ghasharzol,Gholfim,Gholob,Ghorak,Glorzuf,Gluk,Glurkub,Gorzog,Grambak,Gulfim,Gurakh,Gurub,Kashug,Khagdum,Kharbush,Kharz,Khash,Khashnar,Khatub,Khazor,Lag,Lagdub,Largum,Lazgarn,Loghash,Logob,Logrob,Lorga,Lumbuk,Lumob,Lurkul,Lurn,Luzgan,Magar,Magrish,Mar,Marob,Mashnar,Mogduk,Moghakh,Mughol,Muk,Mulakh,Murgol,Murug,Murz,Muzgob,Muzgub,Muzgur,Ogar,Ogdub,Ogdum,Olor,Olurba,Orbuma,Rimph,Rugob,Rush,Rushub,Shadbuk,Shagdub,Shagdulg,Shagrak,Shagramph,Shak,Sham,Shamub,Sharbag,Sharga,Sharob,Sharolg,Shat,Shatub,Shazog,Shug,Shugarz,Shugham,Shula,Shulor,Shumba,Shuzgub,Skandar,Snagarz,Snagdu,Ufthamph,Uftharz,Ugruma,Ular,Ulfimph,Urgak,Ushar,Ushug,Ushul,Uzgurn,Uzuk,Yagarz,Yak,Yargul,Yarzol,Baroth,Coblug,Marad,Mogakh,Orum,Shurgak,Agamph,Barak,Bargol,Bharg,Bolmog,Bonk,Brok,Buglump,Bumph,Bura,Burbog,Cromgog,Dragol,Galash,Gamorn,Gash,Gharz,Ghoth,Glurzog,Golpok,Gonk,Grulam,Hubrag,Khar,Magul,Malog,Morgrump,Murgak,Muzgol,Naybek,Orkul,Orkulg,Rugdush,Shadborgob,Shagk,Shatur,Shura,Ugdub,Urgash,Uzgash,Uzug,Yarug,Bagol,Largash,Shugurz,Burzag,Dushnikh,Gatuk,Gortwog,Nolob,Shargakh,Shub,Shurkul,Murtag,Gorzoth,Shugdurbam,Yagash,Gularz,Yamwort"

    OrcFamilyNamesLst1 = new string[127]
        OrcFamilyNamesLst1[126] = "END OF LIST"

    OrcFamilyNamesLst2 = new string[127]
        OrcFamilyNamesLst2[126] = "END OF LIST"

    OrcFamilyNamesLst3 = new string[127]
        OrcFamilyNamesLst3[126] = "END OF LIST"

;/NORD MALE FIRST NAMES/;
    NordMaleFirstNames = ",Urik,Haki,Abbard,Adding,Aenar,Agning,Alfbrand,Alfhedil,Allding,Alof,Alvring,Andrelheim,Areas,Arkming,Arnskar,Audmund,Balfring,Barirrid,Baslod,Bathmar,Bato,Batou,Bedraflod,Bedrir,Bjadmund,Bormir,Botrir,Brandr,Briring,Bronrod,Brurid,Bryngrim,Brynjolfr,Cristus,Daglin,Dathman,Dryngheid,Eifid,Engar,Ennbjof,Ergnir,Erich,Fanar,Fjol,Fjorgeir,Fjori,Fjorrod,Fjrokvar,Forstaag,Fraki,Frald,Frik,Garding,Geilir,Godrod,Gogrek,Graring,Gudling,Haakon,Haening,Haesmar,Hagrad,Hakar,Hallvaror,Halof,Hamlof,Hamvir,Hefadmir,Heidmir,Heifnir,Herd,Higil,Hirnir,Hisin,Hjoldir,Hjrondir,Hlar,Hloggar,Hlora,Hlormar,Hodlismod,Hoki,Holmar,Honthjolf,Horstar,Hrargal,Hrisskar,Hroldar,Hrondar,Hrundi,Hulgarth,Hurg,Hurnfing,Hurolf,Hylf,Idlami,Ingmar,Itar,Jolgeir,Kjeld,Knurguri,Korst,Kunthar,Lassnr,Lorod,Maering,Mendel,Norring,Odmi,Onmi,Ormvard,Pelf,Radd,Raflod,Rakoslod,Ranmir,Reinhardt,Rigmor,Roggvar,Rolf,Saetring,Sarnir,Sattir,Savard,Sifknir,Sigvatr,Sittar,Sjarnfid,Sjoring,Sjorvar,Skjoldr,Snadir,Snedbrir,Snorri,Sold,Sor,Sorkvild,Svog,Tharsten,Thongar,Thorek,Thormoor,Throknolf,Thrud,Thurek,Toralf,Tymvaul,Ulfgar,Urfing,Valgeir,Valund,Velfred,Vigunn,Virgerd,Vongvild,Yngling,Agarmir,Agnar,Algot,Arctus,Beirir,Bittneld,Bjalfi,Burd,Carsten,Eitar,Erich,Fafnir,Fjotreid,Geimund,Gromm,Gukimir,Gunder,Gundlar,Hafid,Hamlof,Hans,Havilstein,Heinrich,Henrik,Hil,Hjar,Hlofgar,Holger,Honmund,Horkvir,Hridi,Hrol,Irroke,Iver,Jayred,Jesper,Jofnhild,Jollring,Jorck,Jorundr,Kalthar,Keld,Larthjar,Logvaar,Lorenz,Lorkmir,Lynch,Manheim,Mogens,Msirae,Nels,Newheim,Niels,Olav,Olfand,Ongar,Orgnolf,Ortis,Regner,Reistr,Rolgarel,Roliand,Snar,Sten,Storn,Styrbjorn,Tolgan,Torbal,Torolf,Tove,Ulfgar,Valdemar,Vidkun,Vigge,Wilhelm,Yngvar,Tyr,Fruth,Garthar,Isilr,Jorunn,Skegglund,Thorbjolf,Vigrod,Einarr"

    NordMaleFirstNamesLst1 = new string[127]
        NordMaleFirstNamesLst1[126] = "END OF LIST"

    NordMaleFirstNamesLst2 = new string[127]
        NordMaleFirstNamesLst2[126] = "END OF LIST"

    NordMaleFirstNamesLst3 = new string[127]
        NordMaleFirstNamesLst3[126] = "END OF LIST"

;/NORD FEMALE FIRST NAMES/;
    NordFemaleFirstNames = ",Aeta,Aldi,Anja,Aumsi,Barri,Bergljot,Brasteir,Eigma,Eiruki,Ekkhi,Erna,Eydis,Fryfnhild,Fryssa,Greidil,Grerid,Haema,Heddvild,Helga,Hidar,Hjotra,Holmgeira,Horski,Hreirek,Hroa,Hrordis,Hyring,Igna,Ilfhild,Imsin,Ingokning,Jolding,Kar,Kili,Kjolver,Kolfinna,Lisaa,Merta,Mette,Olfeigr,Ondi,Rangela,Ringvild,Risi,Rostlogi,Runa,Siri,Sottilde,Svenja,Thalestris,Thunmund,Ulfrun,Ursine,Vori,Ygfa,Agata,Agnete,Alga,Edla,Else,Emfrid,Eyja,Hjolfrodi,Jensine,Kirsten,Maeva,Malene,Maren,Margarte,Merete,Nille,Olava,Petrine,Rigmor,Rona,Signy,Sigrid,Skjorta,Sonje,Ulrika,Vigdis,Lyris,Mera,Froa,Aegilief,Bera,Britte,Finna,Freida,Frilgeth,Fredo,Fjotli,Friga,Freydis,Froda,Gretta,Greyf,Grosta,Grunilda,Hela,Helgi,Helgreir,Inga,Isgeror,Jofrior,Katisha,Koralla,Liesl,Lilija,Mabjaarn,Macalla,Macke,Nette,Nurnhilde,Oddny,Ogra,Reyda,Sorga,Svana,Vibeke,Vunhilde"

    NordFemaleFirstNamesLst1 = new string[127]
        NordFemaleFirstNamesLst1[126] = "END OF LIST"

    NordFemaleFirstNamesLst2 = new string[127]
        NordFemaleFirstNamesLst2[126] = "END OF LIST"

    NordFemaleFirstNamesLst3 = new string[127]
        NordFemaleFirstNamesLst3[126] = "END OF LIST"

;/NORD FAMILY NAMES/;
    NordFamilyNames = ",Half-Troll,Hard-Heart,Ice-Mane,Battle-Born,Gray-Mane,Black-Briar,Snow-Shod,Cruel-Sea,Shatter-Shield,Silver-Blood,Stone-Fist,Fine-Hair,Fire-Eye,Fork-Beard,Gray-Wave,Red-Spear,Snow-Song,Wave-Breaker,Wolf-Runner,Bear-Arm,Oaken-Hull,Banner-Torn,Crag-Strider,Fair-Shield,Knot-Beard,Lonely-Gale,Sable-Hilt,Wild-Blood,Early-Beard,Flame-Hair,Green-Hand,Oaken-Heart,Snow-Strider,White-Beard,Wolf-Child"

    NordFamilyNamesLst1 = new string[127]
        NordFamilyNamesLst1[126] = "END OF LIST"

    NordFamilyNamesLst2 = new string[127]
        NordFamilyNamesLst2[126] = "END OF LIST"

    NordFamilyNamesLst3 = new string[127]
        NordFamilyNamesLst3[126] = "END OF LIST"

;/KHAJIIT MALE FIRST NAMES/;
    KhajiitMaleFirstNames = ",Baadargo,Baana,Baasha,Bakha,Bari,Barri,Bassa,Bhisha,Cherim,Dahlima,Dar,Dara,Darsha,Dasha,Dato,Desi,Dhannar,Dirr,Dirsha,Drassa,Dumiwa,Farahn,Fazir,Ffer,Ghasta,Hanir,Hassiri,Helnor,Hunal,Iffech,Jahirr,Jarsha,Jazha,Jhan,Jhera,Jidarr,Jiradh,Jirr,Jobasha,Jodhur,Joshur,Joto,Kara,Karim,Kazagh,Keem,Khar,Khasha,Kothre,Lasha,Lwef-Dim,Mhad,Mhirr,Nagh,Nashi,Qa,Qanar,Raaj,Radirr,Raksa,Ranarr,Rasha,Rashirr,Raska,Rathad,Rathra,Rava,Raverr,Razirr,Ren,Renji,Riska,Saddha,Sakhar,Sava,Sha'jal,Shajirr,Shan,Shani,Shanji,Sharr,Sharra,Shavir,Sholani,Sirr,Skar,Tasarr,Tassa,Tesh,Thengil,Therris,Thri,Urjabhi,Urjorad,Vandra,Vanir,Vanni,Vassa,Virr,Wadarkhu,Zadabal,Zaddha,Zagun,Zah,Zahn,Zakar,Zamha,Zaymar,Zharim,Zhirr,Zaadha,Zahr,Zhid,Zidzo,Zin,Zoaraym"
    KhajiitMaleFirstNamesLst1 = new string[127]
        KhajiitMaleFirstNamesLst1[126] = "END OF LIST"

    KhajiitMaleFirstNamesLst2 = new string[127]
        KhajiitMaleFirstNamesLst2[126] = "END OF LIST"

    KhajiitMaleFirstNamesLst3 = new string[127]
        KhajiitMaleFirstNamesLst3[126] = "END OF LIST"

;/KHAJIIT MALE PREFICES/;
    KhajiitMalePrefices = ",Dro,Fa,J,Ja,Jo,K,M,Ma,N,Qa,R,Ra,Ri,S,Sa"

    KhajiitMalePreficesLst1 = new string[127]
        KhajiitMalePreficesLst1[126] = "END OF LIST"

    KhajiitMalePreficesLst2 = new string[127]
        KhajiitMalePreficesLst2[126] = "END OF LIST"

    KhajiitMalePreficesLst3 = new string[127]
        KhajiitMalePreficesLst3[126] = "END OF LIST"

;/KHAJIIT MALE SUFFICES/;
    KhajiitMaleSuffices = ",dar,do,dra,ja,jo,ko,ma,ra,ri,sa"

    KhajiitMaleSufficesLst1 = new string[127]
        KhajiitMaleSufficesLst1[126] = "END OF LIST"

    KhajiitMaleSufficesLst2 = new string[127]
        KhajiitMaleSufficesLst2[126] = "END OF LIST"

    KhajiitMaleSufficesLst3 = new string[127]
        KhajiitMaleSufficesLst3[126] = "END OF LIST"

;/KHAJIIT FEMALE FIRST NAMES/;
    KhajiitFemaleFirstNames = ",Abanji,Adanja,Addhiranirr,Adharanji,Affri,Ahdahni,Ahdni,Ahdri,Ahjara,Ahnarra,Ahnassi,Ahndahra,Ahnia,Ahnisa,Ahzini,Aina,Ajira,Anjari,Arabhi,Aravi,Ashidasha,Bahdahna,Bahdrashi,Baissa,Bhusari,Chirranirr,Dahleena,Dahnara,Ekapi,Harassa,Habasi,Idhassi,Inerri,Inorra,Kaasha,Khamuzi,Khazura,Khinjarsi,Kiseena,Kishni,Kisimba,Kisisa,Nisaba,Rabinna,Shaba,Shivani,Shotherra,Shunari,Tsabhi,Tsajadhi,Tsalani,Tsani,Tsiya,Tsrazami,Ubaasi,Udarra,Unjara,Vanjirra,Zahraji,Abhuki,Ahdarji,Ahjazda,Ashni,Atahba,Atrabhi,Ayisha,Nahrahe,Kishashi,Nahsi,Fara,Jirra,Kasha,Krivva,Mirra,Thasa,Shamada,Shomara,Shuravi,Talasma,Tsalajma,Tsarrina,Tsavi,Tsramla,Tsrava,Vajhira,Yushi,Zabhila,Zahrasha,Kishra,Lesspa,Kai,Sharwa,Tsani,Zara,Ahnissi,Anurassa,Bhisha,Kish'na,Mizaba,Rathumi"

    KhajiitFemaleFirstNamesLst1 = new string[127]
        KhajiitFemaleFirstNamesLst1[126] = "END OF LIST"

    KhajiitFemaleFirstNamesLst2 = new string[127]
        KhajiitFemaleFirstNamesLst2[126] = "END OF LIST"

    KhajiitFemaleFirstNamesLst3 = new string[127]
        KhajiitFemaleFirstNamesLst3[126] = "END OF LIST"

;/KHAJIIT FEMALE PREFICES/;
    KhajiitFemalePrefices = ",Dro,Fa,J,Ja,Jo,K,M,Ma,N,Qa,R,Ra,Ri,S,Sa"

    KhajiitFemalePreficesLst1 = new string[127]
        KhajiitFemalePreficesLst1[126] = "END OF LIST"

    KhajiitFemalePreficesLst2 = new string[127]
        KhajiitFemalePreficesLst2[126] = "END OF LIST"

    KhajiitFemalePreficesLst3 = new string[127]
        KhajiitFemalePreficesLst3[126] = "END OF LIST"

;/KHAJIIT FEMALE SUFFICES/;
    KhajiitFemaleSuffices = ",daro,do,dra,dro,ja,jo,ko,la,ma,ra,ri,sa"

    KhajiitFemaleSufficesLst1 = new string[127]
        KhajiitFemaleSufficesLst1[126] = "END OF LIST"

    KhajiitFemaleSufficesLst2 = new string[127]
        KhajiitFemaleSufficesLst2[126] = "END OF LIST"

    KhajiitFemaleSufficesLst3 = new string[127]
        KhajiitFemaleSufficesLst3[126] = "END OF LIST"

;/KHAJIIT FAMILY NAMES/;
    KhajiitFamilyNames = ",Ahzii,Amanni,ap'Anequina,ap'Corinthe,ap'Keirgo,ap'Khaj,ap'Kolthis,ap'Leyawiin,ap'Makapi,ap'Malapi,ap'Maor,ap'Mistral,ap'Ne Quin-al,ap'Niben,ap'Orcrest,ap'Pelletine,ap'Quin'rawl,ap'Rawl'kha,ap'Rimmen,ap'Saimisil,ap'Senchal,ap'Tenmar,ap'Topal,ap'Torval,ap'Zelinin,Barakar,Brae,Budiit,Gzalziit,Hammurabi,Jahan,Keirgiit,Khahir,Kioni,Leyawiit,Lhajiit,Maasziit,Mahtani,Makapiit,Malapiit,Maoriit,Nibeniit,Saimisiit,Skoomiit,Tenmariit,Topaliit,Torvaliit,Zeliniit,za'Alfiq,za'Alfiq-raht,za'Cathay,za'Cathay-raht,za'Dagi,za'Dagi-raht,za'Darloc,za'Jode,za'Jone,za'Ohmes,za'Ohmes-raht,za'Pahmar,za'Pahmar-raht,za'Rajhin,za'Senche,za'Senche-raht,za'Suthay,za'Suthay-raht,za'Tojay,za'Tojay-raht,za'Topal"

    KhajiitFamilyNamesLst1 = new string[127]
        KhajiitFamilyNamesLst1[126] = "END OF LIST"

    KhajiitFamilyNamesLst2 = new string[127]
        KhajiitFamilyNamesLst2[126] = "END OF LIST"

    KhajiitFamilyNamesLst3 = new string[127]
        KhajiitFamilyNamesLst3[126] = "END OF LIST"

;/IMPERIAL MALE FIRST NAMES/;
    ImperialMaleFirstNames = ",Amiel,Attrebus,Crito,Aebondeius,Afer,Albecius,Allian,Antonius,Apelles,Ariulcabor,Arius,Arrinis,Astius,Attelivupis,Audenian,Augurius,Aunius,Bacola,Baro,Benunius,Burcanius,Caccian,Caius,Calvario,Calvus,Canctunian,Cassius,Carnius,Cavortius,Cidius,Clagius,Clasomo,Clecentor,Clibergus,Cocistian,Codus,Corrudus,Crassius,Crottus,Crulius,Cunius,Darius,Depusanis,Drusus,Dunius,Durus,Duvianus,Ertius,Esdrufus,Falco,Falx,Fandus,Frinnius,Furius,Gergio,Glallian,Granius,Gratian,Gualtierus,Ignatius,Ildogesto,Ilnori,Iratian,Iulus,Jonus,Larisus,Larrius,Letreius,Levus,Linus,Lucan,Lucretinaus,Lurius,Luspinian,Luven,Malpenix,Manilian,Marsus,Matus,Mecilvus,Miles,Molvirian,Moris,Murberius,Murudius,Namanian,Naspis,Natalinus,Nirtunus,Nitterius,Nolus,Oritius,Percius,Pilus,Plitinius,Processus,Pustula,Reberio,Rufinus,Sabinus,Saenus,Saprius,Sauleius,Scelian,Segunivus,Seliulus,Sellus,Senilias,Silius,Sirollus,Sitialius,Somutis,Stentus,Strillian,Surus,Synnolian,Tappius,Tasellis,Ticemius,Tienius,Tininnus,Trebonius,Turedus,Tyronius,Vala,Valgus,Valvius,Vantustius,Varian,Varus,Vasesius,Vertilvius,Vinnus,Vodunius,Zeno,Antus,Belisarius,Berich,Jesan,Sergius,Adamus,Adrian,Amantius,Amiel,Amminus,Ancus,Andreas,Arcturus,Arentus,Ashtus,Audens,Bendu,Betto,Bremman,Brielus,Bruccius,Bruscius,Bruscus,Carius,Carmalo,Carodus,Cartrus,Casta,Claudius,Collatinus,Contumeliorus,Corvus,Cosmus,Danus,Decentius,Denel,Desideratus,Dion,Draninus,Fadus,Fagus,Florentius,Fortis,Fralav,Gaius,Galtus,Garrus,Gellius,Gemellus,Gerich,Giovanni,Haderus,Helvius,Helvo,Hieronymus,Hirrus,Humilis,Imus,Iniel,Inius,Irlav,Itius,Januarius,Janus,Jaras,Jirolin,Lerexus,Lurio,Malintus,Marinus,Marius,Maro,Mattius,Melus,Menien,Mercator,Nermus,Nigidius,Olvus,Ontus,Ormellius,Otius,Pennus,Pinarus,Plautis,Primo,Rallus,Raminus,Regulus,Reman,Rielus,Rimalus,Ris,Rufrius,Savlian,Servatius,Severius,Stantus,Sulinus,Surius,Tertius,Toutius,Trenus,Tyrellius,Valandrus,Valus,Vandorallen,Vantus,Velan,Velus,Viator,Viranus,Vontan,Vontus,Adonato,Adventus,Aquilius,Aventus,Bassianus,Calixto,Constatius,Corpulus,Crescius,Dexion,Ennodius,Gavros,Leontius,Mallus,Noster,Orthus,Paratus,Pavo,Proventus,Quentin,Quintus,Reburrus,Rogatus,Septimus,Sergius,Severio,Silus,Sorex,Sulla,Tacitus,Taurinus,Titus,Varnius,Venarus,Attrebus,Colin,Decmitius,Festus,Flaccus,Florius,Longinus,Maximian,Potitus,Acilius,Caccino,Castus,Chaero,Claudius,Clavides,Decumus,Dolcettus,Glabrio,Liodes,Ministes,Quintus,Reglius,Rellicus,Silanus,Teo,Vitellus"

    ImperialMaleFirstNamesLst1 = new string[127]
        ImperialMaleFirstNamesLst1[126] = "END OF LIST"

    ImperialMaleFirstNamesLst2 = new string[127]
        ImperialMaleFirstNamesLst2[126] = "END OF LIST"

    ImperialMaleFirstNamesLst3 = new string[127]
        ImperialMaleFirstNamesLst3[126] = "END OF LIST"

;/IMPERIAL FEMALE FIRST NAMES/;
    ImperialFemaleFirstNames = ",Adraria,Agrippina,Ahetotis,Alcedonia,Apronia,Arenara,Audania,Blatta,Cania,Canodia,Catia,Cinda,Cinia,Clilias,Coventina,Craetia,Eloe,Emelia,Eponis,Ettiene,Fallaise,Fammana,Fentula,Flacassia,Flaenia,Fralvia,Fruscia,Gaea,Germia,Horulia,Idonea,Isobel,Ladia,Lalaine,Lalatia,Larienna,Lassinia,Laurina,Leone,Livia,Lunia,Maesa,Marara,Moria,Nebia,Pallia,Pania,Pelena,Pellecia,Peregrina,Pritia,Raesa,Ruccia,Sabrina,Sarmosia,Selvia,Severa,Severia,Solea,Syloria,Thraccenia,Vala,Vianis,Viatrix,Viccia,Vilbia,Volrina,Alessia,Avita,Camilla,Ida,Jena,Selene,Aia,Ariela,Arnora,Arriana,Astia,Astinia,Augusta,Aviera,Avresa,Britta,Bronsila,Buntara,Caelia,Caldana,Cargas,Carmana,Casta,Caula,Ciirta,Dumania,Dynari,Erina,Etira,Faustina,Gruiand,Herminia,Isa,Ita,Janonia,Jantus,Jastia,Jastira,Javolia,Julitta,Lenka,Lucina,Lyra,Marana,Mariana,Marlena,Martina,Millona,Narina,Naspia,Perennia,Pista,Praxedes,Rena,Renee,Restita,Romana,Rona,Rosentia,Rusia,Salonia,Schlera,Selena,Sibylla,Silana,Simplicia,Tertia,Una,Viera,Vilena,Vinicia,Ysabel,Salonia,Adelaisa,Adrianne,Alexia,Camilla,Carlotta,Constance,Ganna,Gemma,Indara,Leonara,Mila,Pantea,Sosia,Valga,Vigdis,Viola,Vittoria,Cassia,Aelva,Dominitia,Lavinia,Villea"

    ImperialFemaleFirstNamesLst1 = new string[127]
        ImperialFemaleFirstNamesLst1[126] = "END OF LIST"

    ImperialFemaleFirstNamesLst2 = new string[127]
        ImperialFemaleFirstNamesLst2[126] = "END OF LIST"

    ImperialFemaleFirstNamesLst3 = new string[127]
        ImperialFemaleFirstNamesLst3[126] = "END OF LIST"

;/IMPERIAL FAMILY NAMES/;
    ImperialFamilyNames = ",Caius,Maro,Metilius,Tyranus,Verulus,Atrius,Cadiusus,Faustus,Flaccus,Herennia,Olcinius,Pullia,Tullius,Abitius,Acicius,Acilius,Aerius,Afronia,Agrudilius,Albarnian,Albuttian,Alfena,Alleius,Amatius,Amiulusus,Amnis,Amphia,Angius,Antabolis,Apinia,Artoria,Artorius,Aurrus,Autrus,Baenius,Berne,Blonia,Burtilius,Caerellius,Callonus,Carbo,Caro,Carius,Catanius,Catius,Catraso,Cedus,Celata,Ceno,Cerunia,Cines,Claevius,Clanler,Closcius,Cnisia,Colollius,Conciatius,Conician,Cosades,Crunus,Cullian,Curio,Delitian,Denian,Dergius,Douar,Duronia,Egnatius,Entius,Esdrecus,Essagan,Facian,Famalius,Fauseius,Felannus,Flaeus,Flarugrius,Flavonius,Fulbenus,Fulcinius,Furotis,Gabenagus,Gabinia,Galenus,Gloriosus,Gratius,Gratus,Gravius,Hanotepelus,Harmevus,Harsinia,Hateria,Herennius,Hers,Hertarian,Horatius,Iulus,Jannus,Jucanis,Jullalian,Laecinnius,Laenius,Lalelius,Leontiulonus,Liore,Lusius,Macrina,Macro,Magia,Magius,Mantedius,Maraennius,Maria,Marius,Maro,Matius,Maximus,Menanius,Mercius,Mero,Mevureius,Mico,Mido,Muco,Munia,Musilchiotus,Muspidius,Mussillius,Nigilius,Nuccius,Nuccusius,Nuncius,Olcinius,Oranius,Ostorius,Otiustiris,Palenix,Pelelius,Petilia,Pevengius,Plalocius,Platorius,Plebo,Ponius,Pontanian,Popillius,Posuceius,Pundus,Puruseius,Quarra,Quaspus,Rato,Roscius,Rulician,Saccus,Scerius,Scinia,Secunia,Sialius,Sibassius,Siruliulus,Sosia,Spurius,Statlilius,Talanian,Tiragrius,Truptor,Tunifus,Urgelian,Urgusiso,Urtius,Uulentanus,Valerius,Valius,Valodius,Vandacia,Vant,Vantinius,Varian,Varo,Varro,Velvus,Viciulus,Vinipter,Vitellia,Vitellius,Vunnis"

    ImperialFamilyNamesLst1 = new string[127]
        ImperialFamilyNamesLst1[126] = "END OF LIST"

    ImperialFamilyNamesLst2 = new string[127]
        ImperialFamilyNamesLst2[126] = "END OF LIST"

    ImperialFamilyNamesLst3 = new string[127]
        ImperialFamilyNamesLst3[126] = "END OF LIST"

;/DREMORA MALE FIRST NAMES/;
    DremoraMaleFirstNames = ",Gatanas,Imago,Methats,Rathine,Sumeer,Tanchelm,Anhaedra,Krazzt,Dregas,Khash-Ti,Velehk,Amkaos,Kathutet,Orthe,Ranyu,Aez,Akezel,Amkaos,Aruz,Atris,Bogtro,Demis,Dias,Dogas,Druitularg,Erunas,Gamir,Gnilokelo,Grel,Grexev,Gnilokelo,Grexev,Ifriz,Kathutet,Kehothar,Kothutuilk,Kyngogurth,Methats,Nariam,Ranyu,Riparius,Seris,Skoria,Sumeer,Tachnim,Tuzunkykz,Uldun,Vaekar,Velehk,Verkynvayl,Voriak,Vregas,Vyrotesn,Xiven,Xznotell,Ykal,Reive,Xalxorkig,Zexxil,Zzedenkathik"

    DremoraMaleFirstNamesLst1 = new string[127]
        DremoraMaleFirstNamesLst1[126] = "END OF LIST"

    DremoraMaleFirstNamesLst2 = new string[127]
        DremoraMaleFirstNamesLst2[126] = "END OF LIST"

    DremoraMaleFirstNamesLst3 = new string[127]
        DremoraMaleFirstNamesLst3[126] = "END OF LIST"

;/DREMORA FEMALE FIRST NAMES/;
    DremoraFemaleFirstNames = ",Zenaida,Akata,Angata,Barasatii,Brolsgerbwd,Clorsatari,Codaermet,Davlinn,Duneir,Dekamma,Doshia,Drulshasa,Ephae,Gadrah,Gytha-Kitha,Kixathi,Kvotai,Lyranth,Marruz,Mati,Mirok,Raynia,Rykuna,Ukaezai,Urata,Valkyn,Velark,Vintev,Vonshala"

    DremoraFemaleFirstNamesLst1 = new string[127]
        DremoraFemaleFirstNamesLst1[126] = "END OF LIST"

    DremoraFemaleFirstNamesLst2 = new string[127]
        DremoraFemaleFirstNamesLst2[126] = "END OF LIST"

    DremoraFemaleFirstNamesLst3 = new string[127]
        DremoraFemaleFirstNamesLst3[126] = "END OF LIST"

;/DREMORA FAMILY NAMES/;
    DremoraFamilyNames = ",Volar,Sain,Dhrur,Solkyn,Nacarra,Tephra,Jabran,Storm"

    DremoraFamilyNamesLst1 = new string[127]
        DremoraFamilyNamesLst1[126] = "END OF LIST"

    DremoraFamilyNamesLst2 = new string[127]
        DremoraFamilyNamesLst2[126] = "END OF LIST"

    DremoraFamilyNamesLst3 = new string[127]
        DremoraFamilyNamesLst3[126] = "END OF LIST"

;/GIANT NAMES/;
    GiantNames = ",Aki,Bomaz,Bor,Cracktooth,Grunt,Malvor,Sinmur,Ulgorus,Urgkail,Vigi"

    GiantNamesLst1 = new string[127]
        GiantNamesLst1[126] = "END OF LIST"

    GiantNamesLst2 = new string[127]
        GiantNamesLst2[126] = "END OF LIST"

    GiantNamesLst3 = new string[127]
        GiantNamesLst3[126] = "END OF LIST"

;/DRAGON NAMES/;
    DragonNames = ",Aak,Aam,Aar,Aaz,Ag,Ah,Al,Am,Aus,Bah,Bein,Beyn,Bii,Bo,Bok,Brii,Brit,Brom,Daan,Dein,Dez,Diil,Diin,Diiv,Dir,Drem,Drog,Du,Duul,Dun,Dur,Dwiin,Faad,Faas,Faaz,Fah,Feim,Fel,Feyn,Fo,Frin,Fus,Gaaf,Gaan,Gaar,Gol,Golt,Golz,Graag,Graan,Grah,Gram,Gro,Gron,Gruth,Gut,Haas,Hah,Heim,Heyv,Hind,Hon,Hun,Iiz,In,Jer,Jot,Jud,Jun,Kaal,Kah,Kein,Kest,Kiir,Klo,Ko,Kod,Koor,Krah,Kras,Kreh,Krein,Kren,Krent,Krif,Krii,Kriid,Kril,Krin,Kro,Kron,Kul,Laas,Laat,Lah,Lein,Liiv,Lir,Lo,Lok,Lon,Lot,Lun,Luv,Maar,Mah,Mal,Mey,Mid,Miin,Mir,Mon,Mul,Naak,Naar,Nah,Nahl,Nau,Nax,Neh,Nil,Nin,Nir,Nok,Nol,Nos,Nus,Od,Ol,Oth,Ov,Paak,Paal,Paar,Paaz,Peyt,Pook,Prem,Qah,Qeth,Qo,Qoth,Raan,Rah,Rath,Rein,Rel,Reyth,Rii,Riik,Ro,Ru,Ruth,Sah,Shaan,Shul,Sil,Slen,So,Sod,Sos,Sot,Spaan,Stin,Strun,Su,Suum,Tah,Thaarn,Thur,Tiid,Toor,Tu,Tuz,Ul,Uth,Vaat,Vah,Ved,Ven,Vey,Viik,Viin,Viir,Vith,Vol,Vu,Vul,Vur,Wahl,Wol,Wuld,Wuth,Yah,Yol,Zaam,Zaan,Zein,Zii,Zin,Zind,Zol,Zoor,Zul,Zun"

    DragonNamesLst1 = new string[127]
        DragonNamesLst1[126] = "END OF LIST"

    DragonNamesLst2 = new string[127]
        DragonNamesLst2[126] = "END OF LIST"

    DragonNamesLst3 = new string[127]
        DragonNamesLst3[126] = "END OF LIST"

;/DRAGON ONE SYLLABLE/;
    DragonOneSyllable = ",aak,aam,aar,aaz,ag,ah,al,am,aus,bah,bein,beyn,bii,bo,bok,brii,brit,brom,daan,dein,dez,diil,diin,diiv,dir,drem,drog,du,duul,dun,dur,dwiin,faad,faas,faaz,fah,feim,fel,feyn,fo,frin,fus,gaaf,gaan,gaar,gol,golt,golz,graag,graan,grah,gram,gro,gron,gruth,gut,haas,hah,heim,heyv,hind,hon,hun,iiz,in,jer,jot,jud,jun,kaal,kah,kein,kest,kiir,klo,ko,kod,koor,krah,kras,kreh,krein,kren,krent,krif,krii,kriid,kril,krin,kro,kron,kul,laas,laat,lah,lein,liiv,lir,lo,lok,lon,lot,lun,luv,maar,mah,mal,mey,mid,miin,mir,mon,mul,naak,naar,nah,nahl,nau,nax,neh,nil,nin,nir,nok,nol,nos,nus,od,ol,oth,ov,paak,paal,paar,paaz,peyt,pook,prem,qah,qeth,qo,qoth,raan,rah,rath,rein,rel,reyth,rii,riik,ro,ru,ruth,sah,shaan,shul,sil,slen,so,sod,sos,sot,spaan,stin,strun,su,suum,tah,thaarn,thur,tiid,toor,tu,tuz,ul,uth,vaat,vah,ved,ven,vey,viik,viin,viir,vith,vol,vu,vul,vur,wahl,wol,wuld,wuth,yah,yol,zaam,zaan,zein,zii,zin,zind,zol,zoor,zul,zun"

    DragonOneSyllableLst1 = new string[127]
        DragonOneSyllableLst1[126] = "END OF LIST"

    DragonOneSyllableLst2 = new string[127]
        DragonOneSyllableLst2[126] = "END OF LIST"

    DragonOneSyllableLst3 = new string[127]
        DragonOneSyllableLst3[126] = "END OF LIST"

;/DRAGON TWO SYLLABLE/;
    DragonTwoSyllable = ",ahkrin,ahmik,ahraan,ahzid,alun,alok,alun,bahlaan,bahlok,bonaar,bormah,boziik,brendon,briinah,bruniik,daanik,deyra,dilon,dilos,dinok,evgir,faasnu,fahdon,feykro,folook,gahrot,golah,goraan,grohiik,gravuun,hahdrim,hahnu,hevno,horvut,hungaar,inhus,kendov,kinbok,kinzon,kodaav,koraav,kosill,krasaar,krilot,krivaan,krongrah,kroniid,krosis,kruziik,kulaan,kulaas,liivrah,lingrah,lovaas,mahlaan,malhaan,midrot,midun,monah,morah,moro,mulaag,munax,nahkriin,nahlaas,nahlot,nikriin,nonvul,norok,odus,okaaz,onik,pahlok,prodah,pruzaan,pruzah,qahnaar,qalos,qolaas,qostiid,rahgol,rahgot,rahgron,revak,ronaaz,ronit,rovaan,sadon,sahlo,sahqo,sahqon,sahrot,sahvot,saviik,sivaas,smoliin,sosaal,staadnau,strunmah,suleyk,sunvaar,tafiir,tiraaz,unslaad,vahdin,vahlok,vahriin,vahzah,vahzen,viidost,viintaas,vokrii,vokul,vokun,volaan,vonun,vonuz,vulom,vulon,yolos,yuvon,zeymah"

    DragonTwoSyllableLst1 = new string[127]
        DragonTwoSyllableLst1[126] = "END OF LIST"

    DragonTwoSyllableLst2 = new string[127]
        DragonTwoSyllableLst2[126] = "END OF LIST"

    DragonTwoSyllableLst3 = new string[127]
        DragonTwoSyllableLst3[126] = "END OF LIST"

;/WARRIOR TITLES/;
    WarriorTitles = ",Armsman,Bold,Brave,Burly,Cavalier,Fearless,Fighter,Hardy,Ironclad,Mighty,Peacekeeper,Peerless,Protector,Shining,Soldier,Swaggering,Tireless,Valiant"

    WarriorTitlesLst1 = new string[127]
        WarriorTitlesLst1[126] = "END OF LIST"

    WarriorTitlesLst2 = new string[127]
        WarriorTitlesLst2[126] = "END OF LIST"

    WarriorTitlesLst3 = new string[127]
        WarriorTitlesLst3[126] = "END OF LIST"

;/THIEF TITLES/;
    ThiefTitles = ",Acrobat,Cunning,Discreet,Eloquent,Fleet,Honest,Irreverent,Keeneye,Naughty,Nervous,Ranger,Shifty,Slippery,Sly,Sneaky,Tricky,Twitchy,Wanderer,Watchful,Wily"

    ThiefTitlesLst1 = new string[127]
        ThiefTitlesLst1[126] = "END OF LIST"

    ThiefTitlesLst2 = new string[127]
        ThiefTitlesLst2[126] = "END OF LIST"

    ThiefTitlesLst3 = new string[127]
        ThiefTitlesLst3[126] = "END OF LIST"

;/MAGE TITLES/;
    MageTitles = ",Artificer,Astrologer,Bookish,Brilliant,Charmed,Enlightened,Illustrious,Learned,Mage,Magician,Magnificent,Philosopher,Sage,Seer,Sorcerer,Spellslinger,Strange,Unseen,Warlock,Wise"

    MageTitlesLst1 = new string[127]
        MageTitlesLst1[126] = "END OF LIST"

    MageTitlesLst2 = new string[127]
        MageTitlesLst2[126] = "END OF LIST"

    MageTitlesLst3 = new string[127]
        MageTitlesLst3[126] = "END OF LIST"

;/GENERIC TITLES/;
    GenericTitles = ",Able,Affable,Barefoot,Careless,Charming,Cross-Eyed,Debonaire,Drunk,Eager,Famous,Fickle,Fortunate,Gentle,Good,Handsome,Hapless,Haughty,Homeless,Hot-Headed,Indolent,Kind,Lisping,Long-Winded,Lucky,Merry,Mild,Mild-Mannered,Plain,Proud,Resourceful,Simple,Thoughtful,Ugly,Unlucky,Vain,Young"

    GenericTitlesLst1 = new string[127]
        GenericTitlesLst1[126] = "END OF LIST"

    GenericTitlesLst2 = new string[127]
        GenericTitlesLst2[126] = "END OF LIST"

    GenericTitlesLst3 = new string[127]
        GenericTitlesLst3[126] = "END OF LIST"

;/WARRIOR EPITHETS/;
    WarriorEpithets = ",the Aegis,the Armsman,the Angry,the Axe,the Battler,the Blade,the Bold,the Booming,the Brave,the Brawny,the Bulwark,the Burly,the Cavalier,the Edge,the Fearless,the Fighter,the Glorious,the Hammer,the Hardy,the Hart,the Hawk,the Hound,the Iron,the Ironclad,the Lion,the Lionheart,the Lofty,the Mauler,the Mighty,the Peacekeeper,the Peerless,the Powerful,the Protector,the Ram,the Roarer,the Shining,the Soldier,the Steed,the Stone,the Stout,the Strong,the Swaggering,the Thundering,the Tireless,the Tower,the Unbowed,the Unbroken,the Unwavering,the Valiant,the Warden,the Warrior,the Wolf"

    WarriorEpithetsLst1 = new string[127]
        WarriorEpithetsLst1[126] = "END OF LIST"

    WarriorEpithetsLst2 = new string[127]
        WarriorEpithetsLst2[126] = "END OF LIST"

    WarriorEpithetsLst3 = new string[127]
        WarriorEpithetsLst3[126] = "END OF LIST"

;/THIEF EPITHETS/;
    ThiefEpithets = ",the Acrobat,the Agent,the Crow,the Cunning,the Dagger,the Discreet,the Eloquent,the Fleet,the Fox,the Harrier,the Honest,the Irreverent,the Jester,the Keeneye,the Knife,the Naughty,the Nervous,the Precise,the Quick,the Ranger,the Rascal,the Robber,the Rogue,the Shadow,the Shifty,the Sly,the Smuggler,the Sneaky,the Swift,the Thief,the Tricky,the Twitchy,the Wanderer,the Watchful,the Wily,the Wit,the Worm"

    ThiefEpithetsLst1 = new string[127]
        ThiefEpithetsLst1[126] = "END OF LIST"

    ThiefEpithetsLst2 = new string[127]
        ThiefEpithetsLst2[126] = "END OF LIST"

    ThiefEpithetsLst3 = new string[127]
        ThiefEpithetsLst3[126] = "END OF LIST"

;/MAGE EPITHETS/;
    MageEpithets = ",of the Piercing Eyes,the Adept,the Apprentice,the Artificer,the Astrologer,the Brilliant,the Bookish,the Caller,the Charmed,the Conjurer,the Enlightened,the Illustrious,the Incendiary,the Learned,the Mage,the Magician,the Magnificent,the Mender,the Philosopher,the Sage,the Scholar,the Scribe,the Seer,the Sorcerer,the Spellslinger,the Strange,the Unseen,the Unveiled,the Warlock,the Willful,the Wise,the Wizard"

    MageEpithetsLst1 = new string[127]
        MageEpithetsLst1[126] = "END OF LIST"

    MageEpithetsLst2 = new string[127]
        MageEpithetsLst2[126] = "END OF LIST"

    MageEpithetsLst3 = new string[127]
        MageEpithetsLst3[126] = "END OF LIST"

;/GENERIC EPITHETS/;
    GenericEpithets = ",of the Forest,of the Hills,of the Isles,of the Mountains,of the Plains,of the River,of the Sea,of the Valley,the Able,the Adopted,the Affable,the Ambitious,the Amusing,the Barefoot,the Bastard,the Blowhard,the Braggart,the Brash,the Careless,the Changeling,the Charming,the Cross-Eyed,the Debonaire,the Determined,the Diplomat,the Dreamer,the Drunk,the Drunkard,the Eager,the Easterner,the Fair,the Famed,the Fawn,the Fickle,the Foreigner,the Fortunate,the Gentle,the Good,the Great,the Greater,the Handsome,the Hapless,the Haughty,the Homeless,the Hot-Headed,the Ill-Tempered,the Indolent,the Kind,the Lamb,the Last,the Light,the Lisping,the Long-Winded,the Lucky,the Merry,the Mild,the Mild-Mannered,the Northerner,the Pale,the Pickled,the Plain,the Portly,the Proud,the Quiet,the Ready,the Red,the Resourceful,the Short,the Simple,the Sot,the Southerner,the Tall,the Tiny,the Thoughtful,the Ugly,the Unlucky,the Vain,the Westerner,the Young,the Younger"

    GenericEpithetsLst1 = new string[127]
        GenericEpithetsLst1[126] = "END OF LIST"

    GenericEpithetsLst2 = new string[127]
        GenericEpithetsLst2[126] = "END OF LIST"

    GenericEpithetsLst3 = new string[127]
        GenericEpithetsLst3[126] = "END OF LIST"

;/BOSS TITLES/;
    BossSir = ",Accursed,Apostate,Blood-Thirsty,Bloody,Crazy,Cruel,Contemptible,Daedra-Blooded,Damned,Dark,Elf-Killer,Exiled,Fearsome,Ferocious,Forgotten,Frenzied,God-Like,Grim,Mad,Mer-Killer,Nefarious,Terrible,Treacherous,Tyrant,Vile,Voracious,Wicked,Wild,Wretched"

    BossSirLst1 = new string[127]
        BossSirLst1[126] = "END OF LIST"

    BossSirLst2 = new string[127]
        BossSirLst2[126] = "END OF LIST"

    BossSirLst3 = new string[127]
        BossSirLst3[126] = "END OF LIST"

;/BOSS EPITHETS/;
    BossName = ",of the Wastes,the Accursed,the Apostate,the Black,the Black Prince,the Blood-Thirsty,the Bloody,the Breaker,the Broken,the Butcher,the Cleaver,the Colossus,the Contemptible,the Corruptor,the Craven,the Crazy,the Cruel,the Contemptible,the Daedra-Blooded,the Daedric,the Damned,the Dark,the Darkened,the Deceiver,the Devil,the Dreg,the Elf-killer,the Executioner,the Exiled,the Fearsome,the Ferocious,the Flayer,the Forgotten,the Frenzied,the Glutton,the God-Like,the Grim,the Hearteater,the Hungry,the Hunter,the Impaler,the Ink-Blooded,the Lost,the Mad,the Mer-killer,the Monster,the Nefarious,the Outcast,the Outlaw,the Outsider,the Ravager,the Raven,the Reaper,the Ripper,the Serpent,the Serpent-Tongue,the Terrible,the Treacherous,the Tyrant,the Unending,the Unliving,the Unmentioned,the Unworthy,the Vile,the Voracious,the Wicked,the Wild,the Wretched,the Yellow"

    BossNameLst1 = new string[127]
        BossNameLst1[126] = "END OF LIST"

    BossNameLst2 = new string[127]
        BossNameLst2[126] = "END OF LIST"

    BossNameLst3 = new string[127]
        BossNameLst3[126] = "END OF LIST"

Notification("Real Names: Filling arrays ...")

    FillList(GenericNameSearchTermsLst1, GenericNameSearchTerms)
    FillList(GenericNameSearchTermsLst2, GenericNameSearchTerms)
    FillList(GenericNameSearchTermsLst3, GenericNameSearchTerms)
    lastentry = 0
;Notification("Real Names: Completed filling Generic Name Search Terms.")

    FillList(GenericBossSearchTermsLst1, GenericBossSearchTerms)
    FillList(GenericBossSearchTermsLst2, GenericBossSearchTerms)
    FillList(GenericBossSearchTermsLst3, GenericBossSearchTerms)
    lastentry = 0
;Notification("Real Names: Completed filling Generic Boss Search Terms.")

    FillList(BosmerMaleFirstNamesLst1, BosmerMaleFirstNames)
    FillList(BosmerMaleFirstNamesLst2, BosmerMaleFirstNames)
    FillList(BosmerMaleFirstNamesLst3, BosmerMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Bosmer Male First Names.")

    FillList(BosmerFemaleFirstNamesLst1, BosmerFemaleFirstNames)
    FillList(BosmerFemaleFirstNamesLst2, BosmerFemaleFirstNames)
    FillList(BosmerFemaleFirstNamesLst3, BosmerFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Bosmer Female First Names.")

    FillList(BosmerFamilyNamesLst1, BosmerFamilyNames)
    FillList(BosmerFamilyNamesLst2, BosmerFamilyNames)
    FillList(BosmerFamilyNamesLst3, BosmerFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Bosmer Tribe Names.")

    FillList(BretonMaleFirstNamesLst1, BretonMaleFirstNames)
    FillList(BretonMaleFirstNamesLst2, BretonMaleFirstNames)
    FillList(BretonMaleFirstNamesLst3, BretonMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Breton Male First Names.")

    FillList(BretonFemaleFirstNamesLst1, BretonFemaleFirstNames)
    FillList(BretonFemaleFirstNamesLst2, BretonFemaleFirstNames)
    FillList(BretonFemaleFirstNamesLst3, BretonFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Breton Female First Names.")

    FillList(BretonFamilyNamesLst1, BretonFamilyNames)
    FillList(BretonFamilyNamesLst2, BretonFamilyNames)
    FillList(BretonFamilyNamesLst3, BretonFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Breton Surnames.")

    FillList(DunmerMaleFirstNamesLst1, DunmerMaleFirstNames)
    FillList(DunmerMaleFirstNamesLst2, DunmerMaleFirstNames)
    FillList(DunmerMaleFirstNamesLst3, DunmerMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Dunmer Male First Names.")

    FillList(DunmerFemaleFirstNamesLst1, DunmerFemaleFirstNames)
    FillList(DunmerFemaleFirstNamesLst2, DunmerFemaleFirstNames)
    FillList(DunmerFemaleFirstNamesLst3, DunmerFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Dunmer Female First Names.")

    FillList(DunmerFamilyNamesLst1, DunmerFamilyNames)
    FillList(DunmerFamilyNamesLst2, DunmerFamilyNames)
    FillList(DunmerFamilyNamesLst3, DunmerFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Dunmer House Names.")

    FillList(AshlanderMaleFirstNamesLst1, AshlanderMaleFirstNames)
    FillList(AshlanderMaleFirstNamesLst2, AshlanderMaleFirstNames)
    FillList(AshlanderMaleFirstNamesLst3, AshlanderMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Ashlander Male First Names.")

    FillList(AshlanderFemaleFirstNamesLst1, AshlanderFemaleFirstNames)
    FillList(AshlanderFemaleFirstNamesLst2, AshlanderFemaleFirstNames)
    FillList(AshlanderFemaleFirstNamesLst3, AshlanderFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Ashlander Female First Names.")

    FillList(AshlanderFamilyNamesLst1, AshlanderFamilyNames)
    FillList(AshlanderFamilyNamesLst2, AshlanderFamilyNames)
    FillList(AshlanderFamilyNamesLst3, AshlanderFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Ashlander Tribe Names.")

    FillList(ArgonianMaleFirstNamesLst1, ArgonianMaleFirstNames)
    FillList(ArgonianMaleFirstNamesLst2, ArgonianMaleFirstNames)
    FillList(ArgonianMaleFirstNamesLst3, ArgonianMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Argonian Male First Names.")

    FillList(ArgonianFemaleFirstNamesLst1, ArgonianFemaleFirstNames)
    FillList(ArgonianFemaleFirstNamesLst2, ArgonianFemaleFirstNames)
    FillList(ArgonianFemaleFirstNamesLst3, ArgonianFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Argonian Female First Names.")

    FillList(ArgonianFamilyNamesLst1, ArgonianFamilyNames)
    FillList(ArgonianFamilyNamesLst2, ArgonianFamilyNames)
    FillList(ArgonianFamilyNamesLst3, ArgonianFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Argonian Tribe Names.")

    FillList(AltmerMaleFirstNamesLst1, AltmerMaleFirstNames)
    FillList(AltmerMaleFirstNamesLst2, AltmerMaleFirstNames)
    FillList(AltmerMaleFirstNamesLst3, AltmerMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Altmer Male First Names.")

    FillList(AltmerFemaleFirstNamesLst1, AltmerFemaleFirstNames)
    FillList(AltmerFemaleFirstNamesLst2, AltmerFemaleFirstNames)
    FillList(AltmerFemaleFirstNamesLst3, AltmerFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Altmer Female First Names.")

    FillList(AltmerFamilyNamesLst1, AltmerFamilyNames)
    FillList(AltmerFamilyNamesLst2, AltmerFamilyNames)
    FillList(AltmerFamilyNamesLst3, AltmerFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Altmer House Names.")

    FillList(RedguardMaleFirstNamesLst1, RedguardMaleFirstNames)
    FillList(RedguardMaleFirstNamesLst2, RedguardMaleFirstNames)
    FillList(RedguardMaleFirstNamesLst3, RedguardMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Redguard Male First Names.")

    FillList(RedguardFemaleFirstNamesLst1, RedguardFemaleFirstNames)
    FillList(RedguardFemaleFirstNamesLst2, RedguardFemaleFirstNames)
    FillList(RedguardFemaleFirstNamesLst3, RedguardFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Redguard Female First Names.")

    FillList(RedguardFamilyNamesLst1, RedguardFamilyNames)
    FillList(RedguardFamilyNamesLst2, RedguardFamilyNames)
    FillList(RedguardFamilyNamesLst3, RedguardFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Redguard Surnames.")

    FillList(OrcMaleFirstNamesLst1, OrcMaleFirstNames)
    FillList(OrcMaleFirstNamesLst2, OrcMaleFirstNames)
    FillList(OrcMaleFirstNamesLst3, OrcMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Orc Male First Names.")

    FillList(OrcFemaleFirstNamesLst1, OrcFemaleFirstNames)
    FillList(OrcFemaleFirstNamesLst2, OrcFemaleFirstNames)
    FillList(OrcFemaleFirstNamesLst3, OrcFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Orc Female First Names.")

    FillList(OrcFamilyNamesLst1, OrcFamilyNames)
    FillList(OrcFamilyNamesLst2, OrcFamilyNames)
    FillList(OrcFamilyNamesLst3, OrcFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Orc Clan Names.")

    FillList(NordMaleFirstNamesLst1, NordMaleFirstNames)
    FillList(NordMaleFirstNamesLst2, NordMaleFirstNames)
    FillList(NordMaleFirstNamesLst3, NordMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Nord Male First Names.")

    FillList(NordFemaleFirstNamesLst1, NordFemaleFirstNames)
    FillList(NordFemaleFirstNamesLst2, NordFemaleFirstNames)
    FillList(NordFemaleFirstNamesLst3, NordFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Nord Female First Names.")

    FillList(NordFamilyNamesLst1, NordFamilyNames)
    FillList(NordFamilyNamesLst2, NordFamilyNames)
    FillList(NordFamilyNamesLst3, NordFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Nord Clan Names.")

    FillList(KhajiitMaleFirstNamesLst1, KhajiitMaleFirstNames)
    FillList(KhajiitMaleFirstNamesLst2, KhajiitMaleFirstNames)
    FillList(KhajiitMaleFirstNamesLst3, KhajiitMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Khajiit Male First Names.")

    FillList(KhajiitMalePreficesLst1, KhajiitMalePrefices)
    FillList(KhajiitMalePreficesLst2, KhajiitMalePrefices)
    FillList(KhajiitMalePreficesLst3, KhajiitMalePrefices)
    lastentry = 0
;Notification("Real Names: Completed filling Khajiit Male Prefices.")

    FillList(KhajiitMaleSufficesLst1, KhajiitMaleSuffices)
    FillList(KhajiitMaleSufficesLst2, KhajiitMaleSuffices)
    FillList(KhajiitMaleSufficesLst3, KhajiitMaleSuffices)
    lastentry = 0
;Notification("Real Names: Completed filling Khajiit Male Suffices.")

    FillList(KhajiitFemaleFirstNamesLst1, KhajiitFemaleFirstNames)
    FillList(KhajiitFemaleFirstNamesLst2, KhajiitFemaleFirstNames)
    FillList(KhajiitFemaleFirstNamesLst3, KhajiitFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Khajiit Female First Names.")

    FillList(KhajiitFemalePreficesLst1, KhajiitFemalePrefices)
    FillList(KhajiitFemalePreficesLst2, KhajiitFemalePrefices)
    FillList(KhajiitFemalePreficesLst3, KhajiitFemalePrefices)
    lastentry = 0
;Notification("Real Names: Completed filling Khajiit Female Prefices.")

    FillList(KhajiitFemaleSufficesLst1, KhajiitFemaleSuffices)
    FillList(KhajiitFemaleSufficesLst2, KhajiitFemaleSuffices)
    FillList(KhajiitFemaleSufficesLst3, KhajiitFemaleSuffices)
    lastentry = 0
;Notification("Real Names: Completed filling Khajiit Female Suffices.")

    FillList(KhajiitFamilyNamesLst1, KhajiitFamilyNames)
    FillList(KhajiitFamilyNamesLst2, KhajiitFamilyNames)
    FillList(KhajiitFamilyNamesLst3, KhajiitFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Khajiit Tribe Names.")

    FillList(ImperialMaleFirstNamesLst1, ImperialMaleFirstNames)
    FillList(ImperialMaleFirstNamesLst2, ImperialMaleFirstNames)
    FillList(ImperialMaleFirstNamesLst3, ImperialMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Imperial Male First Names.")

    FillList(ImperialFemaleFirstNamesLst1, ImperialFemaleFirstNames)
    FillList(ImperialFemaleFirstNamesLst2, ImperialFemaleFirstNames)
    FillList(ImperialFemaleFirstNamesLst3, ImperialFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Imperial Female First Names.")

    FillList(ImperialFamilyNamesLst1, ImperialFamilyNames)
    FillList(ImperialFamilyNamesLst2, ImperialFamilyNames)
    FillList(ImperialFamilyNamesLst3, ImperialFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Imperial Family Names.")

    FillList(DremoraMaleFirstNamesLst1, DremoraMaleFirstNames)
    FillList(DremoraMaleFirstNamesLst2, DremoraMaleFirstNames)
    FillList(DremoraMaleFirstNamesLst3, DremoraMaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Dremora Male First Names.")

    FillList(DremoraFemaleFirstNamesLst1, DremoraFemaleFirstNames)
    FillList(DremoraFemaleFirstNamesLst2, DremoraFemaleFirstNames)
    FillList(DremoraFemaleFirstNamesLst3, DremoraFemaleFirstNames)
    lastentry = 0
;Notification("Real Names: Completed filling Dremora Female First Names.")

    FillList(DremoraFamilyNamesLst1, DremoraFamilyNames)
    FillList(DremoraFamilyNamesLst2, DremoraFamilyNames)
    FillList(DremoraFamilyNamesLst3, DremoraFamilyNames)
    lastentry = 0
;Notification("Real Names: Completed filling Dremora Kyn Names.")

    FillList(GiantNamesLst1, GiantNames)
    FillList(GiantNamesLst2, GiantNames)
    FillList(GiantNamesLst3, GiantNames)
    lastentry = 0
;Notification("Real Names: Completed filling Giant Names.")

    FillList(DragonNamesLst1, DragonNames)
    FillList(DragonNamesLst2, DragonNames)
    FillList(DragonNamesLst3, DragonNames)
    lastentry = 0
;Notification("Real Names: Completed filling Dragon Names.")

    FillList(DragonOneSyllableLst1, DragonOneSyllable)
    FillList(DragonOneSyllableLst2, DragonOneSyllable)
    FillList(DragonOneSyllableLst3, DragonOneSyllable)
    lastentry = 0
;Notification("Real Names: Completed filling Dragon One Syllable list.")

    FillList(DragonTwoSyllableLst1, DragonTwoSyllable)
    FillList(DragonTwoSyllableLst2, DragonTwoSyllable)
    FillList(DragonTwoSyllableLst3, DragonTwoSyllable)
    lastentry = 0
;Notification("Real Names: Completed filling Dragon Two Syllable list.")

    FillList(WarriorTitlesLst1, WarriorTitles)
    FillList(WarriorTitlesLst2, WarriorTitles)
    FillList(WarriorTitlesLst3, WarriorTitles)
    lastentry = 0
;Notification("Real Names: Completed filling Warrior Titles.")

    FillList(ThiefTitlesLst1, ThiefTitles)
    FillList(ThiefTitlesLst2, ThiefTitles)
    FillList(ThiefTitlesLst3, ThiefTitles)
    lastentry = 0
;Notification("Real Names: Completed filling Thief Titles.")

    FillList(MageTitlesLst1, MageTitles)
    FillList(MageTitlesLst2, MageTitles)
    FillList(MageTitlesLst3, MageTitles)
    lastentry = 0
;Notification("Real Names: Completed filling Mage Titles.")

    FillList(GenericTitlesLst1, GenericTitles)
    FillList(GenericTitlesLst2, GenericTitles)
    FillList(GenericTitlesLst3, GenericTitles)
    lastentry = 0
;Notification("Real Names: Completed filling Generic Titles.")

    FillList(WarriorEpithetsLst1, WarriorEpithets)
    FillList(WarriorEpithetsLst2, WarriorEpithets)
    FillList(WarriorEpithetsLst3, WarriorEpithets)
    lastentry = 0
;Notification("Real Names: Completed filling Warrior Epithets.")

    FillList(ThiefEpithetsLst1, ThiefEpithets)
    FillList(ThiefEpithetsLst2, ThiefEpithets)
    FillList(ThiefEpithetsLst3, ThiefEpithets)
    lastentry = 0
;Notification("Real Names: Completed filling Thief Epithets.")

    FillList(MageEpithetsLst1, MageEpithets)
    FillList(MageEpithetsLst2, MageEpithets)
    FillList(MageEpithetsLst3, MageEpithets)
    lastentry = 0
;Notification("Real Names: Completed filling Mage Epithets.")

    FillList(GenericEpithetsLst1, GenericEpithets)
    FillList(GenericEpithetsLst2, GenericEpithets)
    FillList(GenericEpithetsLst3, GenericEpithets)
    lastentry = 0
;Notification("Real Names: Completed filling Generic Epithets.")

    FillList(BossSirLst1, BossSir)
    FillList(BossSirLst2, BossSir)
    FillList(BossSirLst3, BossSir)
    lastentry = 0
;Notification("Real Names: Completed filling Boss Titles.")

    FillList(BossNameLst1, BossName)
    FillList(BossNameLst2, BossName)
    FillList(BossNameLst3, BossName)
    lastentry = 0
;Notification("Real Names: Completed filling Boss Epithets.")

Notification("Real Names: Completed filling all arrays.")
Notification("Names will now be assigned automatically.")

    GotoState("UpdateSettings")

EndEvent

EndState


State ResetExceptions

;   This State is invoked by the MCM.

Event OnBeginState()

    Exceptions1 = ",Hrothmund,Aicantar,Ancano,Ancarion,Aringoth,Arondil,Calcelmo,Estormo,Fasendil,Kornalus,Linwe,Lorcalin,Melaran,Muril,Naris,Nelacar,Nerien,Nurelion,Ondolemar,Orthorn,Quaranir,Rulindil,Runil,Sanyon,Tandil,Ulundil,Valmir,Viarmo,Vingalmo,Arivanya,Curwe,Elenwen,Endarie,Faralda,Minorne,Nenya,Niranye,Nirya,Reldith,Taarie,Valie,Deekus,Derkeethus,Madesi,Neetrenaza,Usha,Veezara,Beem-Ja,Brand-Shei,Gajul-Lei,Gulum-Ei,Ilas-Tei,Jaree-Ra,Talen-Jei,Teeba-Ei,Deep-In-His-Cups,Scouts-Many-Marshes,Stands-In-Shallows,Watches-The-Roots,Deeja,Keerava,Shahvee,Wujeeta,From-Deepest-Fathoms,Takes-In-Light,Anoriath,Dervenin,Edorfin,Elrindir,Endrast,Enthir,Faendal,Gadnor,Gelebros,Girduin,Gwilin,Malborn,Niruin,Orchendor,Ronthil,Syndus,Thaer,Ungrien,Valindor,Anuriel,Ardwen,Beleval,Brelas,Galathil,Nimphaneth,Nimriel,Nivenor,Wylandriah,Morven,Adeber,Ainethach,Alain,Amaund,Anton,Arniel,Banning,Belchimac,Belethor,Bendt,Blaise,Braig,Brandish,Cedran,Celann,Clinton,Cosnach,Cynric,Dalan,Degaine,Delacourt,Delvin,Donnel,Dorian,Dryston,Duach,Eltrys,Emmanuel,Enmon,Ennoc,Erj,Esmond,Etienne,Francois,Garvey,Gauldur,Giraud,Glover,Hamelyn,Hathrasil,Herluin,Imedhnain,Jouane,Krag,Louis,Madanach,Malkoran,Maurice,Mercer,Moric,Nepos,Octieve,Odvan,Omluag,Perth,Phinis,Rahd,Rondach,Sam,Sebastian,Staubin,Stromm,Thomas,Tuthul,Tynan,Uraccen,Weylin,Willem,Angeline,Babette,Bothela,Cairine,Carcette,Colette,Daighre,Delphine,Eola,Erith,Fianna,Isabelle,Julienne,Kaie,Laelette,Lisette,Madena,Mena,Michel,Mirabelle,Modhna,Muiri,Nana,Rhiada,Sabine,Senna,Sorine,Sybille,Uaile,Vivienne,Voada,Adril,Ambarys,Arvel,Athis,Aval,Belyn,Bradyn,Casimir,Daynas,Dravin,Drelas,Drevis,Drovas,Erandur,Evul,Falas,Faldrus,Faryl,Feran,Fethis,Galdrus,Garan,Garyn,Geldis,Indaryn,Jiub,Lleril,Malthyr,Malur,Maluril,Malyn,Meden,Mithorpa,Modyn,Naris,Neloth,Orini,Othreloth,Raleth,Ralis,Ravam,Ravyn,Revus,Revyn,Rirns,Romlyn,Saden,Sarthis,Savos,Servos,Sevan,Slitter,Sondas,Talvas,Talvur,Taron,Teldryn,Tolendos,Tythis,Ulves,Ulyn,Valin,Vals,Vanryth,Vendil,Veren,Wyndelius,Aduri,Aphia,Aranea,Avrusa,Bralsa,Brelyna,Cindiri,Dinya,Dravynea,Dreyla,Elvali,Elynea,Gabriella,Idesa,Ildari,Irileth,Jenassa,Karliah,Luaffyn,Marise,Medresi,Merilar,Milore,Mireli,Mirri,Niluva,Selveni,Suvaris,Synda,Tilisu,Varona,Voldsea,Adonato,Adventus,Aerin,Agrius,Alethius,Aquillius,Aventus,Bassianus,Caius,Calixto,Cicero,Constantius,Corpulus,Crescius,Dexion,Dirge,Ennodius,Falx,Florentius,Gaius,Gallus,Gavros,Gian the Fist,Leontius,Lucan,Mallus,Marcurio,Maro,Metilius,Noster,Orthus,Paratus,Pavo,Plautis,Proventus,Quentin,Quintus,Reburrus,Rexus,Rogatus,Salvianus,Samuel,Septimus,Sergius,Severio,Silus,Sorex,Sulla,Tacitus,Taurinus,Telrav,Terek,Titus,Tyranus,Vantus,Varnius,Venarus,Verulus,Xander,Tullius,"

    Exceptions2 = ",Salonia,Adelaisa,Adrianne,Aia,Alessandra,Alexia,Arcadia,Camilla,Carlotta,Constance,Edda,Ganna,Gemma,Gianna,Gilfre,Illia,Indara,Larina,Leonara,Lucia,Mila,Pantea,Raen,Ria,Safia,Silana,Silvia,Sosia,Valga,Vex,Vigdis,Viola,Vittoria,Dro'marash,J'darr,J'datharr,J'Kier,J'zargo,J'zhar,Kharjo,Kesh,Ma'dran,Ma'jhad,Ma'randru-jo,Ma'tasarr,Ma'zaka,M'aiq,Ra'jirr,Ra'kheran,Ri'saad,Vasha,Ahkari,Ahjisi,Atahbah,Khayla,Ra'zhinda,Shavari,Tsavani,Tsrasuna,Zaynabi,Barknar,Engar,Fultheim,Gunjar,Kjeld,Nikulas,Sond,Torkild,Adalvald,Addvar,Addvild,Agmaer,Akar,Alding,Aldis,Alfarinn,Alvor,Andurs,Angrenor,Angvid,Argis,Arnbjorn,Arngeir,Arnskar,Arrald,Asbjorn,Asgeir,Aslfur,Assur,Atar,Aventus,Avulstein,Badnir,Balbus,Baldor,Balgruuf,Balimund,Bassianus,Beirand,Belrand,Benkum,Benor,Bersi,Bjorlam,Bjornolfr,Bolgeir,Bolli,Bolund,Borgny,Borri,Borvir,Bottar,Briehl,Brill,Brond,Brunwulf,Brynjolf,Bulfrek,Calder,Christer,Dagur,Dengeir,Deor,Drahff,Drennen,Eimar,Einarth,Elgrim,Elmus,Embry,Eorlund,Erik,Erikur,Erlendr,Esbern,Falk,Farengar,Farkas,Felldir,Fenrig,Filnjar,Frodnar,Frofnir,Froki,Frorkmar,Frothar,Galmar,Garthar,Garuk,Geimund,Geirlund,Gestur,Gissur,Gjak,Gjalund,Gjuk,Gloth,Golldir,Gonnar,Gorm,Gort,Gralnach,Gregor,Grimvar,Grisvar,Gunding,Gunmar,Gunnar,Guthrum,Hadring,Hadvar,Hafnar,Hajvarr,Hakar,Haknir,Hakon,Halbarn,Haming,Hargar,Harkon,Harlaug,Harrald,Heddic,Heimskr,Heimvar,Helvard,Hemming,Henrik,Heratar,Hern,Hewnon,Hjalfar,Hjornskar,Hjorunn,Hod,Hofgrir,Hogni,Holgeir,Horgeir,Horik,Horm,Hreinn,Hroar,Hrodulf,Hroggar,Hrollod,Hrolmir,Hrongar,Hrothmund,Hunroor,Idolaf,Igmund,Irlof,Irnskar,Istar,Jenssen,Jervar,Jesper,Jod,Jofthor,Jolf,Jon,Jorgen,Joric,Jorleif,Jorn,Jurgen,Kai,Karl,Kibell,Kjar,Kleppr,Klimmek,Knjakr,Knud,Kodlak,Kodrir,Korir,Kottir,Kraldar,Kust,Kuvar,Kvenel,Kyr,Lars,Leifur,Leigelf,Lemkil,Llewellyn,Lod,Lodi,Lodvar,Logrolf,Lokil,Lokir,Lond,Lorenz,Lortheim,Lund,Lygrleid,Lynoit,Majni,Markus,Mathies,Maul,Metilius,Mikael,Miraak,Molgrom,Mralki,Narfi,Nelkir,Niels,Nils,Odar,Odfel,Oengul,Ogmund,Olaf,Olfrid,Ollrod,Onmund,Orgnar,Orthjolf,Oslaf,Pactur,Raerek,Ragnar,Ralof,Ranmir,Rargal,Reves,Rhorlak,Rigel,Rissing,Rogen,Roggi,Roggvir,Rolff,Rorik,Rorlund,Rundi,Rustleif,Sabjorn,Saerlund,Sibbi,Siddgeir,Sifnar,Sigaar,Sigar,Sigurd,Sild,Sinding,Sinmir,Sirgar,Sirkjorg,Skaggi,Skald,Skeggr,Skjor,Skuli,Skulnar,Skulvar,Snilf,Snilling,Snorreid,Sogrlaf,Solaf,Stalf,Steirod,Stenvar,Stig,Storn,Styrr,Sulvar,Svaknir,Sven,Talsgar,Thadgeir,Thaer,Thalin,Thjollod,Thonar,Thongvor,Thonjolf,Thonnir,Thorald,Thorek,Thoring,Thorygg,Thorgar,Thrynn,Tolan,Tolfdir,Torbjorn,Torolf,Torsten,Torvar,High King Torygg,Torygg,Trilf,Tsun,Tulvur,Ulfberth,Ulfgar,Ulfr,Ulfric,Unmid,Vald,Valdar,Valdimar,Valdr,Vanik,Vekel,Verner,Vidgrod,Viding,Vidrald,Vignar,Vilkas,Vilod,Vipir,Virkmund,Volk,Vorstag,Vulwulf,Wilhelm,Wilmuth,Wulf,Wulfgar,Wuunferth,Yngvar,Yrsarald,Eydis,Idgrod,Karita,Liesl,"

    Exceptions3 = ",Abelone,Adisla,Aela,Aeri,Aeta,Agna,Agni,Agnis,Alea,Alfhild,Alva,Alvide,Annekke,Angi,Anise,Anska,Argi,Ari,Asta,Astrid,Beitild,Bergritte,Betrid,Birna,Bjartur,Bodil,Bolfrida,Boti,Breya,Brina,Britte,Bryling,Bujold,Dagny,Danica,Dorthe,Drifa,Edith,Edla,Eirid,Eisa,Elda,Elisif,Erdi,Eris,Evette,Faida,Fanari,Fastred,Finna,Fjori,Fjola,Fjotra,Frabbi,Fralia,Frea,Freir,Frida,Fridrika,Fruki,Fura,Gerda,Gerdur,Gisli,Gormlaith,Grelka,Grelod,Greta,Grete,Gretta,Grosta,Gwendolyn,Haelga,Hafjorg,Hamal,Haran,Hefid,Helgi,Helgird,Herkja,Hermir,Hert,Hestla,Hilde,Hillevi,Hilund,Hjordis,Hoddreid,Hrefna,Hroki,Huki,Hulda,Iddra,Illdi,Inge,Ingjard,Ingrid,Ingun,Iona,Irgnir,Jala,Jora,Jordis,Katla,Katria,Laila,Lami,Lillith,Lisbet,Lydia,Lynly,Margret,Matlara,Maven,Minette,Mjoll,Morwen,Narri,Nilsine,Njada,Nura,Olava,Olda,Olfina,Oriella,Orla,Rakel,Rigel,Rikke,Ruki,Runa,Sapphire,Serana,Sigrid,Silda,Sissel,Sofie,Sonir,Sorli,Susanna,Svana,Svari,Swanhvir,Sylgja,Tekla,Temba,Thaena,Threki,Tilde,Tilma,Tormir,Tova,Una,Uthgerd,Valerica,Vori,Yrsa,Ysolda,Mogrul,Balagog,Bashnag,Borkul,Burguk,Durak,Dushnamub,Gadba,Gat,Ghamorz,Ghorbash,Ghunzul,Grogmar,Grushnag,Gularzob,Kharag,Larak,Lob,Lurbuk,Mahk,Makhel,Malkus,Mauhulakh,Moth,Mul,Mulush,Nagrub,Oglub,Ogol,Olur,Ulag,Umurn,Urag,Yamarz,Yar,Arob,Atub,Bagrak,Batum,Bolar,Bor,Borgakh,Dulug,Garakh,Ghak,Gharol,Ghorza,Gul,Lash,Mogdurz,Murbul,Sharamph,Shel,Shuftharz,Ugor,Urog,Urzoga,Yag,Yatul,Ahtar,Alesan,Amren,Ataf,Avidius,Azzada,Azzadal,Baral,Brenuin,Endon,Ennis,Falion,Fihada,Firir,Haldyn,Isran,Jawanan,Kayd,Kematu,Majid,Mani,Maramal,Namasur,Nazeem,Nazir,Razelan,Shadr,Talib,Adara,Ahlam,Anwen,Atmah,Braith,Eriana,Faleen,Iman,Jayri,Jonna,Kerah,Lu'ah,Niyya,Ramati,Rayya,Rochelle,Saadia,Saffir,Saliah,Salma,Sayma,Seren,Sudi,Tonilia,Umana,Yisra,Zaria,Alduin,Durnehviir,Krosulhah,Kruziikrel,Mirmulnir,Naaslaarum,Nahagliiv,Odahviing,Paarthurnax,Relonikiv,Sahloknir,Sahrotaar,Viinturuth,Voslaarum,Vuljotnaak,Vulthuryol,Velehk,Drascua,Esmerelda,Ettiene,Fallaise,Isobel,Lucerne,Melka,Moira,Petra,Curalmil,Saerek,Halldir,Jyrik,Mikrul,Red Eagle,Sigdis,Gathrik,Hevnoraak,Krosis,Morokei,Nahkriin,Otar,Rahgot,Vokun,Volsung,Ahzdial,Dukaan,Miraak,Vahlok,Zahkriisos,Grok,Vyrthur,Gelebor,Ghost,Anger,Confidence,Self Doubt,Nightingale Sentinel,Captain Wayfinder,Captain Lonely-Gale,"

    ExceptionsINPC = ",Alassea,Among-The-Hist,Arghus,Brakh,Callen,Cassock,Daenlyn,Duraz,Erevan,Fa'nar,Fontaine,Froa,Gorr,Helsig,Hjoromir,Jade,Jilkmar,Lajjan,Larkspur,Lurgok,Meresine,Raynes,Skjarn,Tikrid,Vardath,Vaughn,Vawkes,Ylgyne,Amalee,Anum-La,Dagri'Lon,Qa'Dojo,Rumarin,Valgus,Zora,Ortheim,Amras,Lamki,Rosalind,Jorrvid,Bilbus,Jilkmar,Sixby,Kjen,Sven,Maren,Fig,Whig,Kyra,Jaysen,Mowgrol,Ises,Melea,Lyrik,Jerulith,Isobel,Thrice-Bitten,Fjona,Carmella,Lleyara,Glathriel,S'vashni,Olivia,Eleanor,Mugnor,Vigram,Kianna,Al'Hassan,Ingarte,Arelthor,Juryk,Iorel,Jaspar,Frik,Thorolf,Berg,Viranya,Moris,Sylphiron,Jeerah-Nur,Jolene,Lundvar,Clario,Beatrice,Urzub,Two-Tails,Okapi,Tellevi,Gerrod,Asteria,Yseld,Ynvar,Y'tharil,Mithril,Fironet,Orondil,Norgrof,Morndas,Nythriel,Veralene,K'avald,Yggleif,Edwayne,Salty-Throat,Halcar,Darcy,Gwyvane,Nitter,Gilsi,Orenius,Misha,Fjord,Zarlak,Cassius,Iria,Eldawyn,Eldar,Averna,Sonja,Zajhirra,Yarbrough,Ogdul,Leifan,Rongeir,Shelur,Morviah,Rinori,Wander-Lust,Haakon,Valla,Efram,Azzarian,Geel-Jah,Torveld,Fareloth,Fjona's,Gromash,Iris,Griffith,Morrigan,Tanita,Sadrin,Peragorn,Gnives,Olette,Dirassi,Yushari,Caylene,Amicus,Korrilan,Kjoli,Dar'Rakki,Hagravi,Fuki,Feldi,Vukeera,Batheru,Dravos,Ureni,Dro'bar,Offryn,Helsig,Felena,Vinnius,Flin,Puck,Elenthil,Marigoth,Bodan,Dalum-Ei,The Fair Lady,J'Sharr,Valenor,Ignatius,Arilwaen,Telthar,Hiram,Gabania,Joselyn,Theron,Aldi,Camaron,Alendril,Herran,Utei,Peracles,Blue-Throat,An-Ru,Asgeld,Foromir,Thorngar,Frygg,Hrongar,Hjolm,Falatild,Larsdon,Karras,Signus,Henna,Frenna,Gren,Helcyon,Uldvar,Olfrim,Anja,Brun,Cyrus,Sevrilla,Vardath,Ralothi,Fassius,Amsien,The Captain,Prompt-But-Bashful,Sergius,Gancielo,Lorne,Soren,Deneris,Varyan,Renmar,Deirdre,Hrokkar,Parellius,Nasrin,Krillo,Miros,Grenier,Roggstad,Bettany,Erwen,Sahlene,Godrod,Devran,Vartheim,Pelgrut,Benild,Florentus,Bette,Frodmar,Renni,Gormleif,Grotta,Ulrund,Lattimore,Merissa,Julius,Primei,Ester,Carah,Fenrald,Marla,Ignar,Valencia,Horker,Wolf,Skeever,Sweetroll,Ange,Adius,Aera,Afanna,Alammu,Alavva,Alina,Almo,Ambric,Ananril,Androlius,Arentius,Arenus,Aretino,Armion,Arnoth,Arilus,Astor,Atienne,Atul,Aubert,Aurelia,Avar,Avring,Bathes-In-Murky-Waters,Balius,Bentior,Benuni,Bjarni,Bolar,Botram,Brecius,Brenna,Byretus,Caius,Cadius,Cedus,Centos,Cerendil,Cornelius,Corvon,Crecius,Crextus,Crouches-In-Crevices,Dar'seel,Dar,taqto,Dar'zorben,Desilus,Drusus,Edwarin,Elintius,Encarnus,Eranir,Ereia,Erlus,Fadel,Falegus,Faleria,Famalius,Farsvar,Galarynn,Garceus,Garridan,Gavros,Gort,Gracius,Grecus,gro-Morrg,Grushnag,Gryfard,Gurnjar,Harald,Harsvar,Harthir,Hulgard,Hun-Wei,Ila,Ionalla,Iron-Heart,Ja'malgo,Jiriium,Jucani,Kalfimbria,Leo,Lone-Stead,Malumea,Marc,Marcin,Marcius,Mathias,Mirich,Neldam,Neremus,Nurenor,Odrevan,Olaf,Olgvir,Oreej-Tai,Ossi,Ovacca,Pinter,Precilius,Quinus,Raaslan,Ravil,Razzada,Reln-Tei,Reman,Renod,Rentin,Rignar,Rilja,Roland,Sellus,Senhilde,Serulius,Snipes-Many-Innocents,Servus,Silver-Plow,Simund,Sinja,Snorre,Stenar,Surilaron,Svenja,Tedral,Telenis,Terinius,Thorina,Ulchevan,Ulgruff,Ungveir,Urma,Vilenoldil,Yzzira,Zurikki,"

    PlayerExceptions = ","

    if sAddedExceptions != ""
        PlayerExceptions = sAddedExceptions
        sAddedExceptions = ""
    endif

    GotoState("")

EndEvent

EndState


State UpdateLists

;   What this State accomplishes is that when the player makes
;   changes to the Lists in the MCM, this state gets called to
;   have the script pull the new data from the arrays and rewrite
;   all of the strings (I didn't want to bother scripting a
;   complicated list detection logic-loop).
;
;   This State, together with the initial filling of the arrays
;   and the MCM UI is what makes this mod as customizable as it is.

Event OnBeginState()

    MessageBox("REAL NAMES\n\nChanges to arrays detected: rewriting strings ...\n\nThe MCM may not function properly during this process.\n\nPlease wait.")

    GenericNameSearchTerms = "" + AssembleString(GenericNameSearchTermsLst1) + AssembleString(GenericNameSearchTermsLst2) + AssembleString(GenericNameSearchTermsLst3)

    GenericBossSearchTerms = "" + AssembleString(GenericBossSearchTermsLst1) + AssembleString(GenericBossSearchTermsLst2) + AssembleString(GenericBossSearchTermsLst3)

    BosmerMaleFirstNames = "" + AssembleString(BosmerMaleFirstNamesLst1) + AssembleString(BosmerMaleFirstNamesLst2) + AssembleString(BosmerMaleFirstNamesLst3)

    BosmerFemaleFirstNames = "" + AssembleString(BosmerFemaleFirstNamesLst1) + AssembleString(BosmerFemaleFirstNamesLst2) + AssembleString(BosmerFemaleFirstNamesLst3)

    BosmerFamilyNames = "" + AssembleString(BosmerFamilyNamesLst1) + AssembleString(BosmerFamilyNamesLst2) + AssembleString(BosmerFamilyNamesLst3)

    BretonMaleFirstNames = "" + AssembleString(BretonMaleFirstNamesLst1) + AssembleString(BretonMaleFirstNamesLst2) + AssembleString(BretonMaleFirstNamesLst3)

    BretonFemaleFirstNames = "" + AssembleString(BretonFemaleFirstNamesLst1) + AssembleString(BretonFemaleFirstNamesLst2) + AssembleString(BretonFemaleFirstNamesLst3)

    BretonFamilyNames = "" + AssembleString(BretonFamilyNamesLst1) + AssembleString(BretonFamilyNamesLst2) + AssembleString(BretonFamilyNamesLst3)

    DunmerMaleFirstNames = "" + AssembleString(DunmerMaleFirstNamesLst1) + AssembleString(DunmerMaleFirstNamesLst2) + AssembleString(DunmerMaleFirstNamesLst3)

    DunmerFemaleFirstNames = "" + AssembleString(DunmerFemaleFirstNamesLst1) + AssembleString(DunmerFemaleFirstNamesLst2) + AssembleString(DunmerFemaleFirstNamesLst3)

    DunmerFamilyNames = "" + AssembleString(DunmerFamilyNamesLst1) + AssembleString(DunmerFamilyNamesLst2) + AssembleString(DunmerFamilyNamesLst3)

    AshlanderMaleFirstNames = "" + AssembleString(AshlanderMaleFirstNamesLst1) + AssembleString(AshlanderMaleFirstNamesLst2) + AssembleString(AshlanderMaleFirstNamesLst3)

    AshlanderFemaleFirstNames = "" + AssembleString(AshlanderFemaleFirstNamesLst1) + AssembleString(AshlanderFemaleFirstNamesLst2) + AssembleString(AshlanderFemaleFirstNamesLst3)

    AshlanderFamilyNames = "" + AssembleString(AshlanderFamilyNamesLst1) + AssembleString(AshlanderFamilyNamesLst2) + AssembleString(AshlanderFamilyNamesLst3)

    ArgonianMaleFirstNames = "" + AssembleString(ArgonianMaleFirstNamesLst1) + AssembleString(ArgonianMaleFirstNamesLst2) + AssembleString(ArgonianMaleFirstNamesLst3)

    ArgonianFemaleFirstNames = "" + AssembleString(ArgonianFemaleFirstNamesLst1) + AssembleString(ArgonianFemaleFirstNamesLst2) + AssembleString(ArgonianFemaleFirstNamesLst3)

    ArgonianFamilyNames = "" + AssembleString(ArgonianFamilyNamesLst1) + AssembleString(ArgonianFamilyNamesLst2) + AssembleString(ArgonianFamilyNamesLst3)

    AltmerMaleFirstNames = "" + AssembleString(AltmerMaleFirstNamesLst1) + AssembleString(AltmerMaleFirstNamesLst2) + AssembleString(AltmerMaleFirstNamesLst3)

    AltmerFemaleFirstNames = "" + AssembleString(AltmerFemaleFirstNamesLst1) + AssembleString(AltmerFemaleFirstNamesLst2) + AssembleString(AltmerFemaleFirstNamesLst3)

    AltmerFamilyNames = "" + AssembleString(AltmerFamilyNamesLst1) + AssembleString(AltmerFamilyNamesLst2) + AssembleString(AltmerFamilyNamesLst3)

    RedguardMaleFirstNames = "" + AssembleString(RedguardMaleFirstNamesLst1) + AssembleString(RedguardMaleFirstNamesLst2) + AssembleString(RedguardMaleFirstNamesLst3)

    RedguardFemaleFirstNames = "" + AssembleString(RedguardFemaleFirstNamesLst1) + AssembleString(RedguardFemaleFirstNamesLst2) + AssembleString(RedguardFemaleFirstNamesLst3)

    RedguardFamilyNames = "" + AssembleString(RedguardFamilyNamesLst1) + AssembleString(RedguardFamilyNamesLst2) + AssembleString(RedguardFamilyNamesLst3)

    OrcMaleFirstNames = "" + AssembleString(OrcMaleFirstNamesLst1) + AssembleString(OrcMaleFirstNamesLst2) + AssembleString(OrcMaleFirstNamesLst3)

    OrcFemaleFirstNames = "" + AssembleString(OrcFemaleFirstNamesLst1) + AssembleString(OrcFemaleFirstNamesLst2) + AssembleString(OrcFemaleFirstNamesLst3)

    OrcFamilyNames = "" + AssembleString(OrcFamilyNamesLst1) + AssembleString(OrcFamilyNamesLst2) + AssembleString(OrcFamilyNamesLst3)

    NordMaleFirstNames = "" + AssembleString(NordMaleFirstNamesLst1) + AssembleString(NordMaleFirstNamesLst2) + AssembleString(NordMaleFirstNamesLst3)

    NordFemaleFirstNames = "" + AssembleString(NordFemaleFirstNamesLst1) + AssembleString(NordFemaleFirstNamesLst2) + AssembleString(NordFemaleFirstNamesLst3)

    NordFamilyNames = "" + AssembleString(NordFamilyNamesLst1) + AssembleString(NordFamilyNamesLst2) + AssembleString(NordFamilyNamesLst3)

    KhajiitMaleFirstNames = "" + AssembleString(KhajiitMaleFirstNamesLst1) + AssembleString(KhajiitMaleFirstNamesLst2) + AssembleString(KhajiitMaleFirstNamesLst3)

    KhajiitMalePrefices = "" + AssembleString(KhajiitMalePreficesLst1) + AssembleString(KhajiitMalePreficesLst2) + AssembleString(KhajiitMalePreficesLst3)

    KhajiitMaleSuffices = "" + AssembleString(KhajiitMaleSufficesLst1) + AssembleString(KhajiitMaleSufficesLst2) + AssembleString(KhajiitMaleSufficesLst3)

    KhajiitFemaleFirstNames = "" + AssembleString(KhajiitFemaleFirstNamesLst1) + AssembleString(KhajiitFemaleFirstNamesLst2) + AssembleString(KhajiitFemaleFirstNamesLst3)

    KhajiitFemalePrefices = "" + AssembleString(KhajiitFemalePreficesLst1) + AssembleString(KhajiitFemalePreficesLst2) + AssembleString(KhajiitFemalePreficesLst3)

    KhajiitFemaleSuffices = "" + AssembleString(KhajiitFemaleSufficesLst1) + AssembleString(KhajiitFemaleSufficesLst2) + AssembleString(KhajiitFemaleSufficesLst3)

    KhajiitFamilyNames = "" + AssembleString(KhajiitFamilyNamesLst1) + AssembleString(KhajiitFamilyNamesLst2) + AssembleString(KhajiitFamilyNamesLst3)

    ImperialMaleFirstNames = "" + AssembleString(ImperialMaleFirstNamesLst1) + AssembleString(ImperialMaleFirstNamesLst2) + AssembleString(ImperialMaleFirstNamesLst3)

    ImperialFemaleFirstNames = "" + AssembleString(ImperialFemaleFirstNamesLst1) + AssembleString(ImperialFemaleFirstNamesLst2) + AssembleString(ImperialFemaleFirstNamesLst3)

    ImperialFamilyNames = "" + AssembleString(ImperialFamilyNamesLst1) + AssembleString(ImperialFamilyNamesLst2) + AssembleString(ImperialFamilyNamesLst3)

    DremoraMaleFirstNames = "" + AssembleString(DremoraMaleFirstNamesLst1) + AssembleString(DremoraMaleFirstNamesLst2) + AssembleString(DremoraMaleFirstNamesLst3)

    DremoraFemaleFirstNames = "" + AssembleString(DremoraFemaleFirstNamesLst1) + AssembleString(DremoraFemaleFirstNamesLst2) + AssembleString(DremoraFemaleFirstNamesLst3)

    DremoraFamilyNames = "" + AssembleString(DremoraFamilyNamesLst1) + AssembleString(DremoraFamilyNamesLst2) + AssembleString(DremoraFamilyNamesLst3)

    GiantNames = "" + AssembleString(GiantNamesLst1) + AssembleString(GiantNamesLst2) + AssembleString(GiantNamesLst3)

    DragonNames = "" + AssembleString(DragonNamesLst1) + AssembleString(DragonNamesLst2) + AssembleString(DragonNamesLst3)

    DragonOneSyllable = "" + AssembleString(DragonOneSyllableLst1) + AssembleString(DragonOneSyllableLst2) + AssembleString(DragonOneSyllableLst3)

    DragonTwoSyllable = "" + AssembleString(DragonTwoSyllableLst1) + AssembleString(DragonTwoSyllableLst2) + AssembleString(DragonTwoSyllableLst3)

    WarriorTitles = "" + AssembleString(WarriorTitlesLst1) + AssembleString(WarriorTitlesLst2) + AssembleString(WarriorTitlesLst3)

    ThiefTitles = "" + AssembleString(ThiefTitlesLst1) + AssembleString(ThiefTitlesLst2) + AssembleString(ThiefTitlesLst3)

    MageTitles = "" + AssembleString(MageTitlesLst1) + AssembleString(MageTitlesLst2) + AssembleString(MageTitlesLst3)

    GenericTitles = "" + AssembleString(GenericTitlesLst1) + AssembleString(GenericTitlesLst2) + AssembleString(GenericTitlesLst3)

    WarriorEpithets = "" + AssembleString(WarriorEpithetsLst1) + AssembleString(WarriorEpithetsLst2) + AssembleString(WarriorEpithetsLst3)

    ThiefEpithets = "" + AssembleString(ThiefEpithetsLst1) + AssembleString(ThiefEpithetsLst2) + AssembleString(ThiefEpithetsLst3)

    MageEpithets = "" + AssembleString(MageEpithetsLst1) + AssembleString(MageEpithetsLst2) + AssembleString(MageEpithetsLst3)

    GenericEpithets = "" + AssembleString(GenericEpithetsLst1) + AssembleString(GenericEpithetsLst2) + AssembleString(GenericEpithetsLst3)

    BossSir = "" + AssembleString(BossSirLst1) + AssembleString(BossSirLst2) + AssembleString(BossSirLst3)

    BossName = "" + AssembleString(BossNameLst1) + AssembleString(BossNameLst2) + AssembleString(BossNameLst3)

    GotoState("UpdateSettings")

    MessageBox("REAL NAMES\n\nStrings have been rewritten.\nResuming Normal processing.")

EndEvent

EndState


;/  CH02.1 -=FUNCTIONS=-    /;

function FillList(string[] asList, string asSearchString)

;   This function gets called several times during initialization
;   in order to fill the arrays which give the MCM access to the
;   Name Strings (albeit indirectly).

    int index = 0
    while asList[index] != "END OF LIST"
        int iOffset = Find(asSearchString, ",", lastentry) + 1
        if iOffset > 0
            string entry = Substring(asSearchString, iOffset, Find(asSearchString, ",", iOffset) - iOffset)
            asList[index] = entry
            lastentry += GetLength(entry) + 1
        endif
        index += 1
    endwhile
endfunction

function UpdateList(string[] asList, string asSearchString, string asUpdateString)

;    This function gets called during version updates which include
;    additions to List Terms.

    string entry = ","
    int index = 0
    bool bEndOfString = false
    while asList[index] != "END OF LIST"
        while Find(asSearchString, entry) >= 0 && !bEndOfString
            int iOffset = Find(asUpdateString, ",", lastentry) + 1
            if iOffset > 0
                entry = Substring(asUpdateString, iOffset, Find(asUpdateString, ",", iOffset) - iOffset)
                lastentry += GetLength(entry) + 1
            else
                bEndOfString = true
                if Find(asSearchString, entry) >= 0
                    entry = ","
                endif
            endif
        endwhile
	if asList[index] == "" && entry != ","
	    asList[index] = entry
	    entry = ","
        endif
        index += 1
    endwhile

endfunction

function CorrectList(string[] asList, string asSearchString, string asUpdateString)

;    This function gets called during version updates which include
;    corrections to List Terms.

    string entry = ","
    int index = 0
    bool bEndOfString = false
    while asList[index] != "END OF LIST"
        while asList[Index] != "" && entry != asList[Index] && !bEndOfString
            int iOffset = Find(asUpdateString, ",", lastentry) + 1
            if iOffset > 0
                entry = Substring(asUpdateString, iOffset, Find(asUpdateString, ",", iOffset) - iOffset)
                lastentry += GetLength(entry) + 1
            else
                bEndOfString = true
                if Find(asSearchString, entry) == -1
                    entry = ","
                endif
            endif
        endwhile
	if asList[index] == entry
	    asList[index] = entry
	    entry = ","
            lastentry = 0
        endif
        index += 1
    endwhile
    lastentry = 0

endfunction

string function AssembleString(string[] asList)

;   This function gets called any time the arrays are edited either
;   via the MCM or by a Version Update.

    int index = 0
    string sAssembly
    while asList[index] != "END OF LIST"
        if asList[index] != ""
            sAssembly += "," + asList[index]
        endif
        index += 1
    endwhile
    return sAssembly

endfunction